@@ -400,33 +400,15 @@ extern void  reply_text (struct batch_request *, int code, char *text);
 extern int   reply_jobid (struct batch_request *, char *, int);
 extern void  reply_free (struct batch_reply *);
 extern int   authenticate_user (struct batch_request *, struct credential *, char **);
-extern void  dispatch_request (int, struct batch_request *);
 extern void  free_br (struct batch_request *);
 extern int   isode_request_read (int, struct batch_request *);
 
 #ifndef PBS_MOM
-extern void *req_authenuser (void *req);
-extern void *req_altauthenuser (void *req);
 extern void  req_connect (struct batch_request *req);
-extern void *req_locatejob (void *req);
-extern void *req_manager (void *req);
-extern void *req_movejob (void *req);
-void        *req_register (void *req);
-void        *req_registerarray (void *req);
-extern void *req_releasearray (void *req);
-extern void *req_releasejob (void *req);
-void        *req_rescq (void *req);
-void        *req_rerunjob (void *req);
-extern void *req_runjob (void *req);
-void        *req_selectjobs (void *req);
-extern void *req_stat_que (void *req);
-extern void *req_stat_svr (void *req);
 /* DIAGTODO: declr req_stat_diag() */
 extern void  req_trackjob (struct batch_request *req);
 extern void *req_gpuctrl (void *req);
-extern void *req_stat_job (void *req);
 #else
-extern void  req_stat_job (struct batch_request *req);
 extern void  req_cpyfile (struct batch_request *req);
 extern void  req_delfile (struct batch_request *req);
 extern void req_returnfiles (struct batch_request *req);
@@ -439,39 +421,39 @@ int relay_to_mom (job **, struct batch_request *, void (*func)());
 /* PBS Batch Request Decode/Encode routines */
 
 /* udp decode and encode routines */
-extern int decode_DIS_Authen (int socket, struct batch_request *);
-extern int decode_DIS_AltAuthen (int socket, struct batch_request *);
-extern int decode_DIS_CopyFiles (int socket, struct batch_request *);
-extern int decode_DIS_JobCred (int socket, struct batch_request *);
-extern int decode_DIS_JobFile (int socket, struct batch_request *);
-extern int decode_DIS_JobObit (int socket, struct batch_request *);
-extern int decode_DIS_Manage (int socket, struct batch_request *);
-extern int decode_DIS_MoveJob (int socket, struct batch_request *);
-extern int decode_DIS_MessageJob (int socket, struct batch_request *);
-extern int decode_DIS_QueueJob (int socket, struct batch_request *);
-extern int decode_DIS_Register (int socket, struct batch_request *);
-extern int decode_DIS_ReturnFiles (int socket, struct batch_request *);
-extern int decode_DIS_ReqExtend (int socket, struct batch_request *);
-extern int decode_DIS_ReqHdr (int socket, struct batch_request *, int *tp, int *pv);
-extern int decode_DIS_Rescl (int socket, struct batch_request *);
-extern int decode_DIS_Rescq (int socket, struct batch_request *);
-extern int decode_DIS_RunJob (int socket, struct batch_request *);
-extern int decode_DIS_ShutDown (int socket, struct batch_request *);
-extern int decode_DIS_SignalJob (int socket, struct batch_request *);
-extern int decode_DIS_Status (int socket, struct batch_request *);
-extern int decode_DIS_TrackJob (int socket, struct batch_request *);
-extern int decode_DIS_replySvr (int socket, struct batch_reply *);
-extern int decode_DIS_svrattrl (int socket, tlist_head *);
-extern int decode_DIS_GpuCtrl (int socket, struct batch_request *);
+extern int decode_DIS_Authen (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_AltAuthen (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_CopyFiles (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_JobCred (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_JobFile (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_JobObit (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_Manage (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_MoveJob (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_MessageJob (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_QueueJob (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_Register (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_ReturnFiles (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_ReqExtend (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_ReqHdr (struct tcp_chan *chan, struct batch_request *, int *tp, int *pv);
+extern int decode_DIS_Rescl (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_Rescq (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_RunJob (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_ShutDown (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_SignalJob (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_Status (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_TrackJob (struct tcp_chan *chan, struct batch_request *);
+extern int decode_DIS_replySvr (struct tcp_chan *chan, struct batch_reply *);
+extern int decode_DIS_svrattrl (struct tcp_chan *chan, tlist_head *);
+extern int decode_DIS_GpuCtrl (struct tcp_chan *chan, struct batch_request *);
 
-extern int encode_DIS_CopyFiles (int socket, struct batch_request *);
-extern int encode_DIS_JobObit (int socket, struct batch_request *);
-extern int encode_DIS_Register (int socket, struct batch_request *);
-extern int encode_DIS_ReturnFiles (int socket, struct batch_request *);
-extern int encode_DIS_TrackJob (int socket, struct batch_request *);
-extern int encode_DIS_reply (int socket, struct batch_reply *);
-extern int encode_DIS_svrattrl (int socket, svrattrl *);
+extern int encode_DIS_CopyFiles (struct tcp_chan *chan, struct batch_request *);
+extern int encode_DIS_JobObit (struct tcp_chan *chan, struct batch_request *);
+extern int encode_DIS_Register (struct tcp_chan *chan, struct batch_request *);
+extern int encode_DIS_ReturnFiles (struct tcp_chan *chan, struct batch_request *);
+extern int encode_DIS_TrackJob (struct tcp_chan *chan, struct batch_request *);
+extern int encode_DIS_reply (struct tcp_chan *chan, struct batch_reply *);
+extern int encode_DIS_svrattrl (struct tcp_chan *chan, svrattrl *);
 
-extern int dis_request_read (int socket, struct batch_request *);
-extern int dis_reply_read (int socket, struct batch_reply *);
+extern int dis_request_read (struct tcp_chan *chan, struct batch_request *);
+extern int dis_reply_read (struct tcp_chan *chan, struct batch_reply *);
 #endif /* BATCH_REQUEST_H */
