@@ -78,7 +78,7 @@
 */
 
 /*
- * @(#) $Id: list_link.h 3923 2010-07-23 21:16:17Z dbeer $
+ * @(#) $Id: list_link.h 4320 2010-12-01 20:09:35Z dbeer $
  */
 
 /*
@@ -92,6 +92,9 @@
 
 #ifndef LIST_LINK_H
 #define LIST_LINK_H 1
+
+#include <sys/types.h>
+
 /* list entry list sub-structure */
 
 typedef struct list_link
@@ -105,6 +108,16 @@ typedef struct list_link
 
 typedef list_link tlist_head;
 
+/*
+** Simple struct to construct linked PID lists
+*/
+
+typedef struct pidl
+  {
+  pid_t        pid;
+  struct pidl *next;
+  } pidl;
+
 
 /* macros to clear list head or link */
 
@@ -131,6 +144,7 @@ extern void swap_link(list_link *, list_link *);
 extern int  is_linked(list_link *head, list_link *old);
 extern int  is_link_initialized(list_link *);
 extern void list_move(tlist_head *old, tlist_head *new);
+extern void free_pidlist(struct pidl *pl);
 
 #ifndef NDEBUG
 extern void *get_next(list_link, char *file, int line);
