@@ -196,13 +196,14 @@ int  client_to_svr(pbs_net_t, unsigned int, int, char *);
 void close_conn(int,int);
 pbs_net_t get_connectaddr(int,int);
 int  get_connecthost(int sock, char *, int);
+void set_localhost_name(char *, size_t);
 pbs_net_t get_hostaddr(int *, char *);
 int  get_fullhostname(char *, char *, int, char *);
 unsigned int  get_svrport(char *, char *, unsigned int);
 int  init_network(unsigned int, void *(*readfunc)(void *));
 void net_close(int);
 int  wait_request(time_t waittime, long *);
-void net_add_close_func(int, void(*func)(int), int);
+void net_add_close_func(int, void(*func)(int));
 int get_max_num_descriptors(void);
 int get_fdset_size(void);
 char * netaddr_pbs_net_t(pbs_net_t);
@@ -220,6 +221,7 @@ struct connection
   void *(*cn_func)(void *);  /* read function when data rdy */
   void (*cn_oncl)(int);  /* func to call on close */
   pthread_mutex_t *cn_mutex;
+  int cn_stay_open; /* Set to TRUE when the connection needs to remain open */
   };
 
 struct netcounter
