@@ -231,8 +231,26 @@ PbsErrClient(PBSE_TMPNONAME, "tmpdir cannot be named for job")
 PbsErrClient(PBSE_CANTOPENSOCKET, "cannot open demux sockets")
 PbsErrClient(PBSE_CANTCONTACTSISTERS, "cannot send join job to all sisters")
 PbsErrClient(PBSE_CANTCREATETMPDIR, "cannot create tmpdir for job")
-PbsErrClient(PBSE_BADMOMSTATE, "Mom is down, cannot run job")
 /* 15110 */
+PbsErrClient(PBSE_BADMOMSTATE, "Mom is down, cannot run job")
+PbsErrClient(PBSE_SOCKET_INFORMATION, "socket information is not accessible")
+PbsErrClient(PBSE_SOCKET_DATA, "data on socket does not process correctly")
+PbsErrClient(PBSE_CLIENT_INVALID, "client is not allowed/trusted")
+PbsErrClient(PBSE_PREMATURE_EOF, "Premature End of File")
+PbsErrClient(PBSE_CAN_NOT_SAVE_FILE, "Error saving file")
+PbsErrClient(PBSE_CAN_NOT_OPEN_FILE, "Error opening file")
+PbsErrClient(PBSE_CAN_NOT_WRITE_FILE, "Error writing file")
+PbsErrClient(PBSE_JOB_FILE_CORRUPT, "Job file corrupt")
+PbsErrClient(PBSE_JOB_RERUN, "Job can not be rerun")
+/* 15120 */
+PbsErrClient(PBSE_CONNECT, "Can not establish connection")
+PbsErrClient(PBSE_JOBWORKDELAY, "Job function must be temporarily delayed")
+PbsErrClient(PBSE_BAD_PARAMETER, "Parameter of function was invalid")
+PbsErrClient(PBSE_CONTINUE, "Continue processing on job. (Not an error)")
+PbsErrClient(PBSE_JOBSUBSTATE, "Current sub state does not allow trasaction.")
+PbsErrClient(PBSE_CAN_NOT_MOVE_FILE, "Error moving file")
+
+
 
 /* pbs client errors ceiling (max_client_err + 1) */
 PbsErrClient(PBSE_CEILING,           (char*)0)
