@@ -124,6 +124,7 @@
 /* Private Functions */
 
 static int hacl_match(const char *can, const char *master);
+static int uhacl_match(const char *can, const char *master);
 static int user_match(const char *can, const char *master);
 static int gid_match(const char *can, const char *master);
 static int host_order(char *old, char *new);
@@ -177,6 +178,108 @@ int set_hostacl(
 
 
 
+/* this function returns a function pointer */
+int (*get_my_match_func(
+
+  int type))(const char *, const char *)
+
+  {
+  int (*match_func)(const char *, const char *);
+
+  switch (type)
+    {
+
+    case ACL_Host:
+
+      match_func = hacl_match;
+      break;
+
+    case ACL_User:
+
+      match_func = user_match;
+      break;
+
+    case ACL_Gid:
+
+      match_func = gid_match;
+      break;
+
+    case ACL_User_Host:
+      match_func = uhacl_match;
+      break;
+
+    case ACL_Group:
+
+    default:
+
+      match_func = (int (*)())strcmp;
+      break;
+    }
+
+  return(match_func);
+  } /* END get_my_match_func() */
+
+
+
+
+int acl_check_my_array_string(
+
+  struct array_strings *pas,
+  char                 *name,
+  int                   type)
+
+  {
+#ifdef HOST_ACL_DEFAULT_ALL
+  int   default_rtn = 1;
+#else /* HOST_ACL_DEFAULT_ALL */
+  int   default_rtn = 0;
+#endif /* HOST_ACL_DEFAULT_ALL */
+  int (*match_func)(const char *, const char *);
+  int   i;
+  char *pstr;
+ 
+  match_func = get_my_match_func(type);
+
+  for (i = 0; i < pas->as_usedptr;i++)
+    {
+    pstr = pas->as_string[i];
+
+    if ((*pstr == '+') || (*pstr == '-'))
+      {
+      if (*(pstr + 1) == '\0') /* "+" or "-" sets default */
+        {
+        if (*pstr == '+')
+          default_rtn = 1;  /* allow by default */
+        else
+          default_rtn = 0;  /* deny by default */
+        }
+
+      pstr++;  /* skip over +/- if present */
+      }
+
+    if ((name != NULL) && 
+        (!match_func(name, pstr)))
+      {
+      /* acl matches */
+
+      if (*pas->as_string[i] == '-')
+        {
+        /* deny */
+
+        return(0);
+        }
+
+      /* allow */
+
+      return(1);
+      }
+    }    /* END for (i) */
+
+  return(default_rtn);
+  } /* END acl_check_my_array_string() */
+
+
+
 
 
 /*
@@ -198,46 +301,9 @@ int acl_check(
   int      type)
 
   {
-  int        i;
-#ifdef HOST_ACL_DEFAULT_ALL
-  int        default_rtn = 1;
-#else /* HOST_ACL_DEFAULT_ALL */
-  int        default_rtn = 0;
-#endif /* HOST_ACL_DEFAULT_ALL */
-
   struct array_strings *pas;
-  char       *pstr;
-  int (*match_func)(const char *, const char *);
-
   extern char server_host[];
 
-  switch (type)
-    {
-
-    case ACL_Host:
-
-      match_func = hacl_match;
-      break;
-
-    case ACL_User:
-
-      match_func = user_match;
-      break;
-
-    case ACL_Gid:
-
-      match_func = gid_match;
-      break;
-
-    case ACL_Group:
-
-    default:
-
-      match_func = (int (*)())strcmp;
-
-      break;
-    }
-
   if (!(pattr->at_flags & ATR_VFLAG_SET) ||
       ((pas = pattr->at_val.at_arst) == NULL) || (pas->as_usedptr == 0))
     {
@@ -261,41 +327,7 @@ int acl_check(
 #endif
     }
 
-  for (i = 0; i < pas->as_usedptr;i++)
-    {
-    pstr = pas->as_string[i];
-
-    if ((*pstr == '+') || (*pstr == '-'))
-      {
-      if (*(pstr + 1) == '\0') /* "+" or "-" sets default */
-        {
-        if (*pstr == '+')
-          default_rtn = 1;  /* allow by default */
-        else
-          default_rtn = 0;  /* deny by default */
-        }
-
-      pstr++;  /* skip over +/- if present */
-      }
-
-    if ((name != NULL) && !match_func(name, pstr))
-      {
-      /* acl matches */
-
-      if (*pas->as_string[i] == '-')
-        {
-        /* deny */
-
-        return(0);
-        }
-
-      /* allow */
-
-      return(1);
-      }
-    }    /* END for (i) */
-
-  return(default_rtn);
+  return(acl_check_my_array_string(pas, name, type));
   }  /* END acl_check() */
 
 
@@ -384,8 +416,7 @@ static int set_allacl(struct attribute *attr, struct attribute *new, enum batch_
     /* no array_strings control structure, make one */
 
     i = newpas->as_npointers;
-    pas = (struct array_strings *)malloc((i - 1) * sizeof(char *) +
-                                         sizeof(struct array_strings));
+    pas = (struct array_strings *)calloc(1, (i - 1) * sizeof(char *) + sizeof(struct array_strings));
 
     if (!pas)
       return (PBSE_SYSTEM);
@@ -437,7 +468,7 @@ static int set_allacl(struct attribute *attr, struct attribute *new, enum batch_
 
         nsize += nsize / 2;  /* alloc extra space */
 
-        if (!(pas->as_buf = malloc((size_t)nsize)))
+        if (!(pas->as_buf = calloc(1, (size_t)nsize)))
           {
           pas->as_bufsize = 0;
           return (PBSE_SYSTEM);
@@ -475,14 +506,14 @@ static int set_allacl(struct attribute *attr, struct attribute *new, enum batch_
 
         need = pas->as_bufsize + 2 * nsize;  /* alloc new buf */
 
-        if (pas->as_buf)
-          pc = realloc(pas->as_buf, (size_t)need);
-        else
-          pc = malloc((size_t)need);
+        pc = calloc(1, need);
 
         if (pc == (char *)0)
           return (PBSE_SYSTEM);
 
+        if (pas->as_buf)
+          strcat(pc, pas->as_buf);
+
         offset = pc - pas->as_buf;
 
         pas->as_buf   = pc;
@@ -922,6 +953,154 @@ static int hacl_match(
   }
 
 
+int match_strings(const char *can, const char *master)
+  {
+
+    const char *can_end;
+    const char *master_end;
+
+    can_end    = can + strlen(can);
+    master_end = master + strlen(master);
+
+  /* see if the strings match up */
+  while ((can < can_end) && 
+         (master < master_end) && 
+         (*can != 0) && (*master != 0))
+    {
+    switch (*master)
+      {
+      case '[':
+
+        if (acl_check_range(&master,&can) != 0)
+          return(1);
+
+        break;
+
+      case '*':
+
+        if (acl_wildcard_check(&master,&can,master_end,can_end) != 0)
+          return(1);
+
+        break;
+
+      default:
+        if (tolower(*can) != tolower(*master))
+          return(1);
+
+        /* only advance pointers here, other functions advance properly */
+        can++;
+        master++;
+
+        break;
+      }
+    }
+
+  /* make sure both strings have terminated or the master has a wildcard */
+  if (can < can_end)
+    return(1);
+  else if (master < master_end)
+    {
+    if ((*master != '*') ||
+        (master+1 < master_end))
+      return(1);
+    }
+
+  return(0);
+  }
+
+/*
+ * user/host acl order match - match two strings from the tail end first
+ * 
+ * uhacl_match will accept names is a <user>@<host> format. The @ character
+ * is used as a delimeter to separate the user from the host. A string match
+ * is then done on each half of the incoming string. If both halves of can
+ * match both halves of master a 0 is returned otherwise a 1.
+ *
+ *
+ * Master string (2nd parameter) is an entry from a host acl.  It may have a
+ * leading + or - which is ignored.  It may also have an '*' as a leading
+ * name segment to be a wild card - match anything.
+ *
+ * Strings match if identical, or if match up to leading '*' on master which
+ * like a wild card, matches any prefix string on canidate domain name
+ *
+ * Returns 0 if strings match,  1 if not   - to match strcmp()
+ */
+
+static int uhacl_match(
+
+  const char *can,
+  const char *master)
+
+  {
+  const char *pchost;
+  const char *pmhost;
+  const char *pcuser;
+  const char *pmuser;
+  const char *ptr;
+
+  if ((can == NULL) || (!strcmp(can, "LOCAL")))
+    {
+    return(0);
+    }
+
+  /* separate the user and the host portions */
+  ptr = strchr(can, '@');
+  if(ptr == NULL)
+    {
+    /* we are expecting the format <user>@<host>. No @ was found */
+    return(1);
+    }
+
+  pchost = ptr;
+  pchost++;
+  if(*pchost == 0)
+    {
+    /* no host given. Fail*/
+    return(1);
+    }
+
+  /* set pcuser to can and then null terminate user by
+     setting ptr to 0 */
+  pcuser = can;
+  ptr = NULL;
+
+  /* next separate the user and host for the master */
+  ptr = strchr(master, '@');
+  if(ptr == NULL)
+    {
+    /* we are expecting the format <user>@<host>. No @ was found */
+    return(1);
+    }
+
+  pmhost = ptr;
+  pmhost++;
+  if(*pmhost == 0)
+    {
+    /* no host given. Fail */
+    return(1);
+    }
+  
+  pmuser = master;
+  ptr = NULL;
+
+  if(match_strings(pcuser, pmuser))
+    {
+    return(1);
+    }
+
+  if(match_strings(pchost, pmhost))
+    {
+      return(1);
+    }
+
+
+
+  /* if we haven't failed by now, we're golden */
+  return (0);
+  }
+
+
 
 
 
