@@ -89,7 +89,7 @@
  * Encoding the internal representation of the attribute to external
  * Setting the value by =, + or - operators.
  * Comparing a (decoded) value with the attribute value.
- * Freeing the space malloc-ed to the attribute value.
+ * Freeing the space calloc-ed to the attribute value.
  *
  * Some or all of the functions for an attribute type may be shared with
  * other attribute types.
@@ -128,8 +128,9 @@ int decode_arst_direct(
   char      *pbuf = NULL;
   char      *pc = NULL;
   char      *pstr = NULL;
-  size_t     ssize;
+  int     ssize;
   char      *tmpval = NULL;
+  char      *tmp = NULL;
 
   struct array_strings *stp = NULL;
 
@@ -142,15 +143,13 @@ int decode_arst_direct(
   ns = 1;
   ssize = strlen(val) + 1;
 
-  if ((tmpval = malloc((unsigned)ssize)) == NULL)
+  if ((tmpval = calloc(1, ssize)) == NULL)
     {
     /* FAILURE */
 
     return(PBSE_SYSTEM);
     }
 
-  memset(tmpval, 0, ssize);
-
   strcpy(tmpval,val);
 
   for (pc = tmpval;*pc;pc++)
@@ -175,7 +174,7 @@ int decode_arst_direct(
     ssize--;
     }
 
-  if ((pbuf = malloc((unsigned)ssize)) == NULL)
+  if ((pbuf = calloc(1, (unsigned)ssize)) == NULL)
     {
     /* FAILURE */
 
@@ -186,14 +185,13 @@ int decode_arst_direct(
 
   bksize = (ns - 1) * sizeof(char *) + sizeof(struct array_strings);
 
-  if (( patr->at_val.at_arst = (struct array_strings *)malloc(bksize)) == NULL)
+  if ((stp = (struct array_strings *)calloc(1, bksize)) == NULL)
     {
     /* FAILURE */
 
     return(PBSE_SYSTEM);
     }
 
-  stp = patr->at_val.at_arst;
   memset(stp, 0, bksize);
 
   stp->as_npointers = ns;
@@ -211,7 +209,7 @@ int decode_arst_direct(
 
   j  = 0;
 
-  pstr = parse_comma_string(tmpval);
+  pstr = parse_comma_string(tmpval,&tmp);
 
   while ((pstr != NULL) && (j < ns))
     {
@@ -230,7 +228,7 @@ int decode_arst_direct(
 
     j++;
 
-    pstr = parse_comma_string(NULL);
+    pstr = parse_comma_string(NULL,&tmp);
     }  /* END while ((pstr != NULL) && (j < ns)) */
 
   stp->as_usedptr = j;
@@ -239,7 +237,7 @@ int decode_arst_direct(
 
   patr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
 
-/*  patr->at_val.at_arst = stp;*/
+  patr->at_val.at_arst = stp;
 
   free(tmpval);
  
@@ -268,7 +266,8 @@ int decode_arst(
   struct attribute *patr,    /* O (modified) */
   char             *name,    /* I attribute name (notused) */
   char             *rescn,   /* I resource name (notused) */
-  char             *val)     /* I attribute value */
+  char             *val,     /* I attribute value */
+  int               perm) /* only used for resources */
 
   {
   int   rc;
@@ -420,7 +419,8 @@ int encode_arst(
   tlist_head *phead,  /* O ptr to head of attrlist list */
   char       *atname, /* I attribute name */
   char       *rsname, /* I resource name or NULL (optional) */
-  int         mode)   /* I encode mode */
+  int         mode,   /* I encode mode */
+  int         perm)   /* only used for resources */
 
   {
   char  *end;
@@ -472,7 +472,7 @@ int encode_arst(
       ++j;
     }
 
-  pal = attrlist_create(atname, rsname, j);
+  pal = attrlist_create(atname, rsname, j + 1);
 
   if (pal == NULL)
     {
@@ -590,7 +590,7 @@ int set_arst(
 
     need = sizeof(struct array_strings) + (j - 1) * sizeof(char *);
 
-    pas = (struct array_strings *)malloc((size_t)need);
+    pas = (struct array_strings *)calloc(1, (size_t)need);
 
     if (pas == NULL)
       {
@@ -644,7 +644,7 @@ int set_arst(
 
         nsize += nsize / 2;   /* alloc extra space */
 
-        if (!(pas->as_buf = malloc((size_t)nsize)))
+        if (!(pas->as_buf = calloc(1, (size_t)nsize)))
           {
           pas->as_bufsize = 0;
 
@@ -675,9 +675,13 @@ int set_arst(
         need = pas->as_bufsize + 2 * nsize;  /* alloc new buf */
 
         if (pas->as_buf)
+          {
           pc = realloc(pas->as_buf, (size_t)need);
+          if (pc != NULL)
+            memset(pc + pas->as_bufsize, 0, need - pas->as_bufsize);
+          }
         else
-          pc = malloc((size_t)need);
+          pc = calloc(1, (size_t)need);
 
         if (pc == NULL)
           {
@@ -759,9 +763,9 @@ int set_arst(
 
       j = 1.5 * (pas->as_usedptr + newpas->as_usedptr);
 
-      /* malloc the tmp array strings */
+      /* calloc the tmp array strings */
       need = sizeof(struct array_strings) + (j-1) * sizeof(char *);
-      tmp_arst = (struct array_strings *)malloc(need);
+      tmp_arst = (struct array_strings *)calloc(1, need);
 
       if (tmp_arst == NULL)
         return(PBSE_SYSTEM);
@@ -773,8 +777,8 @@ int set_arst(
       need = 2 * (nsize + used);
       tmp_arst->as_bufsize = need;
 
-      /* malloc the buffer size */
-      pc = malloc(need);
+      /* calloc the buffer size */
+      pc = calloc(1, need);
 
       if (pc == NULL)
         return(PBSE_SYSTEM);
@@ -846,6 +850,7 @@ int set_arst(
           if (!strcmp(pas->as_string[i], newpas->as_string[j]))
             {
             /* compact buffer */
+            int k;
 
             nsize = strlen(pas->as_string[i]) + 1;
 
@@ -853,7 +858,10 @@ int set_arst(
 
             need = pas->as_next - pc;
 
-            memcpy(pas->as_string[i], pc, (size_t)need);
+            for (k = 0; k < need; k++)
+              pas->as_string[i][k] = *(pc + k);
+              
+/*            memcpy(pas->as_string[i], pc, (size_t)need);*/
 
             pas->as_next -= nsize;
 
