@@ -100,7 +100,7 @@
  * Encoding the machine representation of the value to a string
  * Setting the value by =, + or - operators.
  * Comparing a (decoded) value with the attribute value.
- * Freeing the space malloc-ed to the attribute value.
+ * Freeing the space calloc-ed to the attribute value.
  *
  * Some or all of the functions for an attribute type may be shared with
  * other attribute types.
@@ -124,13 +124,14 @@ static char false_val[] = ATR_FALSE;
  *  *patr elements set
  */
 
-int
-decode_b(
-  struct attribute *patr,
-  char *name,  /* attribute name */
-  char *rescn,  /* resource name, unused here */
-  char *val  /* attribute value */
-)
+int decode_b(
+
+  attribute *patr,
+  char      *name,  /* attribute name */
+  char      *rescn,  /* resource name, unused here */
+  char      *val,  /* attribute value */
+  int        perm) /* only used for resources */
+
   {
   char *pc;
 
@@ -181,14 +182,15 @@ decode_b(
  */
 /*ARGSUSED*/
 
-int
-encode_b(
-  attribute *attr,  /* ptr to attribute */
+int encode_b(
+
+  attribute  *attr,  /* ptr to attribute */
   tlist_head *phead,  /* head of attribute list */
-  char *atname, /* attribute name */
-  char *rsname, /* resource name or null */
-  int mode  /* encode mode, unused here */
-)
+  char       *atname, /* attribute name */
+  char       *rsname, /* resource name or null */
+  int         mode,  /* encode mode, unused here */
+  int         perm) /* only used for resources */
+
   {
   size_t   ct;
   svrattrl *pal;
@@ -209,14 +211,14 @@ encode_b(
     value = false_val;
     }
 
-  ct = strlen(value) + 1;
+  ct = strlen(value);
 
-  pal = attrlist_create(atname, rsname, ct);
+  pal = attrlist_create(atname, rsname, ct+1);
 
   if (pal == (svrattrl *)0)
     return (-1);
 
-  strcpy(pal->al_value, value);
+  strncpy(pal->al_value, value, ct);
 
   pal->al_flags = attr->at_flags;
 
