@@ -116,10 +116,6 @@
  * ----------------------------------------------------------------------------
  */
 
-/* Global Variables */
-
-int resc_access_perm;
-
 /* External Global Items */
 
 int comp_resc_gt; /* count of resources compared > */
@@ -143,7 +139,8 @@ int decode_resc(
   struct attribute *patr,  /* Modified on Return */
   char             *name,  /* attribute name */
   char             *rescn, /* I resource name - is used here */
-  char             *val)   /* resource value */
+  char             *val,   /* resource value */
+  int               perm)  /* access permissions */
 
   {
   resource *prsc;
@@ -190,15 +187,15 @@ int decode_resc(
 
   /* note special use of ATR_DFLAG_ACCESS, see server/attr_recov() */
 
-  if (((prsc->rs_defin->rs_flags & resc_access_perm & ATR_DFLAG_WRACC) == 0) &&
-      (resc_access_perm != ATR_DFLAG_ACCESS))
+  if (((prsc->rs_defin->rs_flags & perm & ATR_DFLAG_WRACC) == 0) &&
+      (perm != ATR_DFLAG_ACCESS))
     {
     return(PBSE_ATTRRO);
     }
 
   patr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
 
-  rv = prdef->rs_decode(&prsc->rs_value, name, rescn, val);
+  rv = prdef->rs_decode(&prsc->rs_value, name, rescn, val, perm);
 
   if (rv == 0)
     {
@@ -230,7 +227,7 @@ int decode_resc(
  * reset if the default changes or it is moved.
  *
  * If the mode is ATR_ENCODE_CLIENT or ATR_ENCODE_MOM, the client permission
- * passed in the global variable resc_access_perm is checked against each
+ * passed in ac_perm is checked against each
  * definition.  This allows a resource by resource access setting, not just
  * on the attribute.
  *
@@ -241,11 +238,12 @@ int decode_resc(
 
 int encode_resc(
 
-  attribute *attr,   /* ptr to attribute to encode */
+  attribute  *attr,    /* ptr to attribute to encode */
   tlist_head *phead,   /* head of attrlist list */
-  char  *atname,   /* attribute name */
-  char  *rsname,   /* resource name, null on call */
-  int   mode)   /* encode mode */
+  char       *atname,  /* attribute name */
+  char       *rsname,  /* resource name, null on call */
+  int         mode,    /* encode mode */
+  int         ac_perm) /* access permissions */
 
   {
   int     dflt;
@@ -276,7 +274,7 @@ int encode_resc(
      * encode if sending to server and not default and have permission
      */
 
-    perm = prsc->rs_defin->rs_flags & resc_access_perm ;
+    perm = prsc->rs_defin->rs_flags & ac_perm ;
     dflt = prsc->rs_value.at_flags & ATR_VFLAG_DEFLT;
 
     if (((mode == ATR_ENCODE_CLIENT) && perm)  ||
@@ -286,8 +284,7 @@ int encode_resc(
       {
 
       rsname = prsc->rs_defin->rs_name;
-      rc = prsc->rs_defin->rs_encode(&prsc->rs_value, phead,
-                                     atname, rsname, mode);
+      rc = prsc->rs_defin->rs_encode(&prsc->rs_value, phead, atname, rsname, mode, ac_perm);
 
       if (rc < 0)
         return (rc);
@@ -504,16 +501,18 @@ int comp_resc(
 
 int comp_resc2(
 
-  struct attribute *attr,           /* I queue's min/max attributes */
-  struct attribute *with,           /* I job's current requirements/attributes */
-  int               IsQueueCentric, /* I */
-  char             *EMsg)           /* O (optional,minsize=1024) */
+  struct attribute   *attr,           /* I queue's min/max attributes */
+  struct attribute   *with,           /* I job's current requirements/attributes */
+  int                 IsQueueCentric, /* I */
+  char               *EMsg,           /* O (optional,minsize=1024) */
+  enum compare_types  type)           /* I type of comparison to detect */
 
   {
   resource *atresc;
   resource *wiresc;
-  int rc;
-  char *LimitName;
+  int       rc;
+  int       comp_ret = 0;
+  char     *LimitName;
 
   comp_resc_gt = 0;
   comp_resc_eq = 0;
@@ -558,21 +557,33 @@ int comp_resc2(
                 sprintf(EMsg, "cannot satisfy queue min %s requirement",
                         (LimitName != NULL) ? LimitName : "resource");
                 }
+              
+              if (type == GREATER)
+                comp_ret++;
 
               comp_resc_gt++;
               }
             else if (rc < 0)
               {
+              if (type == LESS)
+                comp_ret++;
+
               comp_resc_lt++;
               }
             else
               {
+              if (type == EQUAL)
+                comp_ret++;
+
               comp_resc_eq++;
               }
             }
           }
         else
           {
+          if (type == NOT_COMPARED)
+            comp_ret++;
+
           comp_resc_nc++;
           }
         }
@@ -613,21 +624,33 @@ int comp_resc2(
                 sprintf(EMsg, "cannot satisfy queue min %s requirement",
                         (LimitName != NULL) ? LimitName : "resource");
                 }
+              
+              if (type == GREATER)
+                comp_ret++;
 
               comp_resc_gt++;
               }
             else if (rc < 0)
               {
+              if (type == LESS)
+                comp_ret++;
+
               comp_resc_lt++;
               }
             else
               {
+              if (type == EQUAL)
+                comp_ret++;
+
               comp_resc_eq++;
               }
             }
           }
         else
           {
+          if (type == NOT_COMPARED)
+            comp_ret++;
+
           comp_resc_nc++;
           }
         }
@@ -636,7 +659,7 @@ int comp_resc2(
       }  /* END while() */
     }
 
-  return(0);
+  return(comp_ret);
   }  /* END comp_resc2() */
 
 
@@ -756,7 +779,7 @@ resource *find_resc_entry(
  * The parent attribute is marked with ATR_VFLAG_SET and ATR_VFLAG_MODIFY
  *
  * Returns: pointer to the newly added entry or NULL if unable
- *   to add it (malloc failed).  If the resource already
+ *   to add it (calloc failed).  If the resource already
  *   exists (it shouldn't) then that one is returned.
  */
 
@@ -784,11 +807,11 @@ resource *add_resource_entry(
     pr = (resource *)GET_NEXT(pr->rs_link);
     }
 
-  new = (resource *)malloc(sizeof(resource));
+  new = (resource *)calloc(1, sizeof(resource));
 
-  if (new == (resource *)0)
+  if (new == NULL)
     {
-    return ((resource *)0);
+    return NULL;
     }
 
   CLEAR_LINK(new->rs_link);
@@ -798,7 +821,7 @@ resource *add_resource_entry(
   new->rs_value.at_flags = 0;
   prdef->rs_free(&new->rs_value);
 
-  if (pr != (resource *)0)
+  if (pr != NULL)
     {
     insert_link(&pr->rs_link, &new->rs_link, new, LINK_INSET_BEFORE);
     }
