@@ -124,10 +124,11 @@
 
 int decode_time(
 
-  struct attribute *patr,  /* I/O (modified) */
-  char             *name,  /* I - attribute name (not used) */
-  char             *rescn, /* I - resource name (not used) */
-  char             *val)   /* I - attribute value */
+  attribute *patr,  /* I/O (modified) */
+  char      *name,  /* I - attribute name (not used) */
+  char      *rescn, /* I - resource name (not used) */
+  char      *val,   /* I - attribute value */
+  int        perm)  /* only used for resources */
 
   {
   int   i;
@@ -261,19 +262,20 @@ badval:
 
 /* NOTE:  if phead not specified, report output via atname (minsize=1024) */
 
-#define CVNBUFSZ 21
+#define ENCODE_TIME_SIZE 21
 
 int encode_time(
 
-  attribute *attr,   /* ptr to attribute (value in attr->at_val.at_long) */
+  attribute  *attr,   /* ptr to attribute (value in attr->at_val.at_long) */
   tlist_head *phead,  /* head of attrlist list (optional) */
-  char      *atname, /* attribute name */
-  char      *rsname, /* resource name (optional) */
-  int        mode)   /* encode mode (not used) */
+  char       *atname, /* attribute name */
+  char       *rsname, /* resource name (optional) */
+  int         mode,   /* encode mode (not used) */
+  int         perm)  /* only used for resources */
 
   {
   size_t  ct;
-  char   cvnbuf[CVNBUFSZ];
+  char   cvnbuf[ENCODE_TIME_SIZE];
   int    hr;
   int   min;
   long   n;
@@ -309,11 +311,11 @@ int encode_time(
 
   pv += strlen(pv);
 
-  ct = strlen(cvnbuf) + 1;
+  ct = strlen(cvnbuf);
 
   if (phead != NULL)
     {
-    pal = attrlist_create(atname, rsname, ct);
+    pal = attrlist_create(atname, rsname, ct+1);
 
     if (pal == NULL)
       {
@@ -330,7 +332,8 @@ int encode_time(
     }
   else
     {
-    strcpy(atname, cvnbuf);
+    /* Is this right? Copy time to the name of the attr? */
+    snprintf(atname, strlen(atname), "%s", cvnbuf);
     }
 
   /* SUCCESS */
