@@ -95,6 +95,7 @@
 #include "pbs_nodes.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../Liblog/pbs_log.h"
 #if SYSLOG
 #include <syslog.h>
 #endif
@@ -172,6 +173,8 @@ int PNodeStateToString(
   int   BufSize) /* I */
 
   {
+  int len; 
+
   if (Buf == NULL)
     {
     return(-1);
@@ -183,67 +186,118 @@ int PNodeStateToString(
 
   if (SBM & (INUSE_DOWN))
     {
-    if (Buf[0] != '\0')
-      strncat(Buf, ",", BufSize - strlen(Buf));
+    len = strlen(ND_down);
 
-    strncat(Buf, ND_down, BufSize - strlen(Buf));
+    if (len < BufSize)
+      {
+      strcat(Buf, ND_down);
+      BufSize -= len;
+      }
     }
 
   if (SBM & (INUSE_OFFLINE))
     {
-    if (Buf[0] != '\0')
-      strncat(Buf, ",", BufSize - strlen(Buf));
+    len = strlen(ND_offline) + 1;
 
-    strncat(Buf, ND_offline, BufSize - strlen(Buf));
+    if (len < BufSize)
+      {
+      if (Buf[0] != '\0')
+        strcat(Buf, ",");
+      else
+        len--;
+  
+      strcat(Buf, ND_offline);
+      BufSize -= len;
+      }
     }
 
   if (SBM & (INUSE_JOB))
     {
-    if (Buf[0] != '\0')
-      strncat(Buf, ",", BufSize - strlen(Buf));
+    len = strlen(ND_job_exclusive) + 1;
 
-    strncat(Buf, ND_job_exclusive, BufSize - strlen(Buf));
+    if (len < BufSize)
+      {
+      if (Buf[0] != '\0')
+        strcat(Buf, ",");
+      else
+        len--;
+      
+      strcat(Buf, ND_job_exclusive);
+      BufSize -= len;
+      }
     }
 
   if (SBM & (INUSE_JOBSHARE))
     {
-    if (Buf[0] != '\0')
-      strncat(Buf, ",", BufSize - strlen(Buf));
-
-    strncat(Buf, ND_job_sharing, BufSize - strlen(Buf));
+    len = strlen(ND_job_sharing) + 1;
+    
+    if (len < BufSize)
+      {
+      if (Buf[0] != '\0')
+        strcat(Buf, ",");
+      else
+        len--;
+      
+      strcat(Buf, ND_job_sharing);
+      BufSize -= len;
+      }
     }
 
   if (SBM & (INUSE_BUSY))
     {
-    if (Buf[0] != '\0')
-      strncat(Buf, ",", BufSize - strlen(Buf));
+    len = strlen(ND_busy) + 1;
 
-    strncat(Buf, ND_busy, BufSize - strlen(Buf));
+    if (len < BufSize)
+      {
+      if (Buf[0] != '\0')
+        strcat(Buf, ",");
+      else
+        len--;
+      
+      strcat(Buf, ND_busy);
+      BufSize -= len;
+      }
     }
 
   if (SBM & (INUSE_RESERVE))
     {
-    if (Buf[0] != '\0')
-      strncat(Buf, ",", BufSize - strlen(Buf));
+    len = strlen(ND_reserve) + 1;
 
-    strncat(Buf, ND_reserve, BufSize - strlen(Buf));
+    if (len < BufSize)
+      {
+      if (Buf[0] != '\0')
+        strcat(Buf, ",");
+      else
+        len--;
+
+      strcat(Buf, ND_reserve);
+      BufSize -= len;
+      }
     }
 
   if (SBM & (INUSE_UNKNOWN))
     {
-    if (Buf[0] != '\0')
-      strncat(Buf, ",", BufSize - strlen(Buf));
+    len = strlen(ND_state_unknown) + 1;
 
-    strncat(Buf, ND_state_unknown, BufSize - strlen(Buf));
+    if (len < BufSize)
+      {
+      if (Buf[0] != '\0')
+        strcat(Buf, ",");
+      else
+        len--;
+
+      strcat(Buf, ND_state_unknown);
+      BufSize -= len;
+      }
     }
 
   if (Buf[0] == '\0')
     {
-    strncpy(Buf, ND_free, BufSize);
+    snprintf(Buf, BufSize, "%s", ND_free);
     }
 
   return(0);
-  }
+  } /* END PNodeStateToString() */
 
 
 
@@ -266,7 +320,8 @@ int encode_state(
 
   char      *aname, /*attribute's name    */
   char      *rname, /*resource's name (null if none)  */
-  int        mode) /*mode code, unused here   */
+  int        mode, /*mode code, unused here   */
+  int        perm) /* only used for resources */
 
   {
   int   i;
@@ -340,14 +395,14 @@ int encode_state(
  *       or nothing to encode
  */
 
-int
-encode_ntype(
-  attribute *pattr, /*struct attribute being encoded  */
-  tlist_head *ph,  /*head of a list of  "svrattrl"   */
-  char *aname, /*attribute's name    */
-  char *rname, /*resource's name (null if none)  */
-  int mode  /*mode code, unused here   */
-)
+int encode_ntype(
+  
+  attribute  *pattr, /*struct attribute being encoded  */
+  tlist_head *ph,    /*head of a list of  "svrattrl"   */
+  char       *aname, /*attribute's name    */
+  char       *rname, /*resource's name (null if none)  */
+  int         mode,  /*mode code, unused here   */
+  int         perm)  /* only used for resources */
 
   {
   svrattrl *pal;
@@ -404,25 +459,25 @@ encode_ntype(
 
 int encode_jobs(
 
-  attribute *pattr, /*struct attribute being encoded  */
-  tlist_head *ph, /*head of a  list of "svrattrl"   */
+  attribute  *pattr, /*struct attribute being encoded  */
+  tlist_head *ph,    /*head of a  list of "svrattrl"   */
   /*structs which are to be returned*/
-  char      *aname, /*attribute's name    */
-  char      *rname, /*resource's name (null if none)  */
-  int      mode) /*mode code, unused here   */
+  char       *aname, /*attribute's name    */
+  char       *rname, /*resource's name (null if none)  */
+  int         mode,  /*mode code, unused here   */
+  int         perm)  /* only used for resources */
 
   {
-  svrattrl  *pal;
+  svrattrl       *pal;
 
   struct jobinfo *jip;
-
   struct pbsnode *pnode;
-
   struct pbssubn *psubn;
-  int  i;
-  int  jobcnt;  /*number of jobs using the node     */
-  int  strsize; /*computed string size      */
-  char  *job_str; /*holds comma separated list of jobs*/
+  int             i;
+  int             jobcnt;  /*number of jobs using the node     */
+  int             strsize; /*computed string size      */
+  char           *job_str; /*holds comma separated list of jobs*/
+  int             job_str_len = 0;
 
   if (pattr == NULL)
     {
@@ -448,7 +503,7 @@ int encode_jobs(
       {
       jobcnt++;
 
-      strsize += strlen(jip->job->ji_qs.ji_jobid) + PCONST_ENCOVERHEAD;
+      strsize += strlen(jip->jobid) + PCONST_ENCOVERHEAD;
       }
     }    /* END for (psubn) */
 
@@ -459,13 +514,11 @@ int encode_jobs(
     return(0);
     }
 
-  if (!(job_str = (char *)malloc(strsize)))
+  if ((job_str = (char *)calloc(1, strsize+1)) == NULL)
     {
     return -(PBSE_SYSTEM);
     }
 
-  memset(job_str, 0, strsize);
-
   i = 0;
 
   for (psubn = pnode->nd_psn;psubn != NULL;psubn = psubn->next)
@@ -478,12 +531,12 @@ int encode_jobs(
         i++;
 
       sprintf(job_str + strlen(job_str), "%d/%s",
-              psubn->index,
-              jip->job->ji_qs.ji_jobid);
+        psubn->index, jip->jobid);
       }
     }    /* END for (psubn) */
+  job_str_len = strlen(job_str);
 
-  pal = attrlist_create(aname, rname, (int)strlen(job_str) + 1);
+  pal = attrlist_create(aname, rname, job_str_len+1);
 
   if (pal == NULL)
     {
@@ -492,7 +545,7 @@ int encode_jobs(
     return -(PBSE_SYSTEM);
     }
 
-  strcpy(pal->al_value, job_str);
+  strncpy(pal->al_value, job_str, job_str_len);
 
   pal->al_flags = ATR_VFLAG_SET;
 
@@ -522,7 +575,8 @@ int decode_state(
   attribute *pattr,   /* I (modified) */
   char      *name,    /* attribute name */
   char      *rescn,   /* resource name, unused here */
-  char      *val)     /* attribute value */
+  char      *val,     /* attribute value */
+  int        perm)    /* only used for resources */
 
   {
   int   rc = 0;  /*return code; 0==success*/
@@ -531,6 +585,7 @@ int decode_state(
 
   char  strbuf[512]; /*should handle most vals*/
   char *sbufp;
+  char *ptr = NULL;
   int   slen;
 
   if (val == NULL)
@@ -551,7 +606,7 @@ int decode_state(
     }
   else
     {
-    if (!(sbufp = (char *)malloc(slen + 1)))
+    if (!(sbufp = (char *)calloc(1, slen + 1)))
       {
       return(PBSE_SYSTEM);
       }
@@ -559,7 +614,7 @@ int decode_state(
 
   strcpy(sbufp, val);
 
-  if ((str = parse_comma_string(sbufp)) == NULL)
+  if ((str = parse_comma_string(sbufp,&ptr)) == NULL)
     {
     if (slen >= 512)
       free(sbufp);
@@ -583,7 +638,7 @@ int decode_state(
   /*last call left off.  The initial comma separated string   */
   /*copy pointed to by sbufp is modified with each func call  */
 
-  while ((str = parse_comma_string(NULL)) != NULL)
+  while ((str = parse_comma_string(NULL,&ptr)) != NULL)
     {
     if ((rc = set_nodeflag(str, &flag)) != 0)
       break;
@@ -625,13 +680,14 @@ int decode_state(
  * that the types are going to be mutually exclusive
  */
 
-int
-decode_ntype(
+int decode_ntype(
+
   attribute *pattr,
-  char *name,             /* attribute name */
-  char *rescn,            /* resource name, unused here */
-  char *val              /* attribute value */
-)
+  char      *name,   /* attribute name */
+  char      *rescn,  /* resource name, unused here */
+  char      *val,    /* attribute value */
+  int        perm)   /* only used for resources */
+
 
   {
   int rc = 0;  /*return code; 0==success*/
@@ -1100,6 +1156,93 @@ int node_status_list(
   return(rc);
   }  /* END node_status_list() */
 
+
+/*
+ * node_gpustatus_list - Either derive a "gpu status list" attribute from the node
+ *                 or update node's status list from attribute's status list.
+ */
+
+int node_gpustatus_list(
+
+  attribute *new,           /*derive status into this attribute*/
+  void      *pnode,         /*pointer to a pbsnode struct     */
+  int        actmode)       /*action mode; "NEW" or "ALTER"   */
+
+  {
+  int              rc = 0;
+
+  struct pbsnode  *np;
+  attribute        temp;
+
+  np = (struct pbsnode *)pnode;    /* because of at_action arg type */
+
+  switch (actmode)
+    {
+
+    case ATR_ACTION_NEW:
+
+      /* if node has a status list, then copy array_strings    */
+      /* into temp to use to setup a copy, otherwise setup empty */
+
+      if (np->nd_gpustatus != NULL)
+        {
+        /* setup temporary attribute with the array_strings */
+        /* from the node                                    */
+
+        temp.at_val.at_arst = np->nd_gpustatus;
+        temp.at_flags = ATR_VFLAG_SET;
+        temp.at_type  = ATR_TYPE_ARST;
+
+        rc = set_arst(new, &temp, SET);
+        }
+      else
+        {
+        /* node has no properties, setup empty attribute */
+
+        new->at_val.at_arst = NULL;
+        new->at_flags       = 0;
+        new->at_type        = ATR_TYPE_ARST;
+        }
+
+      break;
+
+    case ATR_ACTION_ALTER:
+
+      if (np->nd_gpustatus != NULL)
+        {
+        free(np->nd_gpustatus->as_buf);
+        free(np->nd_gpustatus);
+
+        np->nd_gpustatus = NULL;
+        }
+
+      /* update node with new attr_strings */
+
+      np->nd_gpustatus = new->at_val.at_arst;
+
+      new->at_val.at_arst = NULL;
+
+      /* update number of status items listed in node */
+      /* does not include name and subnode property */
+
+      if (np->nd_gpustatus != NULL)
+        np->nd_ngpustatus = np->nd_gpustatus->as_usedptr;
+      else
+        np->nd_ngpustatus = 0;
+
+      break;
+
+    default:
+
+      rc = PBSE_INTERNAL;
+
+      break;
+    }  /* END switch(actmode) */
+
+  return(rc);
+  }  /* END node_status_list() */
+
+
 /*
  * node_note - Either derive a note attribute from the node
  *             or update node's note from attribute's list.
@@ -1189,36 +1332,29 @@ int set_note_str(
 
   {
   static char id[] = "set_note_str";
-  size_t nsize;
-  int rc = 0;
+  size_t      nsize;
+  int         rc = 0;
+  char        log_buf[LOCAL_LOG_BUF_SIZE];
 
   assert(attr && new && new->at_val.at_str && (new->at_flags & ATR_VFLAG_SET));
   nsize = strlen(new->at_val.at_str);    /* length of new note */
 
   if (nsize > MAX_NOTE)
     {
-    sprintf(log_buffer, "Warning: Client attempted to set note with len (%d) > MAX_NOTE (%d)",
-            (int)nsize,
-            MAX_NOTE);
+    sprintf(log_buf, "Warning: Client attempted to set note with len (%d) > MAX_NOTE (%d)",
+      (int)nsize,
+      MAX_NOTE);
 
-    log_record(
-      PBSEVENT_SECURITY,
-      PBS_EVENTCLASS_REQUEST,
-      id,
-      log_buffer);
+    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_REQUEST,id,log_buf);
 
     rc = PBSE_BADNDATVAL;
     }
 
   if (strchr(new->at_val.at_str, '\n') != NULL)
     {
-    sprintf(log_buffer, "Warning: Client attempted to set note with a newline char");
+    sprintf(log_buf, "Warning: Client attempted to set note with a newline char");
 
-    log_record(
-      PBSEVENT_SECURITY,
-      PBS_EVENTCLASS_REQUEST,
-      id,
-      log_buffer);
+    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_REQUEST,id,log_buf);
 
     rc = PBSE_BADNDATVAL;
     }
