@@ -96,6 +96,7 @@
 #include <stdio.h>
 #include <pwd.h>
 #include <string.h>
+#include <ctype.h>
 #include <signal.h>
 #include <netdb.h>
 #include <unistd.h>
@@ -382,7 +383,7 @@ int PBSD_munge_authenticate(
   int handle) /* I */
 
   {
-  int                 rc;
+  int                 rc = PBSE_NONE;
 
   int                 fd;
   FILE               *munge_pipe;
@@ -400,6 +401,7 @@ int PBSD_munge_authenticate(
   unsigned short      user_port = 0;
   struct sockaddr_in  sockname;
   socklen_t           socknamelen = sizeof(sockname);
+  struct tcp_chan *chan = NULL;
 
   snprintf(munge_command,sizeof(munge_command),
     "munge -n 2>/dev/null");
@@ -428,7 +430,7 @@ int PBSD_munge_authenticate(
     /* read failed */
     local_errno = errno;
     log_err(local_errno, __func__, "error reading pipe in PBSD_munge_authenticate");
-    return(-1);
+    return -1;
     }
   
   /* if we got no bytes back then Munge may not be installed etc. */
@@ -446,30 +448,33 @@ int PBSD_munge_authenticate(
   if (rc == -1)
     {
     fprintf(stderr, "getsockname failed: %d\n", errno);
-    return(-1);
+    return rc;
     }
   
   user_port = ntohs(sockname.sin_port);
   
-  DIS_tcp_setup(psock);
-
-  if ((rc = encode_DIS_ReqHdr(psock, PBS_BATCH_AltAuthenUser, pwent->pw_name)) ||
-      (rc = diswui(psock, user_port)) ||
-      (rc = diswst(psock, munge_buf)) ||
-      (rc = encode_DIS_ReqExtend(psock, NULL)) ||
-      (rc = DIS_tcp_wflush(psock)))
+  if ((chan = DIS_tcp_setup(psock)) == NULL)
+    {
+    rc = PBSE_MEM_MALLOC;
+    }
+  else if ((rc = encode_DIS_ReqHdr(chan,PBS_BATCH_AltAuthenUser,pwent->pw_name))
+      || (rc = diswui(chan, user_port))
+      || (rc = diswst(chan, munge_buf))
+      || (rc = encode_DIS_ReqExtend(chan, NULL))
+      || (rc = DIS_tcp_wflush(chan)))
     {
     /* ERROR */
-    return(rc);
     }
   else
     {
     /* read the reply */
     if ((reply = PBSD_rdrpy(&local_errno, handle)) != NULL)
       free(reply);
-  
-    return(PBSE_NONE);
+    rc = PBSE_NONE;
     }
+  if (chan != NULL)
+    DIS_tcp_cleanup(chan);
+  return rc;
   } /* END PBSD_munge_authenticate() */
 #endif /* ifdef MUNGE_AUTH */
 
@@ -495,6 +500,8 @@ int parse_daemon_response(long long code, long long len, char *buf)
   return rc;
   }
 
+
+
 int get_parent_client_socket(int psock, int *pcsock)
   {
   int rc = PBSE_NONE;
@@ -590,6 +597,7 @@ int validate_socket(
         {
         fprintf(stderr, "%s : Connection authorized (server socket %d)\n", id, parent_client_socket);
         }
+      socket_close(local_socket);
       }
     }
   if (rc != PBSE_NONE)
@@ -928,7 +936,6 @@ int pbs_original_connect(
      * try */
 
     connection[out].ch_socket = socket(AF_INET, SOCK_STREAM, 0);
-    DIS_tcp_init(connection[out].ch_socket);
 
     if (connection[out].ch_socket < 0)
       {
@@ -1097,8 +1104,6 @@ int pbs_original_connect(
 
   /* setup DIS support routines for following pbs_* calls */
 
-  DIS_tcp_setup(connection[out].ch_socket);
-
   if ((ptr = getenv("PBSAPITIMEOUT")) != NULL)
     {
     pbs_tcp_timeout = strtol(ptr, NULL, 0);
@@ -1126,11 +1131,13 @@ int pbs_disconnect_socket(
 
   {
   char tmp_buf[THE_BUF_SIZE / 4];
+  struct tcp_chan *chan = NULL;
 
-  DIS_tcp_setup(sock);
-
-  if ((encode_DIS_ReqHdr(sock, PBS_BATCH_Disconnect, pbs_current_user) == 0) &&
-      (DIS_tcp_wflush(sock) == 0))
+  if ((chan = DIS_tcp_setup(sock)) == NULL)
+    {
+    }
+  else if ((encode_DIS_ReqHdr(chan,PBS_BATCH_Disconnect, pbs_current_user) == 0)
+      && (DIS_tcp_wflush(chan) == 0))
     {
     int atime;
     struct sigaction act;
@@ -1158,6 +1165,8 @@ int pbs_disconnect_socket(
     sigaction(SIGALRM, &oldact, NULL);
     }
 
+  if (chan != NULL)
+    DIS_tcp_cleanup(chan);
   close(sock);
   return(0);
   }  /* END pbs_disconnect() */
@@ -1214,28 +1223,21 @@ int pbs_connect(
 
   memset(server_name_list, 0, sizeof(server_name_list));
 
-  /* If a server name is passed in, use it, otherwise use the list from server_name file. */
+  /* Use the list from the server_name file.
+   * If a server name is passed in, append it at the beginning. */
 
   if (server_name_ptr && server_name_ptr[0])
     {
     snprintf(server_name_list, sizeof(server_name_list), "%s", server_name_ptr);
-
-    if (getenv("PBSDEBUG"))
-      {
-      fprintf(stderr, "pbs_connect called with explicit server name \"%s\"\n",
-        server_name_list);
-      }
+    strcat(server_name_list, ",");
     }
-  else
-    {
-    snprintf(server_name_list, sizeof(server_name_list), "%s", pbs_get_server_list());
 
-    if (getenv("PBSDEBUG"))
-      {
-      fprintf(stderr, "pbs_connect using default server name list \"%s\"\n",
+  strncat(server_name_list, pbs_get_server_list(),
+      sizeof(server_name_list) -1 - strlen(server_name_ptr) - 1);
+
+  if (getenv("PBSDEBUG"))
+    fprintf(stderr, "pbs_connect using following server list \"%s\"\n",
         server_name_list);
-      }
-    }
 
   list_len = csv_length(server_name_list);
 
@@ -1245,6 +1247,9 @@ int pbs_connect(
 
     if (tp && tp[0])
       {
+      /* Trim any leading space */
+      while(isspace(*tp)) tp++;
+
       memset(current_name, 0, sizeof(current_name));
       snprintf(current_name, sizeof(current_name), "%s", tp);
 
