@@ -86,6 +86,7 @@
 #include <stdio.h>
 #include "libpbs.h"
 #include "dis.h"
+#include "tcp.h" /* tcp_chan */
 
   
 /* NOTE:  routes over to req_runjob() on server side
@@ -143,6 +144,7 @@ int pbs_runjob_err(
   struct batch_reply   *reply;
   unsigned int          resch = 0;
   int                   sock;
+  struct tcp_chan *chan = NULL;
 
   /* NOTE:  set_task sets WORK_Deferred_Child : request remains until child terminates */
 
@@ -162,25 +164,28 @@ int pbs_runjob_err(
 
   /* setup DIS support routines for following DIS calls */
 
-  DIS_tcp_setup(sock);
-
-  /* send run request */
-
-  if ((rc = encode_DIS_ReqHdr(sock, PBS_BATCH_RunJob, pbs_current_user)) ||
-      (rc = encode_DIS_RunJob(sock, jobid, location, resch)) ||
-      (rc = encode_DIS_ReqExtend(sock, extend)))
+  if ((chan = DIS_tcp_setup(sock)) == NULL)
+    {
+    pthread_mutex_unlock(connection[c].ch_mutex);
+    rc = PBSE_PROTOCOL;
+    return rc;
+    }
+    /* send run request */
+  else if ((rc = encode_DIS_ReqHdr(chan, PBS_BATCH_RunJob, pbs_current_user)) ||
+      (rc = encode_DIS_RunJob(chan, jobid, location, resch)) ||
+      (rc = encode_DIS_ReqExtend(chan, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
 
     pthread_mutex_unlock(connection[c].ch_mutex);
-
+    DIS_tcp_cleanup(chan);
     return(PBSE_PROTOCOL);
     }
 
-  if (DIS_tcp_wflush(sock))
+  if (DIS_tcp_wflush(chan))
     {
     pthread_mutex_unlock(connection[c].ch_mutex);
-
+    DIS_tcp_cleanup(chan);
     return(PBSE_PROTOCOL);
     }
 
@@ -193,7 +198,7 @@ int pbs_runjob_err(
   pthread_mutex_unlock(connection[c].ch_mutex);
 
   PBSD_FreeReply(reply);
-
+  DIS_tcp_cleanup(chan);
   return(rc);
   }  /* END pbs_runjob_err() */
 
