@@ -139,7 +139,7 @@ static volatile int  log_opened = 0;
 static int      syslogopen = 0;
 #endif /* SYSLOG */
 
-pthread_mutex_t *log_mutex;
+pthread_mutex_t *log_mutex = NULL;
 
 /* variables for job logging */
 static int      job_log_auto_switch = 0;
@@ -272,9 +272,7 @@ static char *mk_job_log_name(
             ptm->tm_mday);
     }
 
-  pthread_mutex_lock(job_log_mutex);
   joblog_open_day = ptm->tm_yday; /* Julian date log opened */
-  pthread_mutex_unlock(job_log_mutex);
 
   return(pbuf);
   }  /* END mk_job_log_name() */
@@ -843,7 +841,9 @@ void log_record(
      * if we can't open this then we're going to have a nice surprise failure */
     if (logfile != NULL)
       {
+      pthread_mutex_unlock(log_mutex);
       log_err(rc, "log_record", "PBS cannot write to its log");
+      pthread_mutex_lock(log_mutex);
       fclose(logfile);
       }
     logfile = savlog;
