@@ -121,12 +121,15 @@
 #include "../Libifl/lib_ifl.h" /* DIS_* */
 #include "pbs_error.h" /* PBSE_NONE */
 
+char   local_host_name[PBS_MAXHOSTNAME + 1];
+size_t local_host_name_len = PBS_MAXHOSTNAME;
+
 extern int  LOGLEVEL;
+extern char local_host_name[];
 
 /* External Functions Called */
 
 void initialize_connections_table();
-extern void process_request(int);
 
 extern time_t time(time_t *);
 
@@ -149,7 +152,7 @@ pthread_mutex_t *num_connections_mutex = NULL;
 static fd_set   *GlobalSocketReadSet = NULL;
 pthread_mutex_t *global_sock_read_mutex = NULL;
 
-static void *(*read_func[2])(void *);
+void *(*read_func[2])(void *);
 
 pthread_mutex_t *netrates_mutex = NULL;
 pthread_mutex_t *nc_list_mutex  = NULL;
@@ -263,10 +266,6 @@ int *netcounter_get(void)
   return netrates;
   }
 
-
-
-
-
 /**
  * init_network - initialize the network interface
  * allocate a socket and bind it to the service port,
@@ -306,17 +305,15 @@ int init_network(
 
   if (initialized == 0)
     {
-    DIS_tcp_init(-1);
     initialize_connections_table();
 
     for (i = 0;i < PBS_NET_MAX_CONNECTIONS;i++)
       {
       svr_conn[i].cn_mutex = (pthread_mutex_t *)calloc(1, sizeof(pthread_mutex_t));
       pthread_mutex_init(svr_conn[i].cn_mutex,NULL);
+
       pthread_mutex_lock(svr_conn[i].cn_mutex);
-      
       svr_conn[i].cn_active = Idle;
-      
       pthread_mutex_unlock(svr_conn[i].cn_mutex);
       }
     
@@ -442,15 +439,12 @@ int init_network(
 
 #endif  /* END ENABLE_UNIX_SOCKETS */
 
-  if (port != 0)
-    {
     /* allocate a minute's worth of counter structs */
 
-    if (nc_list_mutex == NULL)
-      {
-      nc_list_mutex = calloc(1, sizeof(pthread_mutex_t));
-      pthread_mutex_init(nc_list_mutex,NULL);
-      }
+  if (nc_list_mutex == NULL)
+    {
+    nc_list_mutex = calloc(1, sizeof(pthread_mutex_t));
+    pthread_mutex_init(nc_list_mutex,NULL);
 
     pthread_mutex_lock(nc_list_mutex);
 
@@ -545,12 +539,7 @@ int wait_request(
           continue;
 
         /* clean up SdList and bad sd... */
-
-        pthread_mutex_lock(global_sock_read_mutex);
-        FD_CLR(i, GlobalSocketReadSet);
-        pthread_mutex_unlock(global_sock_read_mutex);
-        pthread_mutex_unlock(svr_conn[i].cn_mutex);
-
+        globalset_del_sock(i);
         } /* END for each socket in global read set */
 
       free(SelectSet);
@@ -564,10 +553,10 @@ int wait_request(
 
   for (i = 0; (i < max_connection) && (n != 0); i++)
     {
-    pthread_mutex_lock(svr_conn[i].cn_mutex);
 
     if (FD_ISSET(i, SelectSet))
       {
+      pthread_mutex_lock(svr_conn[i].cn_mutex);
       /* this socket has data */
       n--;
 
@@ -592,13 +581,10 @@ int wait_request(
         }
       else
         {
-        pthread_mutex_lock(global_sock_read_mutex);
-        FD_CLR(i, GlobalSocketReadSet);
-        pthread_mutex_unlock(global_sock_read_mutex);
-
-        close_conn(i, TRUE);
-
         pthread_mutex_unlock(svr_conn[i].cn_mutex);
+        globalset_del_sock(i);
+        close_conn(i, FALSE);
+
         pthread_mutex_lock(num_connections_mutex);
 
         sprintf(tmpLine, "closed connections to fd %d - num_connections=%d (select bad socket)",
@@ -609,8 +595,6 @@ int wait_request(
         log_err(-1, id, tmpLine);
         }
       }
-    else
-      pthread_mutex_unlock(svr_conn[i].cn_mutex);
     } /* END for i */
 
   free(SelectSet);
@@ -754,6 +738,21 @@ void *accept_conn(
   }  /* END accept_conn() */
 
 
+void globalset_add_sock(
+    int sock)
+  {
+  pthread_mutex_lock(global_sock_read_mutex);
+  FD_SET(sock, GlobalSocketReadSet);
+  pthread_mutex_unlock(global_sock_read_mutex);
+  }
+
+void globalset_del_sock(
+    int sock)
+  {
+  pthread_mutex_lock(global_sock_read_mutex);
+  FD_CLR(sock, GlobalSocketReadSet);
+  pthread_mutex_unlock(global_sock_read_mutex);
+  }
 
 
 /*
@@ -783,9 +782,7 @@ int add_conn(
   num_connections++;
   pthread_mutex_unlock(num_connections_mutex);
 
-  pthread_mutex_lock(global_sock_read_mutex);
-  FD_SET(sock, GlobalSocketReadSet);
-  pthread_mutex_unlock(global_sock_read_mutex);
+  globalset_add_sock(sock);
 
   pthread_mutex_lock(svr_conn[sock].cn_mutex);
 
@@ -831,8 +828,6 @@ int add_conn(
 
 
 
-
-
 /*
  * close_conn - close a network connection
  * does physical close, also marks the connection table
@@ -844,9 +839,12 @@ void close_conn(
   int has_mutex) /* I */
 
   {
-  char log_message[LOG_BUF_SIZE];
+  int rc;
+  char log_message[LOG_BUF_SIZE+1];
   if ((sd < 0) || (max_connection <= sd))
     {
+    snprintf(log_message, LOG_BUF_SIZE, "sd is invalid %d!!!", sd);
+    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_NODE,__func__,log_message);
     return;
     }
 
@@ -855,9 +853,9 @@ void close_conn(
 
   if (svr_conn[sd].cn_active == Idle)
     {
-    if (has_mutex == FALSE)
+    /* apparently we did not add the socket to the connection table */
+    if (has_mutex == FALSE) 
       pthread_mutex_unlock(svr_conn[sd].cn_mutex);
-
     return;
     }
 
@@ -867,7 +865,7 @@ void close_conn(
     {
     snprintf(log_message, LOG_BUF_SIZE, "Connection %d - func %lx",
         sd, (unsigned long)svr_conn[sd].cn_oncl);
-    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_NODE,"close_conn",log_message);
+    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_SERVER,__func__,log_message);
     svr_conn[sd].cn_oncl(sd);
     }
 
@@ -878,22 +876,16 @@ void close_conn(
 
   if (GlobalSocketReadSet != NULL)
     {
-    pthread_mutex_lock(global_sock_read_mutex);
-    FD_CLR(sd, GlobalSocketReadSet);
-    pthread_mutex_unlock(global_sock_read_mutex);
+    globalset_del_sock(sd);
     }
 
-  close(sd);
-
   svr_conn[sd].cn_addr = 0;
-
   svr_conn[sd].cn_handle = -1;
-
   svr_conn[sd].cn_active = Idle;
-
   svr_conn[sd].cn_func = (void *(*)())0;
-
   svr_conn[sd].cn_authen = 0;
+  svr_conn[sd].cn_stay_open = FALSE;
+  rc = close(sd);
     
   if (has_mutex == FALSE)
     pthread_mutex_unlock(svr_conn[sd].cn_mutex);
@@ -970,6 +962,26 @@ pbs_net_t get_connectaddr(
   return(tmp);
   }
 
+void set_localhost_name(char *localhost_name, size_t len)
+  {
+  struct sockaddr sa;
+  int             rc;
+
+  memset(&sa, 0, sizeof(struct sockaddr));
+  sa.sa_family = AF_INET;
+  sa.sa_data[2] = 0x7F;
+  sa.sa_data[5] = 1;
+  rc = getnameinfo(&sa, sizeof(sa), local_host_name, local_host_name_len, NULL, 0, 0);
+  if(rc != 0)
+    {
+    strcpy(local_host_name, "localhost");
+    strncpy(localhost_name, "localhost", len);
+    }
+  else
+    strncpy(localhost_name, local_host_name, len);
+  }
+
+                    
 
 /*
  * get_connecthost - return name of host connected via the socket
@@ -987,18 +999,9 @@ int get_connecthost(
   struct sockaddr_in      addr_in;
   static struct in_addr   serveraddr;
   static char            *server_name = NULL;
-  static pthread_mutex_t *get_connecthost_mutex = NULL;
-
-  if (get_connecthost_mutex == NULL)
-    {
-    get_connecthost_mutex = calloc(1, sizeof(pthread_mutex_t));
-    pthread_mutex_init(get_connecthost_mutex,NULL);
-    }
 
   addr_in.sin_family = AF_INET;
   addr_in.sin_port = 0;
-  pthread_mutex_lock(get_connecthost_mutex);
-
   if ((server_name == NULL) && (pbs_server_addr != 0))
     {
     /* cache local server addr info */
@@ -1036,9 +1039,9 @@ int get_connecthost(
     }
   else
     {
-    if (strcmp(namebuf, "localhost") == 0)
+    if (strcmp(namebuf, local_host_name) == 0)
       {
-      snprintf(namebuf, size, "%s", server_name);
+      snprintf(namebuf, size, "%s", local_host_name);
       }
     else
       {
@@ -1046,9 +1049,6 @@ int get_connecthost(
       }
     }
 
-  /* SUCCESS */
-  pthread_mutex_unlock(get_connecthost_mutex);
-
   return(0);
   }  /* END get_connecthost() */
 
