@@ -86,6 +86,7 @@
 #ifdef _AIX
 #include <arpa/aixrcmds.h>
 #endif /* _AIX */
+#include <pthread.h>
 #include "portability.h"
 #include "list_link.h"
 #include "attribute.h"
@@ -94,8 +95,11 @@
 #include "queue.h"
 #include "pbs_job.h"
 #include "log.h"
+#include "../Liblog/pbs_log.h"
+#include "../Liblog/log_event.h"
 #include "batch_request.h"
 #include "pbs_nodes.h"
+#include "../Libutils/u_lock_ctl.h" /* unlock_node */
 
 #ifndef TRUE
 #define TRUE 1
@@ -107,6 +111,9 @@ extern char *pbs_o_host;
 extern char  server_host[];
 extern char *msg_orighost; /* error message: no PBS_O_HOST */
 
+/* This is in the server code? */
+extern struct pbsnode *find_nodebyname(char *);
+
 /*
  * site_check_u - site_check_user_map()
  *
@@ -123,7 +130,8 @@ int site_check_user_map(
 
   job  *pjob,  /* I */
   char *luser, /* I */
-  char *EMsg)  /* O (optional,minsize=1024) */
+  char *EMsg,  /* O (optional,minsize=1024) */
+  int logging) /* I */
 
   {
   char *orighost;
@@ -138,6 +146,8 @@ int site_check_user_map(
 
   char  *dptr;
 
+  struct pbsnode *tmp;
+
 #ifdef MUNGE_AUTH
   char  uh[PBS_MAXUSER + PBS_MAXHOSTNAME + 2];
 #endif
@@ -209,9 +219,10 @@ int site_check_user_map(
   if ((HostAllowed == 0) &&
       (server.sv_attr[SRV_ATR_AllowNodeSubmit].at_flags & ATR_VFLAG_SET) &&
       (server.sv_attr[SRV_ATR_AllowNodeSubmit].at_val.at_long == 1) &&
-      (find_nodebyname(orighost) != NULL))
+      ((tmp = find_nodebyname(orighost)) != NULL))
     {
     /* job submitted from compute host, access allowed */
+    unlock_node(tmp, "site_check_user_map", NULL, logging);
 
     if (dptr != NULL)
       *dptr = '.';
@@ -266,7 +277,7 @@ int site_check_user_map(
 
 #ifdef MUNGE_AUTH
   sprintf(uh, "%s@%s", owner, orighost);
-  rc = acl_check(&server.sv_attr[SRV_ATR_authusers], uh, ACL_User);
+  rc = acl_check(&server.sv_attr[SRV_ATR_authusers], uh, ACL_User_Host);
   if(rc <= 0)
     {
     /* rc == 0 means we did not find a match.
