@@ -107,6 +107,7 @@ size_t need_to_grow(
   if (ds->size < ds->used + to_add)
     {
     to_grow = to_add + ds->size;
+
     if (to_grow < (ds->size * 4))
       to_grow = ds->size * 4;
     }
@@ -117,6 +118,35 @@ size_t need_to_grow(
 
 
 
+int resize_if_needed(
+
+  dynamic_string *ds,
+  char           *to_check)
+
+  {
+  size_t  new_size = need_to_grow(ds, to_check);
+  size_t  difference;
+  char   *tmp;
+
+  if (new_size > 0)
+    {
+    /* need to resize */
+    difference = new_size - ds->size;
+
+    if ((tmp = realloc(ds->str, new_size)) == NULL)
+      return(ENOMEM);
+
+    ds->str = tmp;
+    /* zero out the new space as well */
+    memset(ds->str + ds->size, 0, difference);
+    ds->size = new_size;
+    }
+
+  return(PBSE_NONE);
+  } /* END resize_if_needed() */
+
+
+
 
 int size_to_dynamic_string(
     
@@ -124,31 +154,16 @@ int size_to_dynamic_string(
   struct size_value  szv)  /* I */
 
   {
-  char    buffer[MAXLINE];
-  size_t  len;
-  size_t  to_grow;
+  char buffer[MAXLINE];
+  int  add_one = FALSE;
+
+  if (ds->used == 0)
+    add_one = TRUE;
+
+  sprintf(buffer, "%lukb", szv.atsv_num);
+  resize_if_needed(ds, buffer);
 
   sprintf(buffer, "%lu", szv.atsv_num);
-  len = 2 + strlen(buffer);
-  if (ds->used == 0)
-    len++;
-
-  if (ds->size - ds->used < len)
-    {
-    char   *tmp;
-
-    if (len > ds->size * 2)
-      to_grow = len;
-    else
-      to_grow = ds->size * 2;
-
-    if ((tmp = realloc(ds->str, to_grow)) == NULL)
-      return(ENOMEM);
-
-    ds->str  = tmp;
-    ds->size = to_grow;
-    }
-
   strcat(ds->str, buffer);
   
   switch (szv.atsv_shift)
@@ -184,7 +199,9 @@ int size_to_dynamic_string(
       break;
     }
 
-  ds->used += len;
+  ds->used += strlen(buffer) + 2;
+  if (add_one == TRUE)
+    ds->used += 1;
 
   return(PBSE_NONE);
   } /* END size_to_dynamic_string() */
@@ -204,33 +221,23 @@ int append_dynamic_string(
   char *to_append)    /* I */
 
   {
-  size_t     to_grow = need_to_grow(ds,to_append);
-  char      *tmp;
-  int        len = strlen(to_append);
+  int len = strlen(to_append);
+  int add_one = FALSE;
+  int offset = ds->used;
 
   if (ds->used == 0)
-    ds->used += 1;
-
-  if (to_grow > 0)
-    {
-    tmp = realloc(ds->str,to_grow);
-
-    if (tmp == NULL)
-      return(ENOMEM);
-
-    strcat(tmp,to_append);
-
-    ds->str  = tmp;
-    ds->size = to_grow;
-    }
+    add_one = TRUE;
   else
-    {
-    /* no need to grow, just append to the string */
-    strcat(ds->str,to_append);
-    }
+    offset -= 1;
+
+  resize_if_needed(ds, to_append);
+  strcat(ds->str + offset, to_append);
     
   ds->used += len;
 
+  if (add_one == TRUE)
+    ds->used += 1;
+
   return(PBSE_NONE);
   } /* END append_dynamic_string() */
 
@@ -255,22 +262,7 @@ int append_dynamic_string_xml(
     {
     /* Resize if needed
      * note - QUOT_ESCAPED_LEN is the most we could be adding right now */
-    if (ds->size - ds->used < QUOT_ESCAPED_LEN)
-      {
-      char   *tmp;
-      size_t  to_grow;
-
-      if (ds->size * 2 < QUOT_ESCAPED_LEN)
-        to_grow = QUOT_ESCAPED_LEN * 10;
-      else
-        to_grow = ds->size * 2;
-
-      if ((tmp = realloc(ds->str, to_grow)) == NULL)
-        return(ENOMEM);
-
-      ds->str  = tmp;
-      ds->size = to_grow;
-      }
+    resize_if_needed(ds, QUOT_ESCAPED);
 
     switch (str[i])
       {
@@ -327,6 +319,8 @@ int append_dynamic_string_xml(
 
 
 
+
+
 /*
  * performs a strcpy at the end of the string (used for node status strings)
  * the difference is this function places strings with their terminating 
@@ -342,28 +336,11 @@ int copy_to_end_of_dynamic_string(
   char           *to_copy)
 
   {
-  size_t  to_grow = need_to_grow(ds,to_copy);
-  char   *tmp;
   int     len = strlen(to_copy) + 1;
 
-  if (to_grow > 0)
-    {
-    tmp = realloc(ds->str,to_grow);
-    
-    if (tmp == NULL)
-      return(ENOMEM);
-
-    strcpy(tmp + ds->used,to_copy);
-
-    ds->str  = tmp;
-    ds->size = to_grow;
-    }
-  else
-    {
-    /* no need to grow, just append to the string */
-    strcpy(ds->str + ds->used,to_copy);
-    }
+  resize_if_needed(ds, to_copy);
     
+  strcpy(ds->str + ds->used,to_copy);
   ds->used += len;
 
   return(PBSE_NONE);
@@ -385,21 +362,17 @@ dynamic_string *get_dynamic_string(
   char *str)          /* I (optional) */
 
   {
-  dynamic_string *ds = malloc(sizeof(dynamic_string));
+  dynamic_string *ds = calloc(1, sizeof(dynamic_string));
 
   if (ds == NULL)
     return(ds);
 
   if (initial_size > 0)
-    {
     ds->size = initial_size;
-    ds->str = malloc(initial_size);
-    }
   else
-    {
     ds->size = DS_INITIAL_SIZE;
-    ds->str = malloc(DS_INITIAL_SIZE);
-    }
+    
+  ds->str = calloc(1, ds->size);
 
   if (ds->str == NULL)
     {
@@ -408,7 +381,6 @@ dynamic_string *get_dynamic_string(
     }
     
   /* initialize empty str */
-  ds->str[0] = '\0';
   ds->used = 0;
 
   /* add the string if it exists */
@@ -427,6 +399,40 @@ dynamic_string *get_dynamic_string(
 
 
 
+int delete_last_word_from_dynamic_string(
+
+  dynamic_string *ds)
+
+  {
+  char *curr;
+
+  if (ds->used <= 0)
+    return(-1);
+
+  /* used is always strlen(str) + 1 + 1 */
+  curr = ds->str + ds->used - 1;
+
+  if ((curr > ds->str) &&
+      (*curr == '\0'))
+    {
+    curr--;
+    ds->used -= 1;
+    }
+
+  /* walk backwards across str until you hit a null terminator */
+  while ((curr >= ds->str) &&
+         (*curr != '\0'))
+    {
+    *curr = '\0';
+    curr--;
+    ds->used -= 1;
+    }
+
+  return(PBSE_NONE);
+  } /* END delete_last_word_from_dynamic_string() */
+
+
+
 
 /* 
  * frees a dynamic string 
@@ -470,3 +476,19 @@ char *get_string(
   } /* END get_string() */
 
 
+
+
+int append_char_to_dynamic_string(
+    
+  dynamic_string *ds,
+  char            c)
+
+  {
+  char str[2];
+  str[0] = c;
+  str[1] = '\0';
+
+  return(append_dynamic_string(ds, str));
+  } /* END append_char_to_dynamic_string() */
+
+
