@@ -5,7 +5,7 @@
 /* height - returns the height of the given node */
 static int height(NodeEntry node)
   {
-  if (node == NULL)
+	if ( node == NULL )
 		return( -1 );
 	else
 		return( node->height );
@@ -83,18 +83,17 @@ static NodeEntry double_rotate_with_right( NodeEntry K1 )
 
 AvlTree AVL_insert( u_long key, uint16_t port, struct pbsnode *node, AvlTree tree )
   {
-  
-  /* If this key is already in the tree do nothing */
-  if (AVL_is_in_tree( key, port, tree ))
-    {
-    return( tree );
+
+	/* If this key is already in the tree do nothing */
+	if (AVL_is_in_tree( key, port, tree ))
+	  {
+	  return( tree );
 	  }
 
 	if (tree == NULL)
-	  {
-	  /* Create and return a node */
-	  tree = ( AvlTree )malloc( sizeof( struct AvlNode ) );
-    if (tree == NULL)
+    {
+    /* Create and return a node */
+    if ((tree = ( AvlTree )calloc(1, sizeof( struct AvlNode ) )) == NULL)
       {
       return( tree );
       }
@@ -215,6 +214,50 @@ int AVL_is_in_tree(u_long key, uint16_t port, AvlTree tree)
   } /* end AVL_is_in_tree */
 
 
+
+
+int AVL_is_in_tree_no_port_compare(
+
+  u_long   key,
+  uint16_t port,
+  AvlTree  tree)
+
+  {
+  if (tree == NULL)
+    return(0);
+
+  if (key < tree->key)
+    return(AVL_is_in_tree_no_port_compare(key,port,tree->left));
+  else if (key > tree->key)
+    return(AVL_is_in_tree_no_port_compare(key,port,tree->right));
+  else
+    return(1);
+  } /* END AVL_is_in_tree_no_port_compare() */
+
+
+
+
+uint16_t AVL_get_port_by_ipaddr(
+    
+  u_long key,
+  AvlTree tree)
+
+  {
+  if (tree == NULL)
+    return(0);
+
+  if (key < tree->key)
+    return(AVL_get_port_by_ipaddr(key,tree->left));
+  else if (key > tree->key)
+    return(AVL_get_port_by_ipaddr(key,tree->right));
+  else
+    return(tree->port);
+  } /* END AVL_get_port_by_ipaddr() */
+
+
+
+
+
 AvlTree AVL_delete_node( 
 
   u_long   key, 
@@ -222,8 +265,8 @@ AvlTree AVL_delete_node(
   AvlTree  tree)
 
   {
-  AvlTree	h1;
-  AvlTree 	h2=NULL;
+  AvlTree	h1 = NULL;
+  AvlTree 	h2 = NULL;
 
   if (tree == NULL)
     return(NULL);
@@ -319,85 +362,97 @@ AvlTree AVL_delete_node(
   return(tree);
   } /*  End delete_node */
 
+
+int AVL_list_add_item(
+
+  AvlTree   tree,
+  char    **r_buf,
+  long     *current_len,
+  long     *max_len)
+
+  {
+  char    *tmp_buf = NULL;
+  char    *current_buffer = NULL;
+  int     add_len = 0;
+	char    tmp_line[32];
+ 
+	/* each entry can be a maximum of 21 bytes plus one for
+	   NULL termination and one for a ','. We need at least 23 bytes to make
+	   this work. (entry format XXX.XXX.XXX.XXX:XXXXX --
+	   This does not work for IPV6 )*/
+	if ((*max_len - *current_len) < 23)
+    {
+    *max_len = *max_len + 1024;
+    tmp_buf = calloc(1, *max_len + 1);
+    if (tmp_buf == NULL)
+      return PBSE_MEM_MALLOC;
+    memcpy(tmp_buf, *r_buf, *current_len);
+    free(*r_buf);
+    *r_buf = tmp_buf;
+    }
+  current_buffer = *r_buf;
+
+	sprintf( tmp_line, "%ld.%ld.%ld.%ld:%d",
+	  (tree->key & 0xFF000000) >> 24,
+    (tree->key & 0x00FF0000) >> 16,
+    (tree->key & 0x0000FF00) >> 8,
+    (tree->key & 0x000000FF),
+    tree->port);
+
+  add_len = strlen(tmp_line);
+	/* r_buf must come in with at least the first byte set to NULL
+	   initially. Every time after that append
+	   a comma */
+	if (current_buffer[0] == 0)
+	  {
+		strcpy(current_buffer, tmp_line);
+    *current_len = *current_len + add_len;
+	  }
+	else
+	  {
+	  strcat(current_buffer, ",");
+	  strcat(current_buffer, tmp_line);
+    *current_len = *current_len + add_len + 1;
+	  }
+  return PBSE_NONE;
+  }
+
 /* AVL_list -- return the key and port values for
  * each entry in the tree in a ',' delimited list
  * returned in Buf 
  * @param T - root of the tree to list 
  * @param Buf - output buffer 
  * @param BufSize - size of the output buffer 
- * return 0 on success 
- * return -1 if Buffer is too small 
- * return 1 if Buf or T are null 
+ * return PBSE_NONE on success 
+ * return PBSE_MEM_MALLOC if malloc failed 
+ * return PBSE_RMNOPARAM if parameters are invalid.
  */ 
-int AVL_list( 
-
-  AvlTree tree, 
-  char *Buf,
-  long BufSize )
-
+int AVL_list( AvlTree tree, char **Buf, long *current_len, long *max_len )
   {
-	long len;
-	char     tmpLine[32];
-	int rc;
+	int rc = PBSE_NONE;
 
-	if ( tree == NULL || Buf == NULL || BufSize == 0 )
+	if (Buf == NULL || *Buf == NULL || current_len == NULL || max_len == NULL)
     {
-    return( 1 );
+    return PBSE_RMNOPARAM;
     }
 
-	len = BufSize;
-
 	/* start down the left side */
 	if ( tree->left != NULL )
     {
-    rc = AVL_list( tree->left, Buf, len );
-    if ( rc )
-      {
-      return( rc );
-      }
-    len -= strlen( Buf );
+		if ((rc = AVL_list(tree->left, Buf, current_len, max_len)) != PBSE_NONE)
+      return rc;
     }
 
+  if ((rc = AVL_list_add_item(tree, Buf, current_len, max_len)) != PBSE_NONE)
+    return rc;
+
 	/* now go right */
-	if (tree->right != NULL )
+	if ( tree->right != NULL )
     {
-    rc = AVL_list( tree->right, Buf, len );
-    if (rc)
-      {
-			return( rc );
-      }
-    len -= strlen( Buf );
+    if ((rc = AVL_list(tree->right, Buf, current_len, max_len)) != PBSE_NONE)
+      return rc;
     }
 
-	/* each entry can be a maximum of 21 bytes plus one for
-	   NULL termination and one for a ','. We need at least 23 bytes to make
-	   this work. (entry format XXX.XXX.XXX.XXX:XXXXX --
-	   This does not work for IPV6 )*/
-	if (len < 23)
-    {
-    return( -1 );
-    }
-
-	sprintf( tmpLine, "%ld.%ld.%ld.%ld:%d",
-			 (tree->key & 0xFF000000) >> 24,
-			 (tree->key & 0x00FF0000) >> 16,
-			 (tree->key & 0x0000FF00) >> 8,
-			 (tree->key & 0x000000FF),
-			 tree->port);
-
-	/* Buf must come in with at least the first byte set to NULL
-	   initially. Every time after that append
-	   a comma */
-	if (Buf[0] == 0)
-	  {
-		strcpy(Buf, tmpLine);
-	  }
-	else
-	  {
-	  strcat(Buf, ",");
-	  strcat(Buf, tmpLine);
-	  }
-
-	return( 0 );
+	return PBSE_NONE;
   } /* end AVL_list */
 
