@@ -137,7 +137,11 @@
 #include <mach/mach_host.h>
 #include <mach/mach_port.h>
 #include <mach/mach_traps.h>
+#ifdef HAVE_MACH_SHARED_REGION_H
+#include <mach/shared_region.h>
+#else
 #include <mach/shared_memory_server.h>
+#endif
 #include <mach/task.h>
 #include <mach/thread_act.h>
 
@@ -153,6 +157,7 @@
 #include "mom_mach.h"
 #include "resmon.h"
 #include "mom_func.h"
+#include "utils.h"
 #include "../rm_dep.h"
 
 #ifndef TRUE
@@ -168,9 +173,9 @@ typedef struct kinfo_proc kinfo_proc;
 ** external functions and data
 */
 
-extern struct  config *search A_((struct config *, char *));
+extern struct  config *search(struct config *, char *);
 
-extern struct  rm_attribute *momgetattr A_((char *));
+extern struct  rm_attribute *momgetattr(char *);
 extern int     rm_errno;
 extern unsigned int  reqnum;
 extern double cputfactor;
@@ -188,8 +193,8 @@ extern  char   *msg_momsetlim;
 
 extern  time_t  time_now;
 
-extern char *loadave A_((struct rm_attribute *attrib));
-extern char *nullproc A_((struct rm_attribute *attrib));
+extern char *loadave(struct rm_attribute *attrib);
+extern char *nullproc(struct rm_attribute *attrib);
 
 extern int errno;
 
@@ -197,19 +202,19 @@ extern int errno;
 ** local functions
 */
 
-static char *resi A_((struct rm_attribute *attrib));
-static char *physmem A_((struct rm_attribute *attrib));
-static char *walltime A_((struct rm_attribute *attrib));
-static char *quota A_((struct rm_attribute *attrib));
-static char *ncpus A_((struct rm_attribute *attrib));
-static char *totmem A_((struct rm_attribute *attrib));
-static char *availmem A_((struct rm_attribute *attrib));
-static char *netload A_((struct rm_attribute *attrib));
-static int   get_tinfo_by_pid  A_((struct task_basic_info *t_info,
-                                   unsigned int pid));
-static int   get_time_info_by_pid  A_((struct task_thread_times_info *t_info,
-                                       unsigned int pid));
-static kinfo_proc *get_bsd_process_list A_((int *procCount));
+static char *resi(struct rm_attribute *attrib);
+static char *physmem(struct rm_attribute *attrib);
+static char *walltime(struct rm_attribute *attrib);
+static char *quota(struct rm_attribute *attrib);
+static char *ncpus(struct rm_attribute *attrib);
+static char *totmem(struct rm_attribute *attrib);
+static char *availmem(struct rm_attribute *attrib);
+static char *netload(struct rm_attribute *attrib);
+static int  get_tinfo_by_pid(struct task_basic_info *t_info,
+                                   unsigned int pid);
+static int  get_time_info_by_pid(struct task_thread_times_info *t_info,
+                                       unsigned int pid);
+static kinfo_proc *get_bsd_process_list(int *procCount);
 
 struct config dependent_config[] =
   {
@@ -256,6 +261,11 @@ char   nokernel[] = "kernel not available";
 char   noproc[] = "process %d does not exist";
 static  int  nncpus = 0;
 static  int  gpagesize;
+static  int  cputype;
+#if defined HAVE_MACH_SHARED_REGION_H
+static  unsigned long  sharedregionbase;
+static  unsigned int  sharedregionsize;
+#endif
 
 
 void
@@ -280,6 +290,31 @@ dep_initialize(void)
     gpagesize = 4096;
     }
 
+  len = sizeof(cputype);
+
+  if (sysctlbyname("hw.cputype", &cputype, &len, NULL, 0) != 0)
+    {
+    cputype = CPU_TYPE_X86;
+    }
+
+#if defined(HAVE_MACH_SHARED_REGION_H)
+  switch (cputype)
+    {
+    case CPU_TYPE_X86:
+      sharedregionbase = SHARED_REGION_BASE_I386; /* same as x86_64? */
+      sharedregionsize = SHARED_REGION_SIZE_I386; /* same as x86_64? */
+    case CPU_TYPE_POWERPC:
+      sharedregionbase = SHARED_REGION_BASE_PPC;
+      sharedregionsize = SHARED_REGION_SIZE_PPC;
+    case CPU_TYPE_POWERPC64:
+      sharedregionbase = SHARED_REGION_BASE_PPC64;
+      sharedregionsize = SHARED_REGION_SIZE_PPC64;
+    default:
+      sharedregionbase = SHARED_REGION_BASE_I386; /* i386 is current standard */
+      sharedregionsize = SHARED_REGION_SIZE_I386; /* i386 is current standard */
+    }
+#endif
+
   return;
   }
 
@@ -323,7 +358,7 @@ dep_main_loop_cycle(void)
  */
 
 static int
-getsize(resource *pres, unsigned long *ret)
+mm_getsize(resource *pres, unsigned long *ret)
   {
   unsigned long value;
 
@@ -359,7 +394,7 @@ getsize(resource *pres, unsigned long *ret)
  */
 
 static int
-gettime(resource *pres, unsigned long *ret)
+mm_gettime(resource *pres, unsigned long *ret)
   {
 
   if (pres->rs_value.at_type != ATR_TYPE_LONG)
@@ -586,8 +621,8 @@ static unsigned long resi_sum(
     memsize += ctob(t_info.resident_size); */
     memsize += t_info.resident_size;
 
-    DBPRT(("%s: pid=%d ses=%d mem=%d totmem=%lu\n",
-           id, pp->kp_proc.p_pid, sess_tbl[i], t_info.resident_size, memsize))
+    DBPRT(("%s: pid=%d ses=%d mem=%ld totmem=%lu\n",
+           id, pp->kp_proc.p_pid, sess_tbl[i], (long)t_info.resident_size, memsize))
     }  /* END for (i) */
 
   return(memsize);
@@ -726,7 +761,7 @@ mom_set_limits(
       if (igncput == FALSE)
         {
         /* cpu time - check, if less than pcput use it */
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -737,7 +772,7 @@ mom_set_limits(
       if (igncput == FALSE)
         {
         /* process cpu time - set */
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -753,7 +788,7 @@ mom_set_limits(
       {
       if (set_mode == SET_LIMIT_SET)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -771,7 +806,7 @@ mom_set_limits(
       {
       if (ignvmem == FALSE)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -786,8 +821,8 @@ mom_set_limits(
         {
         if (set_mode == SET_LIMIT_SET)
           {
-          retval = getsize(pres, &value);
-  
+          retval = mm_getsize(pres, &value);
+
           if (retval != PBSE_NONE)
             return (error(pname, retval));
 
@@ -795,7 +830,7 @@ mom_set_limits(
             return (error(pname, PBSE_BADATVAL));
 
           if ((mem_limit == 0) || (value < mem_limit))
-            mem_limit = value;  
+            mem_limit = value;
           }
         }
       }
@@ -808,7 +843,7 @@ mom_set_limits(
         {
         if (set_mode == SET_LIMIT_SET)
           {
-          retval = getsize(pres, &value);
+          retval = mm_getsize(pres, &value);
 
           if (retval != PBSE_NONE)
             return (error(pname, retval));
@@ -822,7 +857,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "walltime") == 0)   /* Check */
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -940,7 +975,7 @@ int qs_cmp(
   const void *b)
 
   {
-  return((int)((struct kinfo_proc *)a)->kp_eproc.e_paddr - (int)((struct kinfo_proc *)b)->kp_eproc.e_paddr);
+  return((long)((struct kinfo_proc *)a)->kp_eproc.e_paddr - (long)((struct kinfo_proc *)b)->kp_eproc.e_paddr);
   }
 
 
@@ -988,10 +1023,10 @@ static kinfo_proc *get_bsd_process_list(
       err = errno;
       }
 
-    /* if success malloc buffer based on the results of previous call */
+    /* if success calloc buffer based on the results of previous call */
     if (err == 0)
       {
-      result = malloc(length);
+      result = calloc(1, length);
 
       if (result == NULL)
         {
@@ -1165,7 +1200,7 @@ int mom_over_limit(
 
     if ((igncput == FALSE) && (!strcmp(pname, "cput")))
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1180,7 +1215,7 @@ int mom_over_limit(
       }
     else if (!strcmp(pname, "mem"))
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1195,7 +1230,7 @@ int mom_over_limit(
       }
     else if (!strcmp(pname, "vmem"))
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1210,7 +1245,7 @@ int mom_over_limit(
       }
     else if (!strcmp(pname, "pvmem"))
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1228,7 +1263,7 @@ int mom_over_limit(
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
         continue;
 
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -3118,7 +3153,7 @@ static char *quota(
 
   if ((uid = (uid_t)atoi(attrib->a_value)) == 0)
     {
-    if ((pw = getpwnam(attrib->a_value)) == NULL)
+    if ((pw = getpwnam_ext(attrib->a_value)) == NULL)
       {
       sprintf(log_buffer, "user not found: %s", attrib->a_value);
       log_err(-1, id, log_buffer);
@@ -3206,10 +3241,15 @@ static int get_tinfo_by_pid(
 
   vm_size_t     size;
   mach_port_t   object_name;
-  vm_address_t address = GLOBAL_SHARED_TEXT_SEGMENT;
+  vm_address_t address;
   kern_return_t errno;
 
 
+#if defined(HAVE_MACH_SHARED_REGION_H)
+  address = sharedregionbase;
+#else
+  address = GLOBAL_SHARED_TEXT_SEGMENT;
+#endif
 
   if (task_for_pid(mach_task_self(), pid, &task) != KERN_SUCCESS)
     {
@@ -3228,7 +3268,7 @@ static int get_tinfo_by_pid(
     return(errno);
     }
 
-#ifdef VM_REGION_BASIC_INFO_COUNT_64;
+#ifdef VM_REGION_BASIC_INFO_COUNT_64
   if ((errno = vm_region_64(
                  task,
                  &address,
@@ -3267,12 +3307,15 @@ static int get_tinfo_by_pid(
    * We'll modify t_info inline here.
    */
 
+#if! defined(HAVE_MACH_SHARED_REGION_H)
+/* FIXME: What do we need in the mach/shared_region.h case? */
   if (vm_info.reserved &&
       (size == SHARED_TEXT_REGION_SIZE) &&
       (t_info->virtual_size > (SHARED_DATA_REGION_SIZE + SHARED_TEXT_REGION_SIZE)))
     {
     t_info->virtual_size -= (SHARED_DATA_REGION_SIZE + SHARED_TEXT_REGION_SIZE);
     }
+#endif
 
   return(0);
   }
