@@ -1597,7 +1597,37 @@ int move_to_job_cpuset(
 
 
 
+int get_cpuset_size(char *cpusetStr)
+  {
+  int val1 = -1;
+  int val2 = -2;
+  int j;
+  int range = FALSE;
+  int len = 0;
+  char *ptr;
 
+  val1 = atoi(cpusetStr);
+  range = TRUE;
+
+  len = strlen(cpusetStr);
+  for (j=0; j<len; j++)
+    {
+    if (cpusetStr[j] == '-')
+      {
+      range = TRUE;
+      ptr = cpusetStr;
+      ptr += j + 1;
+      val2 = atoi(ptr);
+      break;
+      }
+    }
+  
+  if (val2 != -1)
+    return(val2+1);
+  else
+    return(val1);
+  
+  }
 
 
 /**
@@ -1606,7 +1636,6 @@ int move_to_job_cpuset(
  *
  * @param cpusetStr - (I) the cpuset string
  * @param cpusetMap - (I/O) the cpuset map
- * @param mapSize - (I) the map size
  * @param add - (I) True to add cpuset to map else we remove cpuset from map
  */
 
@@ -1614,7 +1643,6 @@ void adjust_root_map(
 
     char *cpusetStr, /* I */
     int   cpusetMap[], /* I/O */
-    int   mapSize,   /* I */
     int   add)       /* I */
 
   {
@@ -1695,19 +1723,24 @@ void remove_boot_set(
   static char  id[] = "remove_boot_set";
   int          j;
   int          first;
-  int          cpusetMap[MAXPATHLEN];
+  int          *cpusetMap;
+  int          cpuset_size;
   char         tmpBuf[MAXPATHLEN];
 
   if ((rootStr == NULL) ||
       (bootStr == NULL))
     return;
 
-  /* clear out map */
-  for (j=0; j<1024; j++)
+  cpuset_size = get_cpuset_size(rootStr);
+  if (cpuset_size <= 0)
     {
-    cpusetMap[j] = 0;
+    return;
     }
 
+    cpusetMap = (int *)calloc(1, cpuset_size + 1);
+    if(cpusetMap == NULL)
+      return;
+
   if (LOGLEVEL >= 7)
     {
     sprintf(log_buffer,
@@ -1717,15 +1750,15 @@ void remove_boot_set(
     }
 
   /* add the root cpuset to the map */
-  adjust_root_map(rootStr, cpusetMap, 1024, TRUE);
+  adjust_root_map(rootStr, cpusetMap, TRUE);
 
   /* now remove the boot cpuset from the map */
-  adjust_root_map(bootStr, cpusetMap, 1024, FALSE);
+  adjust_root_map(bootStr, cpusetMap, FALSE);
 
   /* convert the cpuset map back into the root cpuset string */
   rootStr[0] = '\0';
   first = TRUE;
-  for (j=0; j<1024; j++)
+  for (j=0; j<cpuset_size; j++)
     {
     if (cpusetMap[j] > 0 )
       {
@@ -1750,6 +1783,9 @@ void remove_boot_set(
     log_ext(-1, id, log_buffer, LOG_DEBUG);
     }
 
+  free(cpusetMap);
+  return;
+
   } /* END remove_boot_set() */
 
 
