@@ -87,8 +87,6 @@
  *    childern structures.
  *   job_purge   purge job from server
  *
- *   job_clone    clones a job (for use with job_arrays)
- *   job_clone_wt work task for cloning a job
  *   job_unlink_file() unlinks a given file using job credentials
  *
  * Include private function:
@@ -136,6 +134,7 @@
 #include "net_connect.h"
 #include "portability.h"
 #include "threadpool.h"
+#include "dis.h"
 
 
 #ifndef TRUE
@@ -224,7 +223,12 @@ void tasks_free(
       ip = (infoent *)GET_NEXT(tp->ti_info);
       }
 
-    close_conn(tp->ti_fd, FALSE);
+    if (tp->ti_chan != NULL)
+      {
+      close_conn(tp->ti_chan->sock, FALSE);
+      DIS_tcp_cleanup(tp->ti_chan);
+      tp->ti_chan = NULL;
+      }
 
     delete_link(&tp->ti_jobtask);
 
@@ -505,7 +509,7 @@ job *job_alloc(void)
  * job_free - free job structure and its various sub-structures
  */
 
-void job_free(
+void mom_job_free(
 
   job *pj)  /* I (modified) */
 
@@ -848,7 +852,7 @@ void job_purge(
 
 #endif   /* IBM SP */
 
-  job_free(pjob);
+  mom_job_free(pjob);
 
   /* if no jobs are left, check if MOM should be restarted */
 
