@@ -76,7 +76,7 @@
 * This license will be governed by the laws of the Commonwealth of Virginia,
 * without reference to its choice of law rules.
 */
-/* $Id: toolkit.c 2367 2008-09-03 17:17:34Z josh $ */
+/* $Id: toolkit.c 5126 2011-09-29 16:30:12Z dbeer $ */
 /*
  *
  * This file contains a variety of routines that should not need to be
@@ -463,8 +463,8 @@ schd_val2byte(char *val)
 
   if (val[0] == '?')
     {
-    (void)sprintf(log_buffer, "error from getreq(physmem): %s: [%d, %d]",
-                  val, pbs_errno, errno);
+    (void)sprintf(log_buffer, "error from getreq(physmem): %s: [%d]",
+                  val, errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     return (0);
     }
