@@ -394,6 +394,8 @@ int set_jobexid(
     }  /* END if (CheckID == 0) */
   else
     {
+    int perm;
+
     if ((attrry + JOB_ATR_userlst)->at_flags & ATR_VFLAG_SET)
       pattr = attrry + JOB_ATR_userlst;
     else
@@ -411,6 +413,8 @@ int set_jobexid(
 
     pwent = getpwnam_ext(puser);
 
+    perm = svr_get_privilege(puser,get_variable(pjob,"PBS_O_HOST"));
+
     if (pwent == NULL)
       {
       snprintf(log_buf,sizeof(log_buf),
@@ -427,7 +431,8 @@ int set_jobexid(
       return(PBSE_BADUSER);
       }
 
-    if (pwent->pw_uid == 0)
+    if ((pwent->pw_uid == 0) ||
+        (perm & ATR_DFLAG_MGWR))
       {
       struct array_strings *pas = NULL;
       get_svr_attr_arst(SRV_ATR_AclRoot, &pas);
@@ -489,7 +494,7 @@ int set_jobexid(
           return(PBSE_BADUSER); /* root not allowed */
           }
         }
-      else
+      else if (pwent->pw_uid == 0)
         {
         if (EMsg != NULL)
           snprintf(EMsg, 1024, "root user %s not allowed",
@@ -498,6 +503,25 @@ int set_jobexid(
         return(PBSE_BADUSER); /* root not allowed */
         }
       }    /* END if (pwent->pw_uid == 0) */
+    else if (pjob->ji_wattr[JOB_ATR_proxy_user].at_flags & ATR_VFLAG_SET)
+      {
+      /* cannot submit a proxy job if not root or a manager */
+      if (EMsg != NULL)
+        {
+        snprintf(EMsg, 1024,
+          "User '%s' is attempting to submit a proxy job for user '%s' but is not a manager",
+          puser,
+          pjob->ji_wattr[JOB_ATR_proxy_user].at_val.at_str);
+        }
+
+      snprintf(log_buf, 1024,
+        "User '%s' is attempting to submit a proxy job for user '%s' but is not a manager",
+        puser,
+        pjob->ji_wattr[JOB_ATR_proxy_user].at_val.at_str);
+      log_err(PBSE_BADUSER, __func__, log_buf);
+
+      return(PBSE_BADUSER);
+      }
 
     if (site_check_user_map(pjob, puser, EMsg, LOGLEVEL) == -1)
       {
