@@ -98,6 +98,7 @@
  */
 
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "job_func.h"
 
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -143,6 +144,8 @@
 #include "resizable_array.h"
 #include "dynamic_string.h"
 #include "svr_func.h" /* get_svr_attr_* */
+#include "req_signal.h" /* issue_signal */
+#include "issue_request.h" /* release_req */
 
 
 #ifndef TRUE
@@ -155,7 +158,6 @@ extern int LOGLEVEL;
 
 
 int conn_qsub(char *, long, char *);
-void job_purge(job *);
 
 /* External functions */
 
@@ -414,8 +416,12 @@ int job_abt(
   int   old_state;
   int   old_substate;
   int   rc = 0;
+  char  job_id[PBS_MAXSVRJOBID+1];
+  long  job_atr_hold;
+  int   job_exit_status;
 
   job  *pjob = *pjobp;
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
 
   /* save old state and update state to Exiting */
 
@@ -465,24 +471,31 @@ int job_abt(
         if (pjob->ji_wattr[JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
           {
           depend_on_term(pjob);
+          pjob = find_job(job_id);
           }
         
         /* update internal array bookeeping values */
-        if ((pjob->ji_arraystruct != NULL) &&
+        if ((pjob != NULL) &&
+            (pjob->ji_arraystruct != NULL) &&
             (pjob->ji_is_array_template == FALSE))
           {
           job_array *pa = get_jobs_array(&pjob);
           
           if (pjob != NULL)
             {
-            update_array_values(pa,pjob,old_state,aeTerminate);
+            job_atr_hold = pjob->ji_wattr[JOB_ATR_hold].at_val.at_long;
+            job_exit_status = pjob->ji_qs.ji_un.ji_exect.ji_exitstat;
+            pthread_mutex_unlock(pjob->ji_mutex);
+            update_array_values(pa,old_state,aeTerminate,
+                job_id, job_atr_hold, job_exit_status);
             
             if (LOGLEVEL >= 7)
               {
-              sprintf(log_buf, "unlocked ai_mutex: %s", myid);
-              log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+              sprintf(log_buf, "unlocking ai_mutex: %s", myid);
+              log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
               }
             pthread_mutex_unlock(pa->ai_mutex);
+            pjob = find_job(job_id);
             }
           }
       
@@ -501,8 +514,8 @@ int job_abt(
     sprintf(log_buf, msg_abt_err,
       pjob->ji_qs.ji_jobid,
       old_substate);
-
     log_err(-1, myid, log_buf);
+    pthread_mutex_unlock(pjob->ji_mutex);
     }
   else
     {
@@ -517,25 +530,33 @@ int job_abt(
 
     if (pjob->ji_wattr[JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
       {
+      strcpy(job_id, pjob->ji_qs.ji_jobid);
       depend_on_term(pjob);
+      pjob = find_job(job_id);
       }
 
     /* update internal array bookeeping values */
-    if ((pjob->ji_arraystruct != NULL) &&
+    if ((pjob != NULL) &&
+        (pjob->ji_arraystruct != NULL) &&
         (pjob->ji_is_array_template == FALSE))
       {
       job_array *pa = get_jobs_array(&pjob);
 
       if (pjob != NULL)
         {
-        update_array_values(pa,pjob,old_state,aeTerminate);
+        job_atr_hold = pjob->ji_wattr[JOB_ATR_hold].at_val.at_long;
+        job_exit_status = pjob->ji_qs.ji_un.ji_exect.ji_exitstat;
+        pthread_mutex_unlock(pjob->ji_mutex);
+        update_array_values(pa,old_state,aeTerminate,
+            job_id, job_atr_hold, job_exit_status);
         
-        pthread_mutex_unlock(pa->ai_mutex);
         if (LOGLEVEL >= 7)
           {
-          sprintf(log_buf, "unlocked ai_mutex: %s", myid);
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+          sprintf(log_buf, "unlocking ai_mutex: %s", myid);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
           }
+        pthread_mutex_unlock(pa->ai_mutex);
+        pjob = find_job(job_id);
         }
       }
 
@@ -669,7 +690,8 @@ job *job_alloc(void)
 
 void job_free(
 
-  job *pj)  /* I (modified) */
+  job *pj,
+  int use_recycle)  /* I (modified) */
 
   {
   int               i;
@@ -709,9 +731,19 @@ void job_free(
    * the lock and then deletes the job, but thread 2 gets the job's lock as
    * the job is freed, causing segfaults. We use the recycler and the 
    * ji_being_recycled flag to solve this problem --dbeer */
-  insert_into_recycler(pj);
+  if (use_recycle)
+    {
+    insert_into_recycler(pj);
+    pthread_mutex_unlock(pj->ji_mutex);
+    }
+  else
+    {
+    pthread_mutex_unlock(pj->ji_mutex);
+    free(pj->ji_mutex);
+    memset(pj, 254, sizeof(job));
+    free(pj);
+    }
 
-  pthread_mutex_unlock(pj->ji_mutex);
 
   return;
   }  /* END job_free() */
@@ -784,7 +816,7 @@ job *job_clone(
   if ((oldid = strdup(template_job->ji_qs.ji_jobid)) == NULL)
     {
     log_err(ENOMEM, __func__, "no memory");
-    job_free(pnewjob);
+    job_free(pnewjob, FALSE);
 
     return(NULL);
     }
@@ -836,7 +868,7 @@ job *job_clone(
             /* FAILURE */
 
             log_err(errno, __func__, "job file is corrupt");
-            job_free(pnewjob);
+            job_free(pnewjob, FALSE);
 
             return(NULL);
             }
@@ -849,7 +881,7 @@ job *job_clone(
         /* FAILURE */
 
         log_err(errno, __func__, "cannot create job file");
-        job_free(pnewjob);
+        job_free(pnewjob, FALSE);
 
         return(NULL);
         }
@@ -945,7 +977,7 @@ job *job_clone(
     pa = get_array(template_job->ji_qs.ji_jobid);
     if (pa == NULL)
       {
-      job_free(pnewjob);
+      job_free(pnewjob, FALSE);
       return(NULL);
       }
     }
@@ -978,9 +1010,9 @@ job *job_clone(
  * job_clone_wt - worktask to clone jobs for job array
  */
 
-void job_clone_wt(
+void *job_clone_wt(
 
-  struct work_task *ptask)
+  void *cloned_id)
 
   {
   char                log_buf[LOCAL_LOG_BUF_SIZE];
@@ -1002,14 +1034,12 @@ void job_clone_wt(
   int                 start;
   int                 end;
 
-  jobid = (char *)(ptask->wt_parm1);
-  free(ptask->wt_mutex);
-  free(ptask);
+  jobid = (char *)cloned_id;
 
   if (jobid == NULL)
     {
     log_err(ENOMEM, __func__, "Can't malloc");
-    return;
+    return NULL;
     }
 
   /* don't call get_jobs_array because the template job isn't part of the array */
@@ -1019,7 +1049,7 @@ void job_clone_wt(
     free(jobid);
     if (template_job != NULL)
       pthread_mutex_unlock(template_job->ji_mutex);
-    return;
+    return NULL;
     }
 
   free(jobid);
@@ -1141,6 +1171,7 @@ void job_clone_wt(
     }
   
   pthread_mutex_unlock(pa->ai_mutex);
+  return NULL;
   }  /* END job_clone_wt */
 
 
@@ -1552,19 +1583,30 @@ int record_jobinfo(
  * removed.
  */
 
-void job_purge(
+int job_purge(
 
   job *pjob)  /* I (modified) */
 
   {
-  static char   id[] = "job_purge";
-
+  int           rc = PBSE_NONE;
   char          log_buf[LOCAL_LOG_BUF_SIZE];
   char          namebuf[MAXPATHLEN + 1];
   extern char  *msg_err_purgejob;
   time_t        time_now = time(NULL);
   long          record_job_info = FALSE;
+  char          job_id[PBS_MAXSVRJOBID+1];
+  char          job_fileprefix[PBS_JOBBASE+1];
+  int           job_substate;
+  int           job_is_array_template;
+  unsigned int  job_has_checkpoint_file;
+  int           job_has_arraystruct;
 
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
+  strcpy(job_fileprefix, pjob->ji_qs.ji_fileprefix);
+  job_substate = pjob->ji_qs.ji_substate;
+  job_is_array_template = pjob->ji_is_array_template;
+  job_has_arraystruct = ((pjob->ji_arraystruct == NULL)?FALSE:TRUE);
+  job_has_checkpoint_file = pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags;
   /* check to see if we are keeping a log of all jobs completed */
   get_svr_attr_l(SRV_ATR_RecordJobInfo, &record_job_info);
   if (record_job_info)
@@ -1580,19 +1622,11 @@ void job_purge(
       }
     }
     
-
-  if ((pjob->ji_qs.ji_substate != JOB_SUBSTATE_TRANSIN) &&
-      (pjob->ji_qs.ji_substate != JOB_SUBSTATE_TRANSICM))
-    {
-    svr_dequejob(pjob, FALSE);
-
-    if (strlen(pjob->ji_qs.ji_jobid) == 0)
-      return;
-    }
+  if ((job_has_arraystruct == FALSE) || (job_is_array_template == TRUE))
+    remove_job(&array_summary,pjob);
 
   /* if part of job array then remove from array's job list */
-  if ((pjob->ji_arraystruct) != NULL &&
-      (pjob->ji_is_array_template == FALSE))
+  if ((job_has_arraystruct == TRUE) && (job_is_array_template == FALSE))
     {
     /* pa->ai_mutex will come out locked after 
        the call to get_jobs_array */
@@ -1619,117 +1653,131 @@ void job_purge(
         pthread_mutex_unlock(pa->ai_mutex);
         if (LOGLEVEL >=7)
           {
-          sprintf(log_buf, "unlocked ai_mutex: %s", id);
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+          sprintf(log_buf, "unlocked ai_mutex: %s", __func__);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
           }
         }
       }
     else
-      return;
+      return PBSE_JOBNOTFOUND;
     }
 
-  if ((pjob->ji_is_array_template == TRUE) ||
-      (pjob->ji_arraystruct == NULL))
-    remove_job(&array_summary,pjob);
+  if ((job_substate != JOB_SUBSTATE_TRANSIN) &&
+      (job_substate != JOB_SUBSTATE_TRANSICM))
+    {
+    pthread_mutex_unlock(pjob->ji_mutex);
+    rc = svr_dequejob(job_id, FALSE);
+    if (rc != PBSE_JOBNOTFOUND)
+      {
+      pthread_mutex_lock(pjob->ji_mutex);
+      job_free(pjob, TRUE);
+      }
+    }
+  else
+    {
+    job_free(pjob, TRUE);
+    }
+
+  /* remove checkpoint restart file if there is one */
+  /* MUTSU - The following functions, updates a flag, saves the job and returns.
+   * As we are deleting the job (and file we are saving) anyway, this is 
+   * superfluous
+   */
+/*  if (pjob->ji_wattr[JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
+    {
+    cleanup_restart_file(pjob);
+    }
+    */
 
   /* delete the script file */
-  if ((pjob->ji_arraystruct == NULL) ||
-      (pjob->ji_is_array_template == TRUE))
+  if ((job_has_arraystruct == FALSE) || (job_is_array_template == TRUE))
     {
     /* delete script file */        
-    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, pjob->ji_qs.ji_fileprefix, JOB_SCRIPT_SUFFIX);
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_SCRIPT_SUFFIX);
 
     if (unlink(namebuf) < 0)
       {
       if (errno != ENOENT)
-        log_err(errno, id, msg_err_purgejob);
+        log_err(errno, __func__, msg_err_purgejob);
       }
     else if (LOGLEVEL >= 6)
       {
       sprintf(log_buf, "removed job script");
 
-      log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+      log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
       }
     }
 
   /* delete any spooled stdout */
-  snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, pjob->ji_qs.ji_fileprefix, JOB_STDOUT_SUFFIX);
+  snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_STDOUT_SUFFIX);
 
   if (unlink(namebuf) < 0)
     {
     if (errno != ENOENT)
-      log_err(errno, id, msg_err_purgejob);
+      log_err(errno, __func__, msg_err_purgejob);
     }
   else if (LOGLEVEL >= 6)
     {
     sprintf(log_buf, "removed job stdout");
 
-    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
     }
 
   /* delete any spooled stderr */
-  snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, pjob->ji_qs.ji_fileprefix, JOB_STDERR_SUFFIX);
+  snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_STDERR_SUFFIX);
 
   if (unlink(namebuf) < 0)
     {
     if (errno != ENOENT)
-      log_err(errno, id, msg_err_purgejob);
+      log_err(errno, __func__, msg_err_purgejob);
     }
   else if (LOGLEVEL >= 6)
     {
     sprintf(log_buf, "removed job stderr");
 
-    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
-    }
-
-  /* remove checkpoint restart file if there is one */
-  if (pjob->ji_wattr[JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
-    {
-    cleanup_restart_file(pjob);
+    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
     }
 
   /* delete checkpoint file directory if there is one */
-  if (pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET)
+  if (job_has_checkpoint_file & ATR_VFLAG_SET)
     {
-    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_checkpoint, pjob->ji_qs.ji_fileprefix, JOB_CHECKPOINT_SUFFIX);
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_checkpoint, job_fileprefix, JOB_CHECKPOINT_SUFFIX);
 
     if (remtree(namebuf) < 0)
       {
       if (errno != ENOENT)
-        log_err(errno, id, msg_err_purgejob);
+        log_err(errno, __func__, msg_err_purgejob);
       }
     else if (LOGLEVEL >= 6)
       {
       sprintf(log_buf, "removed job checkpoint");
 
-      log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+      log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
       }
     }
 
-  if (pjob->ji_is_array_template == TRUE)
+  if (job_is_array_template == TRUE)
     {
-    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, pjob->ji_qs.ji_fileprefix, JOB_FILE_TMP_SUFFIX);
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_FILE_TMP_SUFFIX);
     }
   else
     {
-    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, pjob->ji_qs.ji_fileprefix, JOB_FILE_SUFFIX);
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_FILE_SUFFIX);
     }
 
   if (unlink(namebuf) < 0)
     {
     if (errno != ENOENT)
-      log_err(errno, id, msg_err_purgejob);
+      log_err(errno, __func__, msg_err_purgejob);
     }
   else if (LOGLEVEL >= 6)
     {
     sprintf(log_buf, "removed job file");
 
-    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
     }
 
-  job_free(pjob);
-
-  return;
+  return PBSE_NONE;
   }  /* END job_purge() */
 
 
@@ -1948,34 +1996,32 @@ char *get_correct_jobname(
   return(correct);
   } /* END get_correct_jobname() */
 
-
-
 /*
- * searches the regular job list for the job
+ * Searches the array passed in for the job_id
  */
 
-job *find_job_regular_jobs(
-
-  char *searchable_jobid)
-
+job *find_job_by_array(
+    struct all_jobs *aj,
+    char *job_id,
+    int need_recycled)
   {
   job *pj = NULL;
   int  i;
 
-  pthread_mutex_lock(alljobs.alljobs_mutex);
+  pthread_mutex_lock(aj->alljobs_mutex);
   
-  i = get_value_hash(alljobs.ht, searchable_jobid);
+  i = get_value_hash(aj->ht, job_id);
   
   if (i >= 0)
-    pj = (job *)alljobs.ra->slots[i].item;
+    pj = (job *)aj->ra->slots[i].item;
+  if (pj != NULL)
+    pthread_mutex_lock(pj->ji_mutex);
   
-  pthread_mutex_unlock(alljobs.alljobs_mutex);
+  pthread_mutex_unlock(aj->alljobs_mutex);
   
   if (pj != NULL)
     {
-    pthread_mutex_lock(pj->ji_mutex);
-
-    if (pj->ji_being_recycled == TRUE)
+    if ((pj->ji_being_recycled == TRUE) && (need_recycled == FALSE))
       {
       pthread_mutex_unlock(pj->ji_mutex);
       pj = NULL;
@@ -1983,48 +2029,7 @@ job *find_job_regular_jobs(
     }
 
   return(pj);
-  } /* END find_job_regular_jobs() */
-
-
-
-/* 
- * searches the array list for the job
- */ 
-
-job *find_job_array_jobs(
-
-  char *searchable_jobid)
-
-  {
-  job *pj = NULL;
-  int  i;
-
-  pthread_mutex_lock(array_summary.alljobs_mutex);
-  
-  i = get_value_hash(array_summary.ht, searchable_jobid);
-  
-  if (i >= 0)
-    pj = (job *)array_summary.ra->slots[i].item;
-  
-  pthread_mutex_unlock(array_summary.alljobs_mutex);
-  
-  if (pj != NULL)
-    {
-    pthread_mutex_lock(pj->ji_mutex);
-
-    if (pj->ji_being_recycled == TRUE)
-      {
-      pthread_mutex_unlock(pj->ji_mutex);
-      pj = NULL;
-      }
-    }
-
-  return(pj);
-  } /* END find_job_array_jobs() */
-
-
-
-
+  }
 
 /*
  * find_job() - find job by jobid
@@ -2063,18 +2068,15 @@ job *find_job(
 
   if (strstr(jobid,"[]") == NULL)
     {
-    pj = find_job_regular_jobs(comp);
+    pj = find_job_by_array(&alljobs, comp, FALSE);
+    }
 
-    /* when remotely routing jobs, they are removed from the 
-     * regular job list first and the array summary after. 
-     * Attempt to find them there if NULL */
-    if (pj == NULL)
-      pj = find_job_array_jobs(comp);
-    } /* END if (not an array template job) */
-  else
-    {
-    pj = find_job_array_jobs(comp);
-    } /* END if (job is array template) */
+  /* when remotely routing jobs, they are removed from the 
+   * regular job list first and the array summary after. 
+   * Attempt to find them there if NULL
+   * OR it's an array, try to find the job */
+  if (pj == NULL)
+    pj = find_job_by_array(&array_summary, comp, FALSE);
 
   if (at)
     *at = '@'; /* restore @server_name */
@@ -2389,13 +2391,13 @@ job *next_job_from_back(
   pthread_mutex_lock(aj->alljobs_mutex);
 
   pjob = (job *)next_thing_from_back(aj->ra,iter);
+  if (pjob != NULL)
+    pthread_mutex_lock(pjob->ji_mutex);
 
   pthread_mutex_unlock(aj->alljobs_mutex);
 
   if (pjob != NULL)
     {
-    pthread_mutex_lock(pjob->ji_mutex);
-
     if (pjob->ji_being_recycled == TRUE)
       {
       pthread_mutex_unlock(pjob->ji_mutex);
@@ -2507,27 +2509,8 @@ pbs_queue *get_jobs_queue(
   job **pjob_ptr)
 
   {
-  char       jobid[PBS_MAXSVRJOBID];
   job       *pjob = *pjob_ptr;
-  pbs_queue *pque = pjob->ji_qhdr;
-
-  if (pque != NULL)
-    {
-    if (pthread_mutex_trylock(pque->qu_mutex))
-      {
-      /* if fail */
-      strcpy(jobid, pjob->ji_qs.ji_jobid);
-      pthread_mutex_unlock(pjob->ji_mutex);
-      lock_queue(pque, __func__, NULL, LOGLEVEL);
-
-      if ((pjob = find_job(jobid)) == NULL)
-        {
-        unlock_queue(pque, __func__, NULL, 0);
-        pque = NULL;
-        *pjob_ptr = NULL;
-        }
-      }
-    }
+  pbs_queue *pque = lock_queue_with_job_held(pjob->ji_qhdr, pjob_ptr);
 
   return(pque);
   } /* END get_jobs_queue() */
