@@ -234,8 +234,8 @@ attribute_def node_attr_def[] =
 	  ATR_TYPE_LONG,
 	  PARENT_TYPE_NODE,
 	},
-  /* ND_ATR_num_numa_nodes */
-  { ATTR_NODE_num_numa_nodes, /* "num_numa_nodes" */
+  /* ND_ATR_num_node_boards */
+  { ATTR_NODE_num_node_boards, /* "num_node_boards" */
     decode_l,
     encode_l,
     set_l,
@@ -247,7 +247,7 @@ attribute_def node_attr_def[] =
     PARENT_TYPE_NODE,
   },
   /* ND_ATR_numa_str */
-  { ATTR_NODE_numa_str, /* "numa_node_str" */
+  { ATTR_NODE_numa_str, /* "node_board_str" */
     decode_str,
     encode_str,
     set_str,
@@ -270,4 +270,29 @@ attribute_def node_attr_def[] =
     ATR_TYPE_LONG,
     PARENT_TYPE_NODE,
   },
+  /* ND_ATR_gpus_str */
+  { ATTR_NODE_gpus_str, /* "numa_gpu_node_str" */
+    decode_str,
+    encode_str,
+    set_str,
+    comp_str,
+    free_str,
+    gpu_str_action,
+    NO_USER_SET | ATR_DFLAG_NOSTAT,
+    ATR_TYPE_STR,
+    PARENT_TYPE_NODE,
+  },
+  /* ND_ATR_gpustatus */
+  {ATTR_NODE_gpustatus,		/* "gpu_status" */
+   decode_arst,
+   encode_arst,
+   set_arst,
+   comp_null,
+   free_arst,
+   node_gpustatus_list,
+   MGR_ONLY_SET,
+   ATR_TYPE_ARST,
+   PARENT_TYPE_NODE,
+   },
+
   };
