@@ -123,6 +123,7 @@
 #include "queue_recycler.h" /* queue_recycler */
 #include "svr_task.h" /* initialize_task_recycler */
 #include "svr_func.h" /* get_svr_attr_* */
+#include "job_func.h" /* job_purge */
 
 /*#ifndef SIGKILL*/
 /* there is some weird stuff in gcc include files signal.h & sys/params.h */
@@ -208,6 +209,8 @@ extern int              scheduler_jobct;
 extern pthread_mutex_t *svr_do_schedule_mutex;
 extern pthread_mutex_t *listener_command_mutex;
 extern pthread_mutex_t *node_state_mutex;
+extern pthread_mutex_t *check_tasks_mutex;
+extern pthread_mutex_t *retry_routing_mutex;
 
 extern int a_opt_init;
 
@@ -225,7 +228,6 @@ extern void   acct_close(void);
 
 extern struct work_task *apply_job_delete_nanny(struct job *, int);
 extern int     net_move(job *, struct batch_request *);
-void          job_clone_wt(struct work_task *);
 void          on_job_exit(struct work_task *);
 
 /* Private functions in this file */
@@ -449,6 +451,7 @@ dynamic_string *make_default_hierarchy()
   dynamic_string *default_hierarchy;
   dynamic_string *level_ds;
   int             iter = -1;
+  char            buf[MAXLINE];
 
   if (((default_hierarchy = get_dynamic_string(-1, NULL)) == NULL) ||
       ((level_ds = get_dynamic_string(-1, NULL)) == NULL))
@@ -466,6 +469,15 @@ dynamic_string *make_default_hierarchy()
       append_dynamic_string(level_ds, ",");
 
     append_dynamic_string(level_ds, pnode->nd_name);
+
+    if (PBS_MANAGER_SERVICE_PORT != pnode->nd_mom_rm_port)
+      {
+      snprintf(buf, sizeof(buf), ":%d", (int)pnode->nd_mom_rm_port);
+      append_dynamic_string(level_ds, buf);
+      }
+    
+    pnode->nd_hierarchy_level = 0;
+
     unlock_node(pnode, __func__, NULL, LOGLEVEL);
     }
 
@@ -512,7 +524,8 @@ int can_resolve_hostname(
 
 void check_if_in_nodes_file(
 
-  char *hostname)
+  char *hostname,
+  int   level_index)
 
   {
   char             log_buf[LOCAL_LOG_BUF_SIZE];
@@ -538,6 +551,10 @@ void check_if_in_nodes_file(
     }
     
   pnode->nd_in_hierarchy = TRUE;
+
+  if (pnode->nd_hierarchy_level > level_index)
+    pnode->nd_hierarchy_level = level_index;
+
   unlock_node(pnode, __func__, NULL, LOGLEVEL);
 
   if (colon != NULL)
@@ -551,10 +568,10 @@ void check_if_in_nodes_file(
 int handle_level(
     
   char           *level_iter,
-  dynamic_string *send_format)
+  dynamic_string *send_format,
+  int             level_index)
 
   {
-  char           *id = "handle_level";
   char            log_buf[LOCAL_LOG_BUF_SIZE];
   char           *delims = ",";
   char           *host_tok;
@@ -562,7 +579,7 @@ int handle_level(
 
   if ((level_buf = get_dynamic_string(-1, NULL)) == NULL)
     {
-    log_err(ENOMEM, id, "Cannot allocate memory");
+    log_err(ENOMEM, __func__, "Cannot allocate memory");
     return(ENOMEM);
     }
 
@@ -580,14 +597,14 @@ int handle_level(
       snprintf(log_buf, sizeof(log_buf),
         "While parsing the mom hierarchy file, cannot resolve hostname %s",
         host_tok);
-      log_err(-1, id, log_buf);
+      log_err(-1, __func__, log_buf);
       }
     else
       {
       if (level_buf->used > 0)
         append_dynamic_string(level_buf, ",");
 
-      check_if_in_nodes_file(host_tok);
+      check_if_in_nodes_file(host_tok, level_index);
 
       append_dynamic_string(level_buf, host_tok);
       }
@@ -612,7 +629,6 @@ int handle_path(
   dynamic_string *send_format)
 
   {
-  char *id = "handle_path";
   char  log_buf[LOCAL_LOG_BUF_SIZE];
   char *level_parent;
   char *level_child;
@@ -626,7 +642,7 @@ int handle_path(
     {
     if (!strncmp(level_parent,"level",strlen("level")))
       {
-      handle_level(level_child, send_format);
+      handle_level(level_child, send_format, level_index);
   
       level_index++;
       }
@@ -636,7 +652,7 @@ int handle_path(
       snprintf(log_buf, sizeof(log_buf),
         "Found noise in the mom hierarchy file. Ignoring <%s>%s</%s>",
         level_parent, level_child, level_parent);
-      log_err(-1, id, log_buf);
+      log_err(-1, __func__, log_buf);
       }
     }
   
@@ -735,6 +751,8 @@ dynamic_string *parse_mom_hierarchy(
         snprintf(log_buf, sizeof(log_buf),
           "Node %s found in the nodes file but not in the mom_hierarchy file. Making it a level 1 node",
           pnode->nd_name);
+
+        pnode->nd_hierarchy_level = 0;
         log_err( -1, __func__, log_buf);
         }
 
@@ -790,19 +808,55 @@ dynamic_string *prepare_mom_hierarchy()
 
 
 
+int get_insertion_point(
+
+  struct pbsnode *pnode,
+  int            *indices)
+
+  {
+  int i;
+  int level = pnode->nd_hierarchy_level;
+  int insertion_point = 0;
+
+  for (i = level - 1; i >= 0; i--)
+    {
+    if (indices[i] != 0)
+      {
+      insertion_point = indices[i];
+      break;
+      }
+    }
+
+  return(insertion_point);
+  } /* END get_insertion_point() */
+
+
+
 
 void add_all_nodes_to_hello_container()
 
   {
   struct pbsnode *pnode;
   int             iter = -1;
+  int             level_indices[MAX_LEVEL_DEPTH];
+  int             insertion_index;
   char           *node_name_dup;
 
+  memset(level_indices, 0, sizeof(level_indices));
+
   while ((pnode = next_host(&allnodes, &iter, NULL)) != NULL)
     {
-    node_name_dup = strdup(pnode->nd_name);
-    if (node_name_dup != NULL)
-      add_hello(&hellos, node_name_dup);
+    if ((node_name_dup = strdup(pnode->nd_name)) != NULL)
+      {
+      /* make sure to insert things in order */
+      if (level_indices[pnode->nd_hierarchy_level] == 0)
+        {
+        insertion_index = get_insertion_point(pnode, level_indices);
+        level_indices[pnode->nd_hierarchy_level] = add_hello_after(&hellos, node_name_dup, insertion_index);
+        }
+      else
+        add_hello_after(&hellos, node_name_dup, level_indices[pnode->nd_hierarchy_level]);
+      }
 
     unlock_node(pnode, __func__, NULL, LOGLEVEL);
     }
@@ -1191,6 +1245,12 @@ int pbsd_init(
   svr_do_schedule_mutex = calloc(1, sizeof(pthread_mutex_t));
   pthread_mutex_init(svr_do_schedule_mutex, NULL);
 
+  check_tasks_mutex = calloc(1, sizeof(pthread_mutex_t));
+  pthread_mutex_init(check_tasks_mutex, NULL);
+
+  retry_routing_mutex = calloc(1, sizeof(pthread_mutex_t));
+  pthread_mutex_init(retry_routing_mutex, NULL);
+
   listener_command_mutex = calloc(1, sizeof(pthread_mutex_t));
   pthread_mutex_init(listener_command_mutex, NULL);
 
@@ -1663,7 +1723,7 @@ int pbsd_init(
           (!(pjob->ji_wattr[JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)) &&
           (pjob->ji_qs.ji_svrflags & JOB_SVFLG_SCRIPT))
         {
-        snprintf(basen, sizeof(basen), "%s%s", pjob->ji_qs.ji_jobid, JOB_SCRIPT_SUFFIX);
+        snprintf(basen, sizeof(basen), "%s%s", pjob->ji_qs.ji_fileprefix, JOB_SCRIPT_SUFFIX);
 
         if (chk_save_file(basen) != 0)
           {
@@ -1785,7 +1845,7 @@ int pbsd_init(
         /* TODO Someone must have been naughty and did a kill -9 on pbs_server,
            we might need to validate that the last job was fully initialized
            before continuing the cloning process. */
-        set_task(WORK_Timed, time_now + 1, job_clone_wt, strdup(pa->ai_qs.parent_id), FALSE);
+        enqueue_threadpool_request(job_clone_wt, strdup(pa->ai_qs.parent_id));
         }
 
       }
@@ -1968,13 +2028,14 @@ int pbsd_init_job(
   int  type)  /* I */
 
   {
-  char              id[] = "pbsd_init_job";
   unsigned int      d;
 
   time_t            time_now = time(NULL);
-  char             *jobid_copy;
   char              log_buf[LOCAL_LOG_BUF_SIZE];
   int               local_errno = 0;
+  char  job_id[PBS_MAXSVRJOBID+1];
+  long  job_atr_hold;
+  int   job_exit_status;
 
   pjob->ji_momhandle = -1;
 
@@ -2142,9 +2203,7 @@ int pbsd_init_job(
 
       apply_job_delete_nanny(pjob, time_now + 60);
 
-      jobid_copy = strdup(pjob->ji_qs.ji_jobid);
-
-      set_task(WORK_Immed, 0, on_job_exit, jobid_copy, FALSE);
+      set_task(WORK_Immed, 0, on_job_exit, strdup(pjob->ji_qs.ji_jobid), FALSE);
 
       pbsd_init_reque(pjob, KEEP_STATE);
 
@@ -2153,9 +2212,7 @@ int pbsd_init_job(
     case JOB_SUBSTATE_COMPLETE:
 
       /* Completed jobs are no longer purged on startup */
-      jobid_copy = strdup(pjob->ji_qs.ji_jobid);
-
-      set_task(WORK_Immed, 0, on_job_exit, jobid_copy, FALSE);
+      set_task(WORK_Immed, 0, on_job_exit, strdup(pjob->ji_qs.ji_jobid), FALSE);
 
       pbsd_init_reque(pjob, KEEP_STATE);
 
@@ -2167,14 +2224,20 @@ int pbsd_init_job(
 
         if (pjob != NULL)
           {
-          update_array_values(pa,pjob,JOB_STATE_RUNNING,aeTerminate);
+          strcpy(job_id, pjob->ji_qs.ji_jobid);
+          job_atr_hold = pjob->ji_wattr[JOB_ATR_hold].at_val.at_long;
+          job_exit_status = pjob->ji_qs.ji_un.ji_exect.ji_exitstat;
+          pthread_mutex_unlock(pjob->ji_mutex);
+          update_array_values(pa,JOB_STATE_RUNNING,aeTerminate,
+              job_id, job_atr_hold, job_exit_status);
           
-          pthread_mutex_unlock(pa->ai_mutex);
           if (LOGLEVEL >= 7)
             {
-            sprintf(log_buf, "%s: unlocking ai_mutex", id);
-            log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+            sprintf(log_buf, "%s: unlocking ai_mutex", __func__);
+            log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
             }
+          pthread_mutex_unlock(pa->ai_mutex);
+          pjob = find_job(job_id);
           }
          
         }
@@ -2220,18 +2283,21 @@ int pbsd_init_job(
   /* if job has IP address of Mom, it may have changed */
   /* reset based on hostname                           */
 
-  if ((pjob->ji_qs.ji_un_type == JOB_UNION_TYPE_EXEC) &&
-      (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != 0))
+  if (pjob != NULL)
     {
-    if (pjob->ji_wattr[JOB_ATR_exec_host].at_flags & ATR_VFLAG_SET)
+    if ((pjob->ji_qs.ji_un_type == JOB_UNION_TYPE_EXEC) &&
+        (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != 0))
       {
-      char *tmp = parse_servername(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, &d);
-      pjob->ji_qs.ji_un.ji_exect.ji_momaddr = get_hostaddr(&local_errno, tmp);
-      free(tmp);
-      }
-    else
-      {
-      pjob->ji_qs.ji_un.ji_exect.ji_momaddr = 0;
+      if (pjob->ji_wattr[JOB_ATR_exec_host].at_flags & ATR_VFLAG_SET)
+        {
+        char *tmp = parse_servername(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, &d);
+        pjob->ji_qs.ji_un.ji_exect.ji_momaddr = get_hostaddr(&local_errno, tmp);
+        free(tmp);
+        }
+      else
+        {
+        pjob->ji_qs.ji_un.ji_exect.ji_momaddr = 0;
+        }
       }
     }
 
@@ -2271,7 +2337,7 @@ void pbsd_init_reque(
     }
 
   pthread_mutex_lock(server.sv_qs_mutex);
-  if (svr_enquejob(pjob, TRUE, -1) == 0)
+  if (svr_enquejob(pjob, TRUE, -1) == PBSE_NONE)
     {
     strcat(logbuf, msg_init_queued);
     strcat(logbuf, pjob->ji_qs.ji_queue);
