@@ -101,6 +101,7 @@
 #include <time.h>
 #include <unistd.h> /* getpid */
 #include <stdlib.h>
+#include <pthread.h>
 #include "list_link.h"
 #include "work_task.h"
 #include "log.h"
@@ -125,13 +126,19 @@
 #include "batch_request.h"
 #include "pbs_proto.h"
 #include "u_tree.h"
-#include <pthread.h>
+#include "utils.h"
 #include "threadpool.h"
 #include "../lib/Libutils/u_lock_ctl.h" /* lock_init */
 #include "svr_func.h" /* get_svr_attr_* */
 #include "../lib/Libifl/lib_ifl.h" /* get_port_from_server_name_file */
 #include "node_manager.h" /* svr_is_request */
+#include "../lib/Libnet/lib_net.h" /* start_listener_addrinfo */
+#include "process_request.h" /* process_request */
+#include "net_connect.h" /* set_localhost_name */
+#include "tcp.h" /* tcp_chan */
 
+#define TASK_CHECK_INTERVAL    10
+#define RETRY_ROUTING_INTERVAL 45
 #define HELLO_WAIT_TIME        600
 #define TSERVER_HA_CHECK_TIME  1  /* 1 second sleep time between checks on the lock file for high availability */
 
@@ -200,6 +207,8 @@ void          restore_attr_default (struct attribute *);
 
 /* Global Data Items */
 
+time_t                  last_routing_retry_time = 0;
+time_t                  last_task_check_time = 0;
 int                     lockfds = -1;
 int                     ForceCreation = FALSE;
 int                     high_availability_mode = FALSE;
@@ -261,6 +270,8 @@ int                     server_init_type = RECOV_WARM;
 char                    server_name[PBS_MAXSERVERNAME + 1]; /* host_name[:service|port] */
 int                     svr_do_schedule = SCH_SCHEDULE_NULL;
 pthread_mutex_t        *svr_do_schedule_mutex;
+pthread_mutex_t        *check_tasks_mutex;
+pthread_mutex_t        *retry_routing_mutex;
 extern all_queues       svr_queues;
 extern int              listener_command;
 extern hello_container  hellos;
@@ -285,7 +296,12 @@ int                     allow_any_mom = FALSE;
 int                     array_259_upgrade = FALSE;
 
 
+/* Globals to thread the accept port */
+pthread_t      accept_thread_id = -1;
 
+
+char server_localhost[PBS_MAXHOSTNAME + 1];
+size_t localhost_len = PBS_MAXHOSTNAME;
 /*
  * need_y_response - on create/clean initialization that would delete
  * information, obtain the operator approval first.
@@ -363,32 +379,32 @@ static void need_y_response(
 
 
 
-void *process_pbs_server_port(
-     
-  void *new_sock)
- 
+int process_pbs_server_port(
+  int sock)
   {
   int          proto_type;
-  int          rc;
+  int          rc = PBSE_NONE;
   int          version;
   char         log_buf[LOCAL_LOG_BUF_SIZE];
-  int sock = *(int *)new_sock;
+  struct tcp_chan *chan = NULL;
   
-  DIS_tcp_setup(sock);
-  
-  proto_type = disrui_peek(sock,&rc);
+  if ((chan = DIS_tcp_setup(sock)) == NULL)
+    {
+    }
+
+  proto_type = disrui_peek(chan,&rc);
   
   switch(proto_type)
     {
     case PBS_BATCH_PROT_TYPE:
       
-      process_request(new_sock);
+      rc = process_request(chan);
       
       break;
       
     case IS_PROTOCOL:
 
-      version = disrsi(sock,&rc);
+      version = disrsi(chan,&rc);
       
       if (rc != DIS_SUCCESS)
         {
@@ -397,7 +413,7 @@ void *process_pbs_server_port(
         break;
         }
       
-      svr_is_request(sock,version,NULL);
+      rc = svr_is_request(chan,version);
       
       break;
 
@@ -421,15 +437,48 @@ void *process_pbs_server_port(
         log_err(-1, __func__, log_buf);
         }
 
-      close_conn(sock, FALSE);
+      /*close_conn(sock, FALSE);*/
+      rc = PBSE_SOCKET_DATA;
 
       break;
       }
     }
+  if (chan != NULL)
+    DIS_tcp_cleanup(chan);
 
-  return(NULL);
+  return rc;
   }  /* END process_pbs_server_port() */
 
+/* process_pbs_server_port_scheduler
+ * This function is a wrapper for process_pbs_server_port
+ * whose signature has been changed to make torques
+ * accept process fully multithreaded. 
+ * This will be run from wait request after being scheduled
+ * by contact_sched
+ */
+void *process_pbs_server_port_scheduler(void *new_sock)
+  {
+  int sock = *(int *)new_sock;
+
+  process_pbs_server_port(sock);
+  return(NULL);
+  }
+
+void *start_process_pbs_server_port(void *new_sock)
+  {
+  int sock = *(int *)new_sock;
+  int rc = PBSE_NONE;
+
+  free(new_sock);
+  while ((rc != PBSE_SOCKET_DATA) && (rc != PBSE_SOCKET_INFORMATION) 
+        && (rc != PBSE_INTERNAL) && (rc != PBSE_SYSTEM)  && (rc != PBSE_SOCKET_CLOSE))
+    {
+    rc = process_pbs_server_port(sock);
+    }
+  close_conn(sock, FALSE);
+  /* Thread exit */
+  return(NULL);
+  }
 
 
 
@@ -453,7 +502,7 @@ void clear_listeners(void)   /* I */
 
 int add_listener(
 
-  pbs_net_t l_addr,  /* I */
+  pbs_net_t    l_addr,  /* I */
   unsigned int l_port)  /* I */
 
   {
@@ -519,7 +568,7 @@ int PBSShowUsage(
 
 void parse_command_line(
     
-  int argc,
+  int   argc,
   char *argv[])
 
   {
@@ -926,29 +975,23 @@ void parse_command_line(
  * Returns: amount of time till next task
  */
 
-static time_t check_tasks()
+void *check_tasks()
 
   {
-  time_t     delay;
-
   work_task *ptask;
-  long       til = 0;
-  time_t     tilwhen;
   int        iter = -1;
 
-  time_t     time_now = time(NULL);
+  time_t     time_now;
 
-  get_svr_attr_l(SRV_ATR_scheduler_iteration, &til);
-  tilwhen = til;
-  iter = -1;
+  pthread_mutex_lock(check_tasks_mutex);
 
-  while ((ptask = next_task(&task_list_timed,&iter)) != NULL)
+  time_now = time(NULL);
+  last_task_check_time = time_now;
+
+  while ((ptask = next_task(&task_list_timed, &iter)) != NULL)
     {
-    if ((delay = ptask->wt_event - time_now) > 0)
+    if (ptask->wt_event - time_now > 0)
       {
-      if (tilwhen > delay)
-        tilwhen = delay;
-
       pthread_mutex_unlock(ptask->wt_mutex);
 
       break;
@@ -959,9 +1002,8 @@ static time_t check_tasks()
       }
     }
 
-  /* should the scheduler be run?  If so, adjust the delay time  */
-
-  if ((delay = server.sv_next_schedule - time_now) <= 0)
+  /* should the scheduler be run?  If so, adjust the schedule time  */
+  if (server.sv_next_schedule - time_now <= 0)
     {
     pthread_mutex_lock(svr_do_schedule_mutex);
     svr_do_schedule = SCH_SCHEDULE_TIME;
@@ -971,10 +1013,10 @@ static time_t check_tasks()
     listener_command = SCH_SCHEDULE_TIME;
     pthread_mutex_unlock(listener_command_mutex);
     }
-  else if (delay < tilwhen)
-    tilwhen = delay;
+  
+  pthread_mutex_unlock(check_tasks_mutex);
 
-  return(tilwhen);
+  return(NULL);
   }  /* END check_tasks() */
 
 
@@ -1038,6 +1080,94 @@ void send_any_hellos_needed()
 
 
 
+void *handle_queue_routing_retries(
+
+  void *vp)
+
+  {
+  pbs_queue *pque;
+  int        iter = -1;
+  time_t     time_now = time(NULL);
+
+  pthread_mutex_lock(retry_routing_mutex);
+
+  if (time_now - last_routing_retry_time > RETRY_ROUTING_INTERVAL)
+    {
+    last_routing_retry_time = time_now;
+    
+    while ((pque = next_queue(&svr_queues, &iter)) != NULL)
+      {
+      if (pque->qu_qs.qu_type == QTYPE_RoutePush)
+        queue_route(pque);
+      
+      unlock_queue(pque, __func__, NULL, 0);
+      }
+    }
+  
+  pthread_mutex_unlock(retry_routing_mutex);
+
+  return(NULL);
+  } /* END handle_queue_routing_retries() */
+
+
+
+
+void *handle_scheduler_contact(
+
+  void *vp)
+
+  {
+  schedule_jobs();
+  notify_listeners();
+
+  return(NULL);
+  } /* END handle_scheduler_contact() */
+
+
+
+void *start_accept_listener()
+  {
+  char server_name_trimmed[PBS_MAXSERVERNAME + 1];
+  char *colon_pos = NULL;
+  colon_pos = strchr(server_name, ':');
+  if (colon_pos == NULL)
+    strcpy(server_name_trimmed, server_name);
+  else
+    strncpy(server_name_trimmed, server_name, colon_pos - server_name);
+  start_listener_addrinfo(server_name_trimmed, pbs_server_port_dis, start_process_pbs_server_port);
+  return NULL;
+  }
+
+
+void start_accept_thread()
+  {
+  pthread_attr_t accept_attr;
+  if ((pthread_attr_init(&accept_attr)) != 0)
+    {
+    perror("pthread_attr_init failed. Could not start accept thread");
+    log_err(-1, msg_daemonname,"pthread_attr_init failed. Could not start accept thread");
+    }
+  else if ((pthread_attr_setdetachstate(&accept_attr, PTHREAD_CREATE_DETACHED) != 0))
+    {
+    perror("pthread_attr_setdetatchedstate failed. Could not start accept thread");
+    log_err(-1, msg_daemonname,"pthread_attr_setdetachedstate failed. Could not start accept thread");
+    }
+  else if ((pthread_create(&accept_thread_id, &accept_attr, start_accept_listener, NULL)) != 0)
+    {
+    perror("could not start listener for pbs_server");
+    log_err(-1, msg_daemonname, "Failed to start listener for pbs_server");
+    }
+  }
+
+
+void monitor_accept_thread()
+  {
+  if (pthread_kill(accept_thread_id, 0) == ESRCH)
+    {
+    start_accept_thread();
+    }
+  }
+
 
 
 void main_loop(void)
@@ -1045,15 +1175,13 @@ void main_loop(void)
   {
   int            c;
   long          state = SV_STATE_DOWN;
-  time_t        waittime;
-  pbs_queue    *pque;
+  time_t        waittime = 5;
   job          *pjob;
   int           iter;
   time_t        last_jobstat_time;
   long          when = 0;
   long          timeout = 0;
   long          log = 0;
-  long          poll_jobs = FALSE;
   long          scheduling = FALSE;
   long          sched_iteration = 0;
   time_t        time_now = time(NULL);
@@ -1115,9 +1243,6 @@ void main_loop(void)
 
   get_svr_attr_l(SRV_ATR_State, &state);
 
-  get_svr_attr_l(SRV_ATR_tcp_timeout, &timeout);
-  DIS_tcp_settimeout(timeout);
-
   if (server_init_type == RECOV_HOT)
     state = SV_STATE_HOT;
   else
@@ -1143,27 +1268,39 @@ void main_loop(void)
   if (wait_for_moms_hierarchy == TRUE)
     try_hellos = time_now + HELLO_WAIT_TIME;
 
+  start_accept_thread();
+
   while (state != SV_STATE_DOWN)
     {
     /* first process any task whose time delay has expired */
     last_jobstat_time = time_now = time(NULL);
 
+    monitor_accept_thread();
+
     if (try_hellos <= time_now)
       send_any_hellos_needed();
 
-    get_svr_attr_l(SRV_ATR_PollJobs, &poll_jobs);
+    if (time_now - last_task_check_time > TASK_CHECK_INTERVAL)
+      enqueue_threadpool_request(check_tasks, NULL);
 
-    if (poll_jobs)
-      waittime = MIN(check_tasks(), JobStatRate - (time_now - last_jobstat_time));
-    else
-      waittime = check_tasks();
+    get_svr_attr_l(SRV_ATR_tcp_timeout, &timeout);
+
+    /* don't allow timeouts to go below 300 seconds - this is a safety
+     * net for an extremely rare error */
+    if (timeout < 300)
+      {
+      snprintf(log_buf, sizeof(log_buf), "tcp timeout was %ld resetting to 300",
+        timeout);
+      timeout = 300;
+      log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, msg_daemonname, log_buf);
+      }
+
+    DIS_tcp_settimeout(timeout);
 
     waittime = MAX(1, waittime);
 
     if (state == SV_STATE_RUN)
       {
-      /* In normal Run state */
-
       /* if time or event says to run scheduler, do it */
       get_svr_attr_l(SRV_ATR_scheduling, &scheduling);
       get_svr_attr_l(SRV_ATR_scheduler_iteration, &sched_iteration);
@@ -1177,8 +1314,7 @@ void main_loop(void)
 
         server.sv_next_schedule = time_now + sched_iteration;
 
-        schedule_jobs();
-        notify_listeners();
+        enqueue_threadpool_request(handle_scheduler_contact, NULL);
         }
       else
         {
@@ -1200,8 +1336,8 @@ void main_loop(void)
         }
 
       /* If more than _LIMIT seconds since start, stop */
-
-      if ((c == 0) || (time_now > server.sv_started + SVR_HOT_LIMIT))
+      if ((c == 0) || 
+          (time_now > server.sv_started + SVR_HOT_LIMIT))
         {
         server_init_type = RECOV_WARM;
 
@@ -1210,20 +1346,9 @@ void main_loop(void)
         }
       }
 
-    iter = -1;
-
-    /* any jobs to route today */
-/*    sprintf(log_buf, "num of queues in svr_queues = %d", svr_queues.ra->num);
-    log_err(-1, __func__, log_buf);
-    */
-
-    while ((pque = next_queue(&svr_queues, &iter)) != NULL)
-      {
-      if (pque->qu_qs.qu_type == QTYPE_RoutePush)
-        queue_route(pque);
-
-      unlock_queue(pque, "main_loop", NULL, LOGLEVEL);
-      } 
+    /* retry routing as needed */
+    if (time_now - last_routing_retry_time > RETRY_ROUTING_INTERVAL)
+      enqueue_threadpool_request(handle_queue_routing_retries, NULL);
 
 #ifdef NO_SIGCHLD
     pthread_mutex_lock(server.sv_qs_mutex);
@@ -1280,7 +1405,10 @@ void main_loop(void)
     pthread_mutex_unlock(server.sv_jobstates_mutex);
 
     get_svr_attr_l(SRV_ATR_State, &state);
+    usleep(100);
     }    /* END while (*state != SV_STATE_DOWN) */
+  
+  pthread_cancel(accept_thread_id);
 
   svr_save(&server, SVR_SAVE_FULL); /* final recording of server */
 
@@ -1408,7 +1536,7 @@ int main(
   extern char *msg_startup1; /* log message   */
 
   ProgName = argv[0];
-  srand((unsigned int)getpid() + (unsigned int)time(NULL));
+  srand(get_random_number());
   tzset(); /* localtime_r needs this */
 
   memset(&server, 0, sizeof(struct server));
@@ -1656,6 +1784,7 @@ int main(
     msg_daemonname,
     log_buf);
 
+  set_localhost_name(server_localhost, localhost_len);
   /* initialize the server objects and perform specified recovery */
   /* will be left in the server's private directory  */
   /* NOTE:  env cleared in pbsd_init() */
@@ -1681,16 +1810,8 @@ int main(
     msg_daemonname,
     log_buf);
 
-  if (init_network(pbs_server_port_dis, process_pbs_server_port) != 0)
-    {
-    perror("pbs_server: network");
 
-    log_err(-1, msg_daemonname, "init_network failed dis");
-
-    exit(3);
-    }
-
-  if (init_network(0, process_pbs_server_port) != 0)
+  if (init_network(0, start_process_pbs_server_port) != 0)
     {
     perror("pbs_server: unix domain socket");
 
@@ -1699,6 +1820,7 @@ int main(
     exit(3);
     }
 
+
 #if (PLOCK_DAEMONS & 1)
   plock(PROCLOCK);
 
@@ -2004,7 +2126,7 @@ char *extract_dir(
 
   Dir[0] = '\0';
 
-  snprintf(Dir,sizeof(Dir),"%s",FullPath);
+  snprintf(Dir,DirSize,"%s",FullPath);
 
   ptr = strrchr(Dir,'/');
 
