@@ -78,7 +78,7 @@
 */
 
 #include <pbs_config.h>   /* the master config generated by configure */
-
+#include "process_request.h"
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -95,6 +95,7 @@
 #include <pthread.h>
 
 #include "libpbs.h"
+#include "../lib/Libifl/lib_ifl.h" /* netaddr_long */
 #include "pbs_error.h"
 #include "server_limits.h"
 #include "pbs_nodes.h"
@@ -115,10 +116,34 @@
 #include "threadpool.h"
 #include "dis.h"
 #include "array.h"
+#include "req_stat.h"
 #include "../lib/Libutils/u_lock_ctl.h" /* lock_node, unlock_node */
+#include "../lib/Libnet/lib_net.h" /* globalset_del_sock */
 #include "svr_func.h" /* get_svr_attr_* */
 #include "req_gpuctrl.h" /* req_gpuctrl_svr */
-
+#include "req_getcred.h" /* req_altauthenuer */ 
+#include "req_quejob.h" /* req_quejob, req_jobcredential, req_mvjobfile */ 
+#include "req_holdjob.h" /* req_holdjob, req_checkpointjob */ 
+#include "req_holdarray.h" /* req_holdarray */ 
+#include "req_stat.h" /* req_stat_node */ 
+#include "req_jobobit.h" /* req_jobobit */ 
+#include "req_runjob.h" /* req_runjob, req_stagein */ 
+#include "req_deletearray.h" /* req_deletearray */ 
+#include "req_delete.h" /* req_deletejob */ 
+#include "req_movejob.h" /* req_orderjob */ 
+#include "req_modify.h" /* req_modifyjob, req_modifyarray */ 
+#include "req_track.h" /* req_track */ 
+#include "req_rescq.h" /* req_rescreserve, req_rescfree */ 
+#include "req_shutdown.h" /* req_shutdown */
+#include "req_signal.h" /* req_signaljob */
+#include "req_message.h" /* req_messagejob */
+#include "req_locate.h" /* req_locatejob */
+#include "req_manager.h" /* req_manager */
+#include "req_rerun.h" /* req_rerunjob */
+#include "req_select.h" /* req_selectjobs */
+#include "req_register.h" /* req_register, req_registerarray */
+#include "job_func.h" /* job_purge */
+#include "tcp.h" /* tcp_chan */
 
 /*
  * process_request - this function gets, checks, and invokes the proper
@@ -163,7 +188,6 @@ static const int munge_on = 0;
 
 static void freebr_manage(struct rq_manage *);
 static void freebr_cpyfile(struct rq_cpyfile *);
-static void close_quejob(int sfds);
 static void free_rescrq(struct rq_rescq *);
 
 /* END private prototypes */
@@ -171,36 +195,6 @@ static void free_rescrq(struct rq_rescq *);
 
 extern struct pbsnode *PGetNodeFromAddr(pbs_net_t);
 
-/* request processing prototypes */
-void *req_jobcredential(void *preq);
-void *req_quejob(void *preq);
-void *req_rdytocommit(void *preq);
-void *req_jobscript(void *preq);
-void *req_commit(void *preq);
-void *req_holdjob(void *preq);
-void *req_holdarray(void *preq);
-void *req_stat_node(void *vp);
-void *req_jobobit(void *preq);
-void *req_stagein(void *preq);
-void *req_deletearray(void *preq);
-void *req_deletejob(void *preq);
-void *req_orderjob(void *preq);
-void *req_modifyarray(void *preq);
-void *req_modifyjob(void *preq);
-
-void *req_track(void *preq);
-void *req_rescreserve(void *preq);
-void *req_rescfree(void *preq);
-
-
-void  req_shutdown(struct batch_request *preq);
-void *req_signaljob(void *preq);
-void *req_mvjobfile(void *preq);
-void *req_checkpointjob(void *preq);
-void *req_messagejob(void *preq);
-
-/* END request processing prototypes */
-
 
 #ifdef ENABLE_UNIX_SOCKETS
 int get_creds(
@@ -313,13 +307,12 @@ int get_creds(
  * NOTE: the caller functions hold the mutex for the connection
  */
 
-void *process_request(
+int process_request(
 
-  void *new_sock) /* file descriptor (socket) to get request */
+  struct tcp_chan *chan) /* file descriptor (socket) to get request */
 
   {
-  int                   rc;
-  
+  int                   rc = PBSE_NONE;
   struct batch_request *request = NULL;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
   long                  acl_enable = FALSE;
@@ -327,18 +320,29 @@ void *process_request(
 
   time_t                time_now = time(NULL);
   int                   free_request = TRUE;
-  int                   sfds = *(int *)new_sock;
   char                  tmpLine[MAXLINE];
-  int                   unlock_mutex = TRUE;
   char                 *auth_err = NULL;
+  enum conn_type        conn_active;
+  unsigned short        conn_socktype;
+  unsigned short        conn_authen;
+  unsigned long         conn_addr;
+  int sfds = chan->sock;
+
+  pthread_mutex_lock(svr_conn[sfds].cn_mutex);
+  conn_active = svr_conn[sfds].cn_active;
+  conn_socktype = svr_conn[sfds].cn_socktype;
+  conn_authen = svr_conn[sfds].cn_authen;
+  conn_addr = svr_conn[sfds].cn_addr;
+  pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
 
   if ((request = alloc_br(0)) == NULL)
     {
     snprintf(tmpLine, sizeof(tmpLine),
         "cannot allocate memory for request from %lu",
-        get_connectaddr(sfds,FALSE));
-    req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
+        conn_addr);
+    req_reject(PBSE_MEM_MALLOC, 0, request, NULL, tmpLine);
     free_request = FALSE;
+    rc = PBSE_SYSTEM;
     goto process_request_cleanup;
     }
 
@@ -347,33 +351,35 @@ void *process_request(
   /*
    * Read in the request and decode it to the internal request structure.
    */
-
-  pthread_mutex_lock(svr_conn[sfds].cn_mutex);
-
-  if (svr_conn[sfds].cn_active == FromClientDIS)
+  if (conn_active == FromClientDIS || conn_active == ToServerDIS)
     {
 #ifdef ENABLE_UNIX_SOCKETS
 
-    if ((svr_conn[sfds].cn_socktype & PBS_SOCK_UNIX) &&
-        (svr_conn[sfds].cn_authen != PBS_NET_CONN_AUTHENTICATED))
+    if ((conn_socktype & PBS_SOCK_UNIX) &&
+        (conn_authen != PBS_NET_CONN_AUTHENTICATED))
       {
+      /* get_creds interestingly always returns 0 */
       get_creds(sfds, conn_credent[sfds].username, conn_credent[sfds].hostname);
       }
 
 #endif /* END ENABLE_UNIX_SOCKETS */
-    rc = dis_request_read(sfds, request);
+    rc = dis_request_read(chan, request);
     }
   else
     {
+    char out[80];
+
+    snprintf(tmpLine, MAXLINE, "request on invalid type of connection: %d, sock type: %d, from address %s", 
+                conn_active,conn_socktype, netaddr_long(conn_addr, out));
     log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_REQUEST,
-      "process_req", "request on invalid type of connection");
+      "process_req", tmpLine);
     snprintf(tmpLine, sizeof(tmpLine),
-        "request on invalid type of connection from%lu",
-        get_connectaddr(sfds,FALSE));
-    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-    unlock_mutex = FALSE;
+        "request on invalid type of connection (%d) from %s",
+        conn_active,
+        netaddr_long(conn_addr, out));
     req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
     free_request = FALSE;
+    rc = PBSE_BADHOST;
     goto process_request_cleanup;
     }
 
@@ -381,10 +387,11 @@ void *process_request(
     {
     /* FAILURE */
     /* premature end of file */
+    rc = PBSE_PREMATURE_EOF;
     goto process_request_cleanup;
     }
 
-  if ((rc == PBSE_SYSTEM) || (rc == PBSE_INTERNAL))
+  if ((rc == PBSE_SYSTEM) || (rc == PBSE_INTERNAL) || (rc == PBSE_SOCKET_CLOSE))
     {
     /* FAILURE */
     /* read error, likely cannot send reply so just disconnect */
@@ -401,8 +408,6 @@ void *process_request(
      * request type, in either case, return reject-reply
      */
 
-    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-    unlock_mutex = FALSE;
     req_reject(rc, 0, request, NULL, "cannot decode message");
     free_request = FALSE;
     goto process_request_cleanup;
@@ -412,18 +417,17 @@ void *process_request(
     {
     sprintf(log_buf, "%s: %lu",
       pbse_to_txt(PBSE_BADHOST),
-      get_connectaddr(sfds,FALSE));
+      conn_addr);
 
     log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_REQUEST, "", log_buf);
 
     snprintf(tmpLine, sizeof(tmpLine),
         "cannot determine hostname for connection from %lu",
-        get_connectaddr(sfds,FALSE));
+        conn_addr);
 
-    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-    unlock_mutex = FALSE;
     req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
     free_request = FALSE;
+    rc = PBSE_BADHOST;
     goto process_request_cleanup;
     }
 
@@ -441,7 +445,7 @@ void *process_request(
 
   /* is the request from a host acceptable to the server */
 
-  if (svr_conn[sfds].cn_socktype & PBS_SOCK_UNIX)
+  if (conn_socktype & PBS_SOCK_UNIX)
     {
     strcpy(request->rq_host, server_name);
     }
@@ -454,7 +458,7 @@ void *process_request(
     struct pbsnode       *isanode;
 
     get_svr_attr_arst(SRV_ATR_acl_hosts, &pas);
-    isanode = PGetNodeFromAddr(get_connectaddr(sfds,FALSE));
+    isanode = PGetNodeFromAddr(conn_addr);
 
     if ((isanode == NULL) &&
         (strcmp(server_host, request->rq_host) != 0) &&
@@ -464,10 +468,9 @@ void *process_request(
       snprintf(tmpLine, sizeof(tmpLine), "request not authorized from host %s",
                request->rq_host);
 
-      pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-      unlock_mutex = FALSE;
       req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
       free_request = FALSE;
+      rc = PBSE_BADHOST;
       goto process_request_cleanup;
       }
 
@@ -480,7 +483,7 @@ void *process_request(
    * set the permissions granted to the client
    */
 
-  if (svr_conn[sfds].cn_authen == PBS_NET_CONN_FROM_PRIVIL)
+  if (conn_authen == PBS_NET_CONN_FROM_PRIVIL)
     {
     /* request came from another server */
 
@@ -519,10 +522,9 @@ void *process_request(
       {
       req_connect(request);
 
-      if (svr_conn[sfds].cn_socktype == PBS_SOCK_INET)
+      if (conn_socktype == PBS_SOCK_INET)
         {
-        pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-        unlock_mutex = FALSE;
+        rc = PBSE_IVALREQ;
         req_reject(rc, 0, request, NULL, NULL);
         free_request = FALSE;
         goto process_request_cleanup;
@@ -530,31 +532,28 @@ void *process_request(
 
       }
 
-    if (svr_conn[sfds].cn_socktype & PBS_SOCK_UNIX)
+    if (conn_socktype & PBS_SOCK_UNIX)
       {
+      pthread_mutex_lock(svr_conn[sfds].cn_mutex);
       svr_conn[sfds].cn_authen = PBS_NET_CONN_AUTHENTICATED;
+      pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
       }
 
     if (ENABLE_TRUSTED_AUTH == TRUE )
-      rc = 0;  /* bypass the authentication of the user--trust the client completely */
+      rc = PBSE_NONE;  /* bypass the authentication of the user--trust the client completely */
     else if (munge_on)
       {
       /* If munge_on is true we will validate the connection now */
-      if ( request->rq_type == PBS_BATCH_AltAuthenUser)
+      if (request->rq_type == PBS_BATCH_AltAuthenUser)
         {
-        enqueue_threadpool_request(req_altauthenuser,request); 
-      
-        if (unlock_mutex == TRUE)
-          pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-
-        return(NULL);
+        rc = req_altauthenuser(request); 
         }
       else
         {
         rc = authenticate_user(request, &conn_credent[sfds], &auth_err);
         }
       }
-    else if (svr_conn[sfds].cn_authen != PBS_NET_CONN_AUTHENTICATED)
+    else if (conn_authen != PBS_NET_CONN_AUTHENTICATED)
       /* skip checking user if we did not get an authenticated credential */
       rc = PBSE_BADCRED;
     else
@@ -562,8 +561,6 @@ void *process_request(
 
     if (rc != 0)
       {
-      pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-      unlock_mutex = FALSE;
       req_reject(rc, 0, request, NULL, auth_err);
       if (auth_err != NULL)
         free(auth_err);
@@ -639,9 +636,8 @@ void *process_request(
       case PBS_BATCH_StageIn:
       case PBS_BATCH_jobscript:
 
-        pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-        unlock_mutex = FALSE;
         req_reject(PBSE_SVRDOWN, 0, request, NULL, NULL);
+        rc = PBSE_SVRDOWN;
         free_request = FALSE;
         goto process_request_cleanup;
         /*NOTREACHED*/
@@ -650,28 +646,25 @@ void *process_request(
       }
     }
 
-  pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
-
   /*
    * dispatch the request to the correct processing function.
    * The processing function must call reply_send() to free
    * the request struture.
    */
 
-  dispatch_request(sfds, request);
+  rc = dispatch_request(sfds, request);
 
-  return(NULL);
+
+  return(rc);
 
 process_request_cleanup:
 
-  close_conn(sfds, TRUE);
-  if (unlock_mutex == TRUE)
-    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+  /*close_conn(sfds, FALSE);*/
 
   if (free_request == TRUE)
     free_br(request);
 
-  return(NULL);
+  return(rc);
   }  /* END process_request() */
 
 
@@ -684,14 +677,16 @@ process_request_cleanup:
  * @see process_request() - parent
  */
 
-void dispatch_request(
+int dispatch_request(
 
   int          sfds,    /* I */
   struct batch_request *request) /* I */
 
   {
+  int  rc = PBSE_NONE;
   char *id = "dispatch_request";
   char  log_buf[LOCAL_LOG_BUF_SIZE];
+  char *job_id = NULL;
 
   if (LOGLEVEL >= 5)
     {
@@ -705,37 +700,49 @@ void dispatch_request(
   switch (request->rq_type)
     {
     case PBS_BATCH_QueueJob:
-
-      net_add_close_func(sfds, close_quejob, FALSE);
-
-      enqueue_threadpool_request(req_quejob,request);
-
+      rc = req_quejob(request, &job_id);
+      if ((rc != PBSE_NONE) && (job_id != NULL))
+        close_quejob_by_jobid(job_id);
+      if (job_id != NULL)
+        free(job_id);
       break;
 
+
     case PBS_BATCH_JobCred:
-
-      enqueue_threadpool_request(req_jobcredential,request);
-
+      rc = req_jobcredential(request);
       break;
 
+
     case PBS_BATCH_jobscript:
-
-      enqueue_threadpool_request(req_jobscript,request);
+      job_id = strdup(request->rq_ind.rq_jobfile.rq_jobid);
+      rc = req_jobscript(request);
+      if ((rc != PBSE_NONE) && (job_id != NULL))
+        close_quejob_by_jobid(job_id);
+      if (job_id != NULL)
+        free(job_id);
       break;
 
+
     case PBS_BATCH_RdytoCommit:
-
-      enqueue_threadpool_request(req_rdytocommit,request);
+      job_id = strdup(request->rq_ind.rq_rdytocommit);
+      rc = req_rdytocommit(request);
+      if ((rc != PBSE_NONE) && (job_id != NULL))
+        close_quejob_by_jobid(job_id);
+      if (job_id != NULL)
+        free(job_id);
       break;
 
+
     case PBS_BATCH_Commit:
-
-      enqueue_threadpool_request(req_commit,request);
-
-      net_add_close_func(sfds, (void (*)())0, FALSE);
-
+      job_id = strdup(request->rq_ind.rq_commit);
+      rc = req_commit(request);
+      if ((rc != PBSE_NONE) && (job_id != NULL))
+        close_quejob_by_jobid(job_id);
+      if (job_id != NULL)
+        free(job_id);
       break;
 
+
     case PBS_BATCH_DeleteJob:
 
       /* if this is a server size job delete request, then the request
@@ -746,41 +753,41 @@ void dispatch_request(
       */
 
       if (is_array(request->rq_ind.rq_delete.rq_objname))
-        enqueue_threadpool_request(req_deletearray,request);
+        rc = req_deletearray(request);
       else
-        enqueue_threadpool_request(req_deletejob,request);
+        rc = req_deletejob(request);
 
       break;
 
     case PBS_BATCH_HoldJob:
       if (is_array(request->rq_ind.rq_hold.rq_orig.rq_objname))
-        enqueue_threadpool_request(req_holdarray, request);
+        rc = req_holdarray(request);
       else
-        enqueue_threadpool_request(req_holdjob, request);
+        rc = req_holdjob(request);
 
       break;
 
     case PBS_BATCH_CheckpointJob:
 
-      enqueue_threadpool_request(req_checkpointjob, request);
+      rc = req_checkpointjob(request);
 
       break;
 
     case PBS_BATCH_LocateJob:
 
-      enqueue_threadpool_request(req_locatejob,request);
+      rc = req_locatejob(request);
 
       break;
 
     case PBS_BATCH_Manager:
 
-      enqueue_threadpool_request(req_manager, request);
+      rc = req_manager(request);
 
       break;
 
     case PBS_BATCH_MessJob:
 
-      enqueue_threadpool_request(req_messagejob, request);
+      rc = req_messagejob(request);
 
       break;
 
@@ -788,62 +795,62 @@ void dispatch_request(
 
     case PBS_BATCH_ModifyJob:
       if (is_array(request->rq_ind.rq_delete.rq_objname))
-        enqueue_threadpool_request(req_modifyarray, request);
+        rc = req_modifyarray(request);
       else
-        enqueue_threadpool_request(req_modifyjob, request);
+        req_modifyjob(request);
 
       break;
 
     case PBS_BATCH_Rerun:
 
-      enqueue_threadpool_request(req_rerunjob, request);
+      rc = req_rerunjob(request);
 
       break;
 
     case PBS_BATCH_MoveJob:
 
-      enqueue_threadpool_request(req_movejob, request);
+      rc = req_movejob(request);
 
       break;
 
     case PBS_BATCH_OrderJob:
 
-      enqueue_threadpool_request(req_orderjob, request);
+      rc = req_orderjob(request);
 
       break;
 
     case PBS_BATCH_Rescq:
 
-      enqueue_threadpool_request(req_rescq, request);
+      rc = req_rescq(request);
 
       break;
 
     case PBS_BATCH_ReserveResc:
 
-      enqueue_threadpool_request(req_rescreserve, request);
+      rc = req_rescreserve(request);
 
       break;
 
     case PBS_BATCH_ReleaseResc:
 
-      enqueue_threadpool_request(req_rescfree, request);
+      rc = req_rescfree(request);
 
       break;
 
     case PBS_BATCH_ReleaseJob:
 
       if (is_array(request->rq_ind.rq_delete.rq_objname))
-        enqueue_threadpool_request(req_releasearray, request);
+        rc = req_releasearray(request);
       else
-        enqueue_threadpool_request(req_releasejob, request);
+        rc = req_releasejob(request);
 
       break;
 
     case PBS_BATCH_RunJob:
 
     case PBS_BATCH_AsyrunJob:
-
-      enqueue_threadpool_request(req_runjob, request);
+      globalset_del_sock(request->rq_conn);
+      rc = req_runjob(request);
 
       break;
 
@@ -854,9 +861,9 @@ void dispatch_request(
       /* handle special 'truncated' keyword */
 
       if (!strncasecmp(request->rq_ind.rq_status.rq_id, "truncated", strlen("truncated")))
-        enqueue_threadpool_request(req_stat_job, request);
+        rc =req_stat_job(request);
       else 
-        enqueue_threadpool_request(req_selectjobs, request);
+        rc = req_selectjobs(request);
 
       break;
 
@@ -870,37 +877,37 @@ void dispatch_request(
 
     case PBS_BATCH_AsySignalJob:
 
-      enqueue_threadpool_request(req_signaljob, request);
+      rc = req_signaljob(request);
 
       break;
 
     case PBS_BATCH_GpuCtrl:
 
-      enqueue_threadpool_request(req_gpuctrl_svr, request);
+      rc = req_gpuctrl_svr(request);
 
       break;
 
     case PBS_BATCH_MvJobFile:
 
-      enqueue_threadpool_request(req_mvjobfile, request);
+      rc = req_mvjobfile(request);
 
       break;
 
     case PBS_BATCH_StatusQue:
 
-      enqueue_threadpool_request(req_stat_que, request);
+      rc = req_stat_que(request);
 
       break;
 
     case PBS_BATCH_StatusNode:
       
-      enqueue_threadpool_request(req_stat_node, request);
+      rc = req_stat_node(request);
 
       break;
 
     case PBS_BATCH_StatusSvr:
 
-      enqueue_threadpool_request(req_stat_svr, request);
+      rc = req_stat_svr(request);
 
       break;
 
@@ -908,7 +915,7 @@ void dispatch_request(
 
     case PBS_BATCH_TrackJob:
 
-      enqueue_threadpool_request(req_track, request);
+      rc = req_track(request);
 
       break;
 
@@ -916,11 +923,11 @@ void dispatch_request(
 
       if (is_array(request->rq_ind.rq_register.rq_parent))
         {
-        enqueue_threadpool_request(req_registerarray, request);
+        rc = req_registerarray(request);
         }
       else
         {
-        enqueue_threadpool_request(req_register, request);
+        rc = req_register(request);
         }
 
       break;
@@ -928,7 +935,7 @@ void dispatch_request(
     case PBS_BATCH_AuthenUser:
 
       /* determine if user is valid */
-      enqueue_threadpool_request(req_authenuser, request); 
+      rc = req_authenuser( request); 
 
       break;
 
@@ -938,19 +945,19 @@ void dispatch_request(
 
     case PBS_BATCH_JobObit:
 
-      enqueue_threadpool_request(req_jobobit, request);
+      rc = req_jobobit(request);
 
       break;
 
     case PBS_BATCH_StageIn:
 
-      enqueue_threadpool_request(req_stagein,request);
+      rc = req_stagein(request);
 
       break;
 
     case PBS_BATCH_StatusJob:
 
-      enqueue_threadpool_request(req_stat_job,request);
+      rc = req_stat_job(request);
       
       break;
 
@@ -958,16 +965,12 @@ void dispatch_request(
 
       req_reject(PBSE_UNKREQ, 0, request, NULL, NULL);
 
-      pthread_mutex_lock(svr_conn[sfds].cn_mutex);
-
-      close_conn(sfds, TRUE);
-      
-      pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+      close_conn(sfds, FALSE);
 
       break;
     }  /* END switch (request->rq_type) */
 
-  return;
+  return rc;
   }  /* END dispatch_request() */
 
 
@@ -1014,69 +1017,38 @@ struct batch_request *alloc_br(
 
 
 
-
-
-/*
- * close_quejob - locate and deal with the new job that was being recevied
- *    when the net connection closed.
- */
-
-static void close_quejob(
-
-  int sfds)
-
+int close_quejob_by_jobid(char *job_id)
   {
-  job *pjob;
+  int rc = PBSE_NONE;
+  job *pjob = NULL;
 
-  int iter = -1;
-
-  while ((pjob = next_job(&newjobs,&iter)) != NULL)
+  if ((pjob = find_job(job_id)) == NULL)
     {
-    if (pjob->ji_qs.ji_un.ji_newt.ji_fromsock == sfds)
+    rc = PBSE_JOBNOTFOUND;
+    }
+  else if (pjob->ji_qs.ji_substate != JOB_SUBSTATE_TRANSICM)
+    {
+    remove_job(&newjobs,pjob);
+    job_purge(pjob);
+    pjob = NULL;
+    }
+  else if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE)
+    {
+    remove_job(&newjobs,pjob);
+    pjob->ji_qs.ji_state = JOB_STATE_QUEUED;
+    pjob->ji_qs.ji_substate = JOB_SUBSTATE_QUEUED;
+    rc = svr_enquejob(pjob, FALSE, -1);
+    if (rc == PBSE_JOBNOTFOUND)
       {
-      if (pjob->ji_qs.ji_substate == JOB_SUBSTATE_TRANSICM)
-        {
-        if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE)
-          {
-          /*
-           * the job was being created here for the first time
-           * go ahead and enqueue it as QUEUED; otherwise, hold
-           * it here as TRANSICM until we hear from the sending
-           * server again to commit.
-           */
-
-          remove_job(&newjobs,pjob);
-
-          pjob->ji_qs.ji_state = JOB_STATE_QUEUED;
-          pjob->ji_qs.ji_substate = JOB_SUBSTATE_QUEUED;
-
-          if (svr_enquejob(pjob, FALSE, -1))
-            job_abt(&pjob, msg_err_noqueue);
-          }
-        }
-      else
-        {
-        /* else delete the job */
-        remove_job(&newjobs,pjob);
-
-        job_purge(pjob);
-
-        pjob = NULL;
-        }
-
-      break;
-      }  /* END if (..) */
-
-    pthread_mutex_unlock(pjob->ji_mutex);
+      pjob = NULL;
+      }
+    else if (rc != PBSE_NONE)
+      job_abt(&pjob, msg_err_noqueue);
     }
-
   if (pjob != NULL)
     pthread_mutex_unlock(pjob->ji_mutex);
-
-  return;
-  }  /* END close_quejob() */
-
-
+  return rc;
+  }
 
 
 
@@ -1092,6 +1064,7 @@ void free_br(
   {
   memmgr *mm = preq->mm;
 
+
   reply_free(&preq->rq_reply);
 
   if (preq->rq_extend) 
