@@ -112,7 +112,7 @@
 #include "array.h"
 #include "utils.h"
 #include "svr_func.h" /* get_svr_attr_* */
-
+#include "job_func.h" /* job_purge */
 
 #define PURGE_SUCCESS 1
 #define MOM_DELETE    2
@@ -287,7 +287,6 @@ int execute_job_delete(
 
   int               rc;
   char             *sigt = "SIGTERM";
-  char             *jobid_copy;
 
   int               has_mutex = TRUE;
   char              log_buf[LOCAL_LOG_BUF_SIZE];
@@ -554,8 +553,6 @@ jump:
     pjob->ji_momhandle = -1;
 
     /* force new connection */
-    jobid_copy = strdup(pjob->ji_qs.ji_jobid);
-
     if(LOGLEVEL >= 7)
       {
       sprintf(log_buf, "calling on_job_exit from %s", id);
@@ -565,7 +562,7 @@ jump:
         pjob->ji_qs.ji_jobid,
         log_buf);
       }
-    set_task(WORK_Immed, 0, on_job_exit, jobid_copy, FALSE);
+    set_task(WORK_Immed, 0, on_job_exit, strdup(pjob->ji_qs.ji_jobid), FALSE);
     }
   else if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_StagedIn) != 0)
     {
@@ -613,9 +610,7 @@ jump:
 
     if (pjob != NULL)
       {
-      jobid_copy = strdup(pjob->ji_qs.ji_jobid);
-      
-      set_task(WORK_Timed, time_now + KeepSeconds, on_job_exit, jobid_copy, FALSE);
+      set_task(WORK_Timed, time_now + KeepSeconds, on_job_exit, strdup(pjob->ji_qs.ji_jobid), FALSE);
       }
     else
       has_mutex = FALSE;
@@ -802,7 +797,7 @@ int handle_single_delete(
  *          |                |                |
  */
 
-void req_deletejob(
+int req_deletejob(
 
   struct batch_request *preq)  /* I */
 
@@ -819,7 +814,7 @@ void req_deletejob(
       /* purge_completed_jobs will respond with either an ack or reject */
       purge_completed_jobs(preq);
       
-      return;
+      return(PBSE_NONE);
       }
     else if (strncmp(preq->rq_extend, deldelaystr, strlen(deldelaystr)) &&
         strncmp(preq->rq_extend, delasyncstr, strlen(delasyncstr)) &&
@@ -836,7 +831,7 @@ void req_deletejob(
         req_reject(PBSE_PERM, 0, preq, NULL,
           "must have operator or manager privilege to use -m parameter");
 
-        return;
+        return(PBSE_NONE);
         }
       }
     /* check if we are getting a asynchronous delete */
@@ -862,7 +857,7 @@ void req_deletejob(
     handle_single_delete(preq, preq_tmp, Msg);
     }
 
-  return;
+  return(PBSE_NONE);
   }  /* END req_deletejob() */
 
 
