@@ -106,6 +106,7 @@
 #include "log.h"
 #include "../lib/Liblog/pbs_log.h"
 #include "../lib/Libnet/lib_net.h" /* global_sock_add */
+#include "req_getcred.h" /* req_altauthenuser */
 
 #define SPACE 32 /* ASCII space character */
 
@@ -131,11 +132,13 @@ void req_connect(
 
   {
   int  sock = preq->rq_conn;
+  unsigned short conn_authen;
 
   /* Called from one location inside a lock */
-/*   pthread_mutex_lock(svr_conn[sock].cn_mutex); */
-
-  if (svr_conn[sock].cn_authen == 0)
+  pthread_mutex_lock(svr_conn[sock].cn_mutex);
+  conn_authen = svr_conn[sock].cn_authen;
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
+  if (conn_authen == 0)
     {
     reply_ack(preq);
     }
@@ -144,7 +147,6 @@ void req_connect(
     req_reject(PBSE_BADCRED, 0, preq, NULL, "Connection not authorized");
     }
 
-/*   pthread_mutex_unlock(svr_conn[sock].cn_mutex); */
 
   return;
   }  /* END req_connect() */
@@ -406,16 +408,15 @@ int unmunge_request(
  * non-privileged connection.  This connection is marked as authenticated.
  */
 
-void *req_authenuser(
-
-  void *vp)  /* I */
-
+int req_authenuser(
+    struct batch_request *preq)
   {
   int                   s;
   int                   debug = 0;
   int                   delay_cntr = 0;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
-  struct batch_request *preq = (struct batch_request *)vp;
+  unsigned short        conn_port;
+  unsigned short        conn_authen;
 
   /*
    * find the socket whose client side is bound to the port named
@@ -431,15 +432,17 @@ void *req_authenuser(
     for (s = 0;s < PBS_NET_MAX_CONNECTIONS;++s)
       {
       pthread_mutex_lock(svr_conn[s].cn_mutex);
+      conn_port = svr_conn[s].cn_port;
+      conn_authen = svr_conn[s].cn_authen;
+      pthread_mutex_unlock(svr_conn[s].cn_mutex);
   
-      if (preq->rq_ind.rq_authen.rq_port != svr_conn[s].cn_port)
+      if (preq->rq_ind.rq_authen.rq_port != conn_port)
         {
-        pthread_mutex_unlock(svr_conn[s].cn_mutex);
         continue;
         }
 
 #ifndef NOPRIVPORTS
-      if (svr_conn[s].cn_authen == 0)
+      if (conn_authen == 0)
 #endif
         {
         strcpy(conn_credent[s].username, preq->rq_user);
@@ -449,16 +452,17 @@ void *req_authenuser(
 
         conn_credent[s].timestamp = time(NULL);
 
+        pthread_mutex_lock(svr_conn[s].cn_mutex);
         svr_conn[s].cn_authen = PBS_NET_CONN_AUTHENTICATED;
+        pthread_mutex_unlock(svr_conn[s].cn_mutex);
         }
 
       reply_ack(preq);
 
       /* SUCCESS */
-      pthread_mutex_unlock(svr_conn[s].cn_mutex);
-      if (debug) printf("(FOUND_PROCESSED) unlock %d (port %d)\n", s, svr_conn[s].cn_port);
+      if (debug) printf("(FOUND_PROCESSED) unlock %d (port %d)\n", s,conn_port);
 
-      return NULL;
+      return PBSE_NONE;
       }  /* END for (s) */
     if (debug) fprintf(stderr, "sock not found, sleeping (%d)\n", delay_cntr);
     usleep(10);
@@ -472,7 +476,7 @@ void *req_authenuser(
 
   /* FAILURE */
 
-  return NULL;
+  return PBSE_BADCRED;
   }  /* END req_authenuser() */
 
 
@@ -483,14 +487,15 @@ void *req_authenuser(
  * utility 
  * 
 */
-void *req_altauthenuser(
+int req_altauthenuser(
 
-  void *vp)  /* I */
+  struct batch_request *preq)  /* I */
 
   {
-  struct batch_request *preq = (struct batch_request *)vp;
   int s;
-  int rc;
+  int rc = PBSE_NONE;
+  unsigned short        conn_port;
+  unsigned short        conn_authen;
   
   /*
    * find the socket whose client side is bound to the port named
@@ -500,11 +505,12 @@ void *req_altauthenuser(
   for (s = 0;s < PBS_NET_MAX_CONNECTIONS;++s)
     {
     pthread_mutex_lock(svr_conn[s].cn_mutex);
+    conn_port = svr_conn[s].cn_port;
+    conn_authen = svr_conn[s].cn_authen;
+    pthread_mutex_unlock(svr_conn[s].cn_mutex);
 
-    if (preq->rq_ind.rq_authen.rq_port != svr_conn[s].cn_port)
+    if (preq->rq_ind.rq_authen.rq_port != conn_port)
       {
-      pthread_mutex_unlock(svr_conn[s].cn_mutex);
-
       continue;
       }
     break;
@@ -513,17 +519,15 @@ void *req_altauthenuser(
   /* If s is less than PBS_NET_MAX_CONNECTIONS we have our port */
   if (s >= PBS_NET_MAX_CONNECTIONS)
     {
-    pthread_mutex_unlock(svr_conn[s].cn_mutex);
 	  req_reject(PBSE_BADCRED, 0, preq, NULL, "cannot authenticate user. Client connection not found");
-    return(NULL);
+    return(PBSE_BADCRED);
     }
 
   rc = unmunge_request(s, preq);
   if (rc)
     {
     /* FAILED */
-    pthread_mutex_unlock(svr_conn[s].cn_mutex);
-    return(NULL);
+    return(rc);
     }
 
   /* SUCCESS */
@@ -532,13 +536,13 @@ void *req_altauthenuser(
 
   conn_credent[s].timestamp = time(NULL);
 
+  pthread_mutex_lock(svr_conn[s].cn_mutex);
   svr_conn[s].cn_authen = PBS_NET_CONN_AUTHENTICATED;
-
   pthread_mutex_unlock(svr_conn[s].cn_mutex);
 
   reply_ack(preq);
   
-  return(NULL);
+  return(PBSE_NONE);
   }  /* END req_altauthenuser() */
 
 
