@@ -246,6 +246,19 @@ void mom_cleanup_checkpoint_hold(
     }
 
   pjob = find_job(jobid);
+  if (pjob == NULL)
+    {
+    if (LOGLEVEL >= 3)
+      {
+      sprintf(log_buf,
+        "%s:failed to find job\n",
+        id);
+
+      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,jobid,log_buf);
+      }
+    free(jobid);
+    return;
+    }
   free(jobid);
 
   if (LOGLEVEL >= 7)
@@ -409,6 +422,13 @@ int modify_job(
 
   job *pjob = (job *)*j;
   
+  if (pjob == NULL)
+    {
+    sprintf(log_buffer, "job structure is NULL");
+    log_err(PBSE_IVALREQ, __func__, log_buffer);
+    return(PBSE_IVALREQ);
+    }
+
   /* cannot be in exiting or transit, exiting has already been checked */
 
   if (pjob->ji_qs.ji_state == JOB_STATE_TRANSIT)
@@ -759,7 +779,8 @@ int copy_batchrequest(
           strcpy(newpal->al_atopl.name, pal->al_atopl.name);
           newpal->al_nameln = pal->al_nameln;
           newpal->al_atopl.resource = newpal->al_atopl.name + newpal->al_nameln;
-          strcpy(newpal->al_atopl.resource, pal->al_atopl.resource);
+          if (pal->al_atopl.resource != NULL)
+            strcpy(newpal->al_atopl.resource, pal->al_atopl.resource);
           newpal->al_rescln = pal->al_rescln;
           newpal->al_atopl.value = newpal->al_atopl.name + newpal->al_nameln + newpal->al_rescln;
           strcpy(newpal->al_atopl.value, pal->al_atopl.value);
