@@ -117,7 +117,7 @@ job  *chk_job_request(char *, struct batch_request *);
  * req_movejob = move a job to a new destination (local or remote)
  */
 
-void *req_movejob(
+int req_movejob(
 
   void *vp) /* I */
 
@@ -134,7 +134,7 @@ void *req_movejob(
 
   if (jobp == NULL)
     {
-    return(NULL);
+    return(PBSE_NONE);
     }
 
   if ((jobp->ji_qs.ji_state != JOB_STATE_QUEUED) &&
@@ -151,7 +151,7 @@ void *req_movejob(
 
     pthread_mutex_unlock(jobp->ji_mutex);
 
-    return(NULL);
+    return(PBSE_NONE);
     }
 
   /*
@@ -199,7 +199,7 @@ void *req_movejob(
 
   pthread_mutex_unlock(jobp->ji_mutex);
 
-  return(NULL);
+  return(PBSE_NONE);
   }  /* END req_movejob() */
 
 
@@ -211,7 +211,7 @@ void *req_movejob(
  * req_orderjob = reorder the jobs in a queue
  */
 
-void *req_orderjob(
+int req_orderjob(
 
   void *vp) /* I */
 
@@ -227,17 +227,19 @@ void *req_orderjob(
   char                  tmpqn[PBS_MAXQUEUENAME+1];
   struct batch_request *req = (struct batch_request *)vp;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  char                  job_id1[PBS_MAXSVRJOBID+1];
+  char                  job_id2[PBS_MAXSVRJOBID+1];
   pbs_queue            *pque1;
   pbs_queue            *pque2;
 
   if ((pjob1 = chk_job_request(req->rq_ind.rq_move.rq_jid, req)) == NULL)
     {
-    return(NULL);
+    return(PBSE_NONE);
     }
 
   if ((pjob2 = chk_job_request(req->rq_ind.rq_move.rq_destin, req)) == NULL)
     {
-    return(NULL);
+    return(PBSE_NONE);
     }
 
   if (((pjob = pjob1)->ji_qs.ji_state == JOB_STATE_RUNNING) ||
@@ -262,7 +264,7 @@ void *req_orderjob(
     pthread_mutex_unlock(pjob1->ji_mutex);
     pthread_mutex_unlock(pjob2->ji_mutex);
 
-    return(NULL);
+    return(PBSE_NONE);
     }
   else if ((pjob1->ji_qhdr == NULL) || (pjob2->ji_qhdr == NULL))
     {
@@ -306,7 +308,7 @@ void *req_orderjob(
       if (pjob2 != NULL)
         pthread_mutex_unlock(pjob2->ji_mutex);
 
-      return(NULL);
+      return(PBSE_NONE);
       }
     }
 
@@ -324,17 +326,21 @@ void *req_orderjob(
     strcpy(tmpqn, pjob1->ji_qs.ji_queue);
     strcpy(pjob1->ji_qs.ji_queue, pjob2->ji_qs.ji_queue);
     strcpy(pjob2->ji_qs.ji_queue, tmpqn);
-    svr_dequejob(pjob1, FALSE);
-    svr_dequejob(pjob2, FALSE);
-    if (strlen(pjob1->ji_qs.ji_jobid) > 0)
-      svr_enquejob(pjob1, FALSE, -1);
-    else
-      pjob1 = NULL;
+    strcpy(job_id1, pjob1->ji_qs.ji_jobid);
+    strcpy(job_id2, pjob2->ji_qs.ji_jobid);
+    pthread_mutex_unlock(pjob1->ji_mutex);
+    pthread_mutex_unlock(pjob2->ji_mutex);
+    pjob1 = NULL;
+    pjob2 = NULL;
 
-    if (strlen(pjob2->ji_qs.ji_jobid) > 0)
+    svr_dequejob(job_id1, FALSE);
+    svr_dequejob(job_id2, FALSE);
+
+    if ((pjob1 = find_job(job_id1)) != NULL)
+      svr_enquejob(pjob1, FALSE, -1);
+
+    if ((pjob2 = find_job(job_id2)) != NULL)
       svr_enquejob(pjob2, FALSE, -1);
-    else
-      pjob2 = NULL;
     }
   else
     {
@@ -348,18 +354,18 @@ void *req_orderjob(
 
   /* need to update disk copy of both jobs to save new order */
   if (pjob1 != NULL)
+    {
     job_save(pjob1, SAVEJOB_FULL, 0);
+    pthread_mutex_unlock(pjob1->ji_mutex);
+    }
+
   if (pjob2 != NULL)
-  job_save(pjob2, SAVEJOB_FULL, 0);
+    {
+    job_save(pjob2, SAVEJOB_FULL, 0);
+    pthread_mutex_unlock(pjob2->ji_mutex);
+    }
 
   reply_ack(req);
-
-  /* SUCCESS */
-  if (pjob1 != NULL)
-    pthread_mutex_unlock(pjob1->ji_mutex);
-  if (pjob2 != NULL)
-    pthread_mutex_unlock(pjob2->ji_mutex);
-
-  return(NULL);
+  return(PBSE_NONE);
   }  /* END req_orderjob() */
 
