@@ -168,12 +168,11 @@ extern int   svr_chk_owner(struct batch_request *, job *);
  * to be graceful.
  */
 
-void *req_register(
+int req_register(
 
-  void *vp)  /* I */
+  struct batch_request *preq)  /* I */
 
   {
-  struct batch_request *preq = (struct batch_request *)vp;
   int                   made;
   attribute            *pattr;
 
@@ -182,10 +181,10 @@ void *req_register(
   struct depend_job    *pdj;
   job                  *pjob;
   char                 *ps;
-  int                   rc = 0;
+  int                   rc = PBSE_NONE;
   int                   revtype;
   int                   type;
-  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  char                  log_buf[LOCAL_LOG_BUF_SIZE + 1];
 
   /*  make sure request is from a server */
 
@@ -193,7 +192,7 @@ void *req_register(
     {
     req_reject(PBSE_IVALREQ, 0, preq, NULL, NULL);
 
-    return(NULL);
+    return(PBSE_IVALREQ);
     }
 
   /* find the "parent" job specified in the request */
@@ -216,14 +215,16 @@ void *req_register(
         preq->rq_ind.rq_register.rq_parent,
         pbse_to_txt(PBSE_UNKJOBID));
 
-      req_reject(PBSE_UNKJOBID, 0, preq, NULL, NULL);
+      rc = PBSE_UNKJOBID;
+      req_reject(rc, 0, preq, NULL, NULL);
       }
     else
       {
       reply_ack(preq);
+      rc = PBSE_JOBNOTFOUND;
       }
 
-    return(NULL);
+    return(rc);
     }
 
   type = preq->rq_ind.rq_register.rq_dependtype;
@@ -251,11 +252,12 @@ void *req_register(
       preq->rq_ind.rq_register.rq_parent,
       pbse_to_txt(PBSE_BADSTATE));
     
-    req_reject(PBSE_BADSTATE, 0, preq, NULL, NULL);
+    rc = PBSE_BADSTATE;
+    req_reject(rc, 0, preq, NULL, NULL);
 
     pthread_mutex_unlock(pjob->ji_mutex);
     
-    return(NULL);
+    return(rc);
     }
 
   if (LOGLEVEL >= 8)
@@ -606,7 +608,7 @@ void *req_register(
   if (pjob != NULL)
     pthread_mutex_unlock(pjob->ji_mutex);
 
-  return(NULL);
+  return(rc);
   }  /* END req_register() */
 
 
@@ -617,20 +619,19 @@ void *req_register(
  * registers a dependency on an array
  */
 
-void *req_registerarray(
+int req_registerarray(
 
-  void *vp)  /* I */
+  struct batch_request *preq)  /* I */
 
   {
-  struct batch_request *preq = (struct batch_request *)vp;
-  char        log_buf[LOCAL_LOG_BUF_SIZE];
+  char        log_buf[LOCAL_LOG_BUF_SIZE + 1];
   job_array  *pa;
   char        array_name[PBS_MAXSVRJOBID + 1];
   char        range[MAXPATHLEN];
   int         num_jobs = -1;
   char       *dot_server;
   char       *bracket_ptr;
-  int         rc = 0;
+  int         rc = PBSE_NONE;
   int         type;
 
   /*  make sure request is from a server */
@@ -638,7 +639,7 @@ void *req_registerarray(
     {
     req_reject(PBSE_IVALREQ, 0, preq, NULL, NULL);
 
-    return(NULL);
+    return(PBSE_IVALREQ);
     }
 
   strcpy(array_name,preq->rq_ind.rq_register.rq_parent);
@@ -676,6 +677,7 @@ void *req_registerarray(
 
           req_reject(PBSE_IVALREQ,0,preq,NULL,
             "No server specified");
+          return(PBSE_IVALREQ);
           }
         }
       else
@@ -684,6 +686,7 @@ void *req_registerarray(
 
         req_reject(PBSE_IVALREQ,0,preq,NULL,
           "Array range format invalid, must have closed bracket ']'");
+        return(PBSE_IVALREQ);
         }
       } /* end second brackets if */
     }
@@ -698,6 +701,7 @@ void *req_registerarray(
     long state = SV_STATE_DOWN;
     get_svr_attr_l(SRV_ATR_State, &state);
 
+    rc = PBSE_UNKARRAYID;
     if (state != SV_STATE_INIT)
       {
       log_event(
@@ -706,14 +710,14 @@ void *req_registerarray(
         preq->rq_ind.rq_register.rq_parent,
         pbse_to_txt(PBSE_UNKJOBID));
 
-      req_reject(PBSE_UNKARRAYID, 0, preq, NULL, "unable to find array");
+      req_reject(rc, 0, preq, NULL, "unable to find array");
       }
     else
       {
       reply_ack(preq);
       }
 
-    return(NULL);
+    return(rc);
     }
 
   type = preq->rq_ind.rq_register.rq_dependtype;
@@ -727,10 +731,11 @@ void *req_registerarray(
       log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
       }
 
-    req_reject(PBSE_IVALREQ,0,preq,NULL,
+    rc = PBSE_IVALREQ;
+    req_reject(rc,0,preq,NULL,
       "Arrays may only be given array dependencies");
 
-    return(NULL);
+    return(rc);
     }
 
   /* register the dependency on the array */
@@ -766,7 +771,7 @@ void *req_registerarray(
     log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
     }
 
-  return(NULL);
+  return(rc);
   } /* END req_registerarray() */
 
 
@@ -1078,6 +1083,11 @@ static void alter_unreg(
   struct depend_job *oldjd;
 
   int                type;
+  char               job_id[PBS_MAXSVRJOBID+1];
+
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
+  pthread_mutex_unlock(pjob->ji_mutex);
+  pjob = NULL;
 
   for (poldd = (struct depend *)GET_NEXT(old->at_val.at_list);
        poldd;
@@ -1095,6 +1105,8 @@ static void alter_unreg(
         {
         if ((pnewd == 0) || (find_dependjob(pnewd, oldjd->dc_child) == 0))
           {
+          if ((pjob = find_job(job_id)) == NULL)
+              return;
           send_depend_req(
             pjob,
             oldjd,
@@ -1102,6 +1114,7 @@ static void alter_unreg(
             JOB_DEPEND_OP_UNREG,
             SYNC_SCHED_HINT_NULL,
             release_req);
+          pjob = NULL;
           }
 
         oldjd = (struct depend_job *)GET_NEXT(oldjd->dc_link);
@@ -1138,6 +1151,9 @@ int depend_on_que(
   int                type;
   job               *pjob = (job *)pj;
   pbs_queue         *pque = get_jobs_queue(&pjob);
+  char job_id[PBS_MAXSVRJOBID+1];
+
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
 
   if (pque == NULL)
     {
@@ -1164,6 +1180,8 @@ int depend_on_que(
     /* if there are dependencies being removed, unregister them */
 
     alter_unreg(pjob, &(pjob)->ji_wattr[JOB_ATR_depend], pattr);
+    if ((pjob = find_job(job_id)) == NULL)
+      return PBSE_JOBNOTFOUND;
     }
 
   /* First set a System hold if required */
@@ -1195,9 +1213,16 @@ int depend_on_que(
 
       while (pparent)
         {
-        if ((rc = send_depend_req(pjob, pparent, type, JOB_DEPEND_OP_REGISTER, SYNC_SCHED_HINT_NULL, post_doq)))
-
+        if ((pjob == NULL) && ((pjob = find_job(job_id)) == NULL))
+          {
+          return PBSE_JOBNOTFOUND;
+          }
+        if ((rc = send_depend_req(pjob, pparent, type, JOB_DEPEND_OP_REGISTER, SYNC_SCHED_HINT_NULL, post_doq)) != PBSE_NONE)
+          {
+          pjob = find_job(job_id);
           return(rc);
+          }
+        pjob = NULL;
 
         pparent = (struct depend_job *)GET_NEXT(pparent->dc_link);
         }
@@ -1258,8 +1283,6 @@ static void post_doe(
 
 
 
-
-
 /*
  * depend_on_exec - Perform actions if job has
  * "beforestart" dependency - send "register-release" to child job; or
@@ -1273,13 +1296,12 @@ int depend_on_exec(
   job *pjob)
 
   {
-
   struct depend     *pdep;
-
   struct depend_job *pdj;
+  char               jobid[PBS_MAXSVRJOBID+1];
 
+  strcpy(jobid, pjob->ji_qs.ji_jobid);
   /* If any jobs come after my start, release them */
-
   pdep = find_depend(
            JOB_DEPEND_TYPE_BEFORESTART,
            &pjob->ji_wattr[JOB_ATR_depend]);
@@ -1297,6 +1319,10 @@ int depend_on_exec(
         JOB_DEPEND_OP_RELEASE,
         SYNC_SCHED_HINT_NULL,
         post_doe);
+      pjob = NULL; /* send_depend_req returns unlocked, NULL--> unlocked */
+
+      if ((pjob = find_job(jobid)) == NULL)
+        return PBSE_JOBNOTFOUND;
 
       pdj = (struct depend_job *)GET_NEXT(pdj->dc_link);
       }
@@ -1322,6 +1348,8 @@ int depend_on_exec(
         JOB_DEPEND_OP_READY,
         SYNC_SCHED_HINT_NULL,
         release_req);
+      if ((pjob == NULL) && ((pjob = find_job(jobid)) == NULL))
+        return PBSE_JOBNOTFOUND;
       }
     }
 
@@ -1376,9 +1404,11 @@ int depend_on_term(
   int        rc;
   int        shouldkill = 0;
   int        type;
+  char job_id[PBS_MAXSVRJOBID+1];
+  int job_unlocked = 0;
 
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
   pattr = &pjob->ji_wattr[JOB_ATR_depend];
-
   pdep = (struct depend *)GET_NEXT(pattr->at_val.at_list);
 
   while (pdep != NULL)
@@ -1449,10 +1479,15 @@ int depend_on_term(
 
           while (pparent)
             {
-            send_depend_req(pjob, pparent, type,
+            if ((pjob == NULL) && ((pjob = find_job(job_id)) == NULL))
+              return PBSE_JOBNOTFOUND;
+
+            rc = send_depend_req(pjob, pparent, type,
                             JOB_DEPEND_OP_DELETE,
                             SYNC_SCHED_HINT_NULL,
                             release_req);
+            pjob = NULL;
+            job_unlocked = 1;
 
             pparent = (struct depend_job *)GET_NEXT(pparent->dc_link);
             }
@@ -1468,11 +1503,15 @@ int depend_on_term(
 
       while (pparent)
         {
-
+        if ((pjob == NULL) && ((pjob = find_job(job_id)) == NULL))
+          return PBSE_JOBNOTFOUND;
         /* "release" the job to execute */
-        if ((rc = send_depend_req(pjob, pparent, type, op,
-                                  SYNC_SCHED_HINT_NULL, release_req)))
+        if ((rc = send_depend_req(pjob, pparent, type, op, SYNC_SCHED_HINT_NULL,
+                release_req)) != PBSE_NONE)
+          {
           return (rc);
+          }
+        job_unlocked = 1;
 
         pparent = (struct depend_job *)GET_NEXT(pparent->dc_link);
         }
@@ -1480,6 +1519,8 @@ int depend_on_term(
 
     pdep = (struct depend *)GET_NEXT(pdep->dp_link);
     }
+  if (!job_unlocked)
+    pthread_mutex_unlock(pjob->ji_mutex);
 
   return(0);
   }  /* END depend_on_term() */
@@ -1499,14 +1540,15 @@ static void release_cheapest(
   struct depend *pdep)
 
   {
+  char job_id[PBS_MAXSVRJOBID+1];
   long     lowestcost = 0;
-
   struct depend_job *cheapest = (struct depend_job *)0;
   int     hint = SYNC_SCHED_HINT_OTHER;
   int     nreleased = 0;
-
   struct depend_job *pdj;
 
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
+
   pdj = (struct depend_job *)GET_NEXT(pdep->dp_jobs);
 
   while (pdj != NULL)
@@ -1543,6 +1585,7 @@ static void release_cheapest(
       {
       cheapest->dc_state = JOB_DEPEND_OP_RELEASE;
       }
+    pjob = find_job(job_id);
     }
 
   return;
@@ -2088,7 +2131,7 @@ static int send_depend_req(
   if (preq == NULL)
     {
     log_err(errno, myid, msg_err_malloc);
-
+    pthread_mutex_unlock(pjob->ji_mutex);
     return PBSE_SYSTEM;
     }
 
@@ -2145,17 +2188,8 @@ static int send_depend_req(
   if ((rc = issue_to_svr(pparent->dc_svr, preq, postfunc)) != PBSE_NONE)
     {
     sprintf(log_buf, "Unable to perform dependency with job %s\n", pparent->dc_child);
-
-    find_job(job_id);
-
     return rc;
     }
-
-  pjob = find_job(job_id);
-
-  if (pjob == NULL)
-    return PBSE_JOBNOTFOUND;
-
   return PBSE_NONE;
   }  /* END send_depend_req() */
 
