@@ -115,6 +115,7 @@
 #include "node_func.h" /* find_nodebyname */
 #include "../lib/Libutils/u_lock_ctl.h" /* unlock_node */
 #include "svr_func.h" /* get_svr_attr_* */
+#include "req_stat.h" /* stat_mom_job */
 
 
 #ifdef HAVE_NETINET_IN_H
@@ -123,13 +124,12 @@
 
 /* External Functions Called: */
 
-extern int   send_job_work(job **,char *,int,int *,struct batch_request *);
+extern int   send_job_work(char *job_id,char *,int,int *,struct batch_request *);
 extern void  set_resc_assigned(job *, enum batch_op);
 
 extern struct batch_request *cpy_stage(struct batch_request *, job *, enum job_atr, int);
 void                         stream_eof(int, u_long, uint16_t, int);
 extern int                   job_set_wait(attribute *, void *, int);
-extern void                  stat_mom_job(job *);
 
 extern int LOGLEVEL;
 
@@ -182,22 +182,23 @@ void poll_job_task(work_task *);
  * This request forces a job into execution.  Client must be privileged.
  */
 
-void *req_runjob(
+int req_runjob(
 
-  void *vp)  /* I (modified) */
+  struct batch_request *preq)  /* I (modified) */
 
   {
-  char   id[] = "req_runjob";
-  struct batch_request *preq = (struct batch_request *)vp;
-  job                  *pjob;
-  int                   rc;
-  void                 *bp;
+  job   *pjob;
+  int   rc;
+  void  *bp;
+  int   setneednodes;
+  char  failhost[MAXLINE];
+  char  emsg[MAXLINE];
+  char  log_buf[LOCAL_LOG_BUF_SIZE + 1];
+  char  job_id[PBS_MAXSVRJOBID+1];
+  long  job_atr_hold;
+  int   job_exit_status;
+  int   job_state;
 
-  int                   setneednodes;
-
-  char                  failhost[MAXLINE];
-  char                  emsg[MAXLINE];
-  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   /* chk_job_torun will extract job id and assign hostlist if specified */
 
@@ -210,7 +211,26 @@ void *req_runjob(
     {
     /* FAILURE - chk_job_torun performs req_reject internally */
 
-    return(NULL);
+    return(PBSE_UNKJOBID);
+    }
+
+  /* we don't currently allow running of an entire job array */
+
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
+  if (strstr(pjob->ji_qs.ji_jobid,"[]") != NULL)
+    {
+    pthread_mutex_unlock(pjob->ji_mutex);
+    req_reject(PBSE_IVALREQ, 0, preq, NULL, "cannot run a job array");
+    return(PBSE_IVALREQ);
+    }
+
+  /* we don't currently allow running of an entire job array */
+
+  if (strstr(pjob->ji_qs.ji_jobid,"[]") != NULL)
+    {
+    req_reject(PBSE_IVALREQ, 0, preq, NULL, "cannot run a job array");
+
+    return(PBSE_IVALREQ);
     }
 
   pthread_mutex_lock(scheduler_sock_jobct_mutex);
@@ -231,8 +251,7 @@ void *req_runjob(
       (preq->rq_type == PBS_BATCH_AsyrunJob))
     {
     reply_ack(preq);
-
-    preq = NULL;  /* cleared so we don't try to reuse */
+    preq->rq_noreply = TRUE;
     }
 
   /* if the job is part of an array, check the slot limit */
@@ -243,13 +262,30 @@ void *req_runjob(
 
     if (pjob == NULL)
       {
-      req_reject(PBSE_JOBNOTFOUND, 0, preq, NULL, "Job unexpectedly deleted");
+      rc = PBSE_JOBNOTFOUND;
+      req_reject(rc, 0, preq, NULL, "Job unexpectedly deleted");
+      pthread_mutex_unlock(pa->ai_mutex);
+      return(rc);
       }
     
     if ((pa->ai_qs.slot_limit < 0) ||
         (pa->ai_qs.slot_limit > pa->ai_qs.jobs_running))
       {
-      update_array_values(pa,pjob,pjob->ji_qs.ji_state,aeRun);
+      job_atr_hold = pjob->ji_wattr[JOB_ATR_hold].at_val.at_long;
+      job_exit_status = pjob->ji_qs.ji_un.ji_exect.ji_exitstat;
+      job_state = pjob->ji_qs.ji_state;
+      pthread_mutex_unlock(pjob->ji_mutex);
+      update_array_values(pa,job_state,aeRun,
+          job_id, job_atr_hold, job_exit_status);
+      if ((pjob = find_job(job_id)) == NULL)
+        {
+        rc = PBSE_JOBNOTFOUND;
+        req_reject(rc, 0, preq, NULL,
+          "Job deleted while updating array values");
+        pthread_mutex_unlock(pa->ai_mutex);
+        return(rc);
+        }
+
       }
     else
       {
@@ -257,25 +293,24 @@ void *req_runjob(
         "Cannot run job. Array slot limit is %d and there are already %d jobs running\n",
         pa->ai_qs.slot_limit,
         pa->ai_qs.jobs_running);
-      
-      if (preq != NULL)
-        req_reject(PBSE_IVALREQ,0,preq,NULL,log_buf);
+     
+      req_reject(PBSE_IVALREQ,0,preq,NULL,log_buf);
       
       if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "%s: unlocked ai_mutex", id);
+        sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
         log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pa->ai_qs.parent_id, log_buf);
         }
 
       pthread_mutex_unlock(pjob->ji_mutex);
       pthread_mutex_unlock(pa->ai_mutex);
 
-      return(NULL);
+      return(PBSE_IVALREQ);
       }
     
     if (LOGLEVEL >= 7)
       {
-      sprintf(log_buf, "%s: unlocked ai_mutex", id);
+      sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
       log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pa->ai_qs.parent_id, log_buf);
       }
     
@@ -305,7 +340,7 @@ void *req_runjob(
 
   pthread_mutex_unlock(pjob->ji_mutex);
 
-  return(NULL);
+  return(rc);
   }  /* END req_runjob() */
 
 
@@ -506,16 +541,15 @@ static int svr_send_checkpoint(
  * Client must be privileged.
  */
 
-void *req_stagein(
+int req_stagein(
 
-  void *vp)  /* I */
+  struct batch_request *preq)  /* I */
 
   {
   job *pjob;
-  int  rc;
+  int  rc = PBSE_NONE;
 
   int  setneednodes;
-  struct batch_request *preq = (struct batch_request *)vp;
 
   if (getenv("TORQUEAUTONN"))
     setneednodes = 1;
@@ -524,7 +558,7 @@ void *req_stagein(
 
   if ((pjob = chk_job_torun(preq, setneednodes)) == NULL)
     {
-    return(NULL);
+    return(PBSE_UNKJOBID);
     }
 
   if ((pjob->ji_wattr[JOB_ATR_stagein].at_flags & ATR_VFLAG_SET) == 0)
@@ -535,7 +569,7 @@ void *req_stagein(
  
     pthread_mutex_unlock(pjob->ji_mutex);
 
-    return(NULL);
+    return(PBSE_IVALREQ);
     }
 
   if ((rc = svr_stagein(
@@ -551,7 +585,7 @@ void *req_stagein(
 
   pthread_mutex_unlock(pjob->ji_mutex);
 
-  return(NULL);
+  return(rc);
   }  /* END req_stagein() */
 
 
@@ -740,7 +774,6 @@ int verify_moms_up(
   job *pjob)
 
   {
-  static char        *id = "svr_startjob";
   int                 sock;
   int                 nodenum;
 
@@ -761,7 +794,7 @@ int verify_moms_up(
   if (hostlist == NULL)
     {
     sprintf(log_buf, "could not allocate temporary buffer (calloc failed) -- skipping TCP connect check");
-    log_err(errno, id, log_buf);
+    log_err(errno, __func__, log_buf);
     }
   else
     {
@@ -796,7 +829,7 @@ int verify_moms_up(
       /* Add this host to the reject destination list for the job */
       if ((bp = (badplace *)calloc(1, sizeof(badplace))) == NULL)
         {
-        log_err(ENOMEM, id, msg_err_malloc);
+        log_err(ENOMEM, __func__, msg_err_malloc);
 
         return(PBSE_SYSTEM);
         }
@@ -833,7 +866,7 @@ int verify_moms_up(
         {
         /* FAILURE - cannot allocate memory */
 
-        log_err(errno, id, msg_err_malloc);
+        log_err(errno, __func__, msg_err_malloc);
 
         return(PBSE_RESCUNAV);
         }
@@ -856,6 +889,7 @@ int verify_moms_up(
     /* Connect to the host. */
     if (connect(sock, (struct sockaddr *)&saddr, sizeof(saddr)) < 0)
       {
+      close(sock);
       sprintf(log_buf, "could not contact %s (connect failed, errno: %d (%s))",
         nodestr,
         errno,
@@ -873,7 +907,7 @@ int verify_moms_up(
       if ((bp = (badplace *)calloc(1, sizeof(badplace))) == NULL)
         {
         /* FAILURE - cannot allocate memory */
-        log_err(errno, id, msg_err_malloc);
+        log_err(errno, __func__, msg_err_malloc);
 
         return(PBSE_RESCUNAV);
         }
@@ -1052,6 +1086,8 @@ static int svr_strtjob2(
   struct timezone  tz;
   long             job_timeout = 0;
   long             tcp_timeout = 0;
+  unsigned long    job_momaddr = -1;
+  char             job_id[PBS_MAXSVRJOBID+1];
 
   old_state = pjob->ji_qs.ji_state;
   old_subst = pjob->ji_qs.ji_substate;
@@ -1090,11 +1126,20 @@ static int svr_strtjob2(
     {
     DIS_tcp_settimeout(job_timeout);
     }
+  job_momaddr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
+  pthread_mutex_unlock(pjob->ji_mutex);
+  *pjob_ptr = NULL;
+  pjob = NULL;
 
-  if (send_job_work(pjob_ptr, NULL, MOVE_TYPE_Exec, &my_err, preq) == PBSE_NONE)
+  if (send_job_work(job_id, NULL, MOVE_TYPE_Exec, &my_err, preq) == PBSE_NONE)
     {
     /* SUCCESS */
     DIS_tcp_settimeout(tcp_timeout);
+    if ((pjob = find_job(job_id)) != NULL)
+      {
+      *pjob_ptr = pjob;
+      }
 
     return(PBSE_NONE);
     }
@@ -1102,12 +1147,11 @@ static int svr_strtjob2(
     {
     DIS_tcp_settimeout(tcp_timeout);
 
-    if (*pjob_ptr != NULL)
+    if ((pjob = find_job(job_id)) != NULL)
       {
-      sprintf(tmpLine, "unable to run job, send to MOM '%lu' failed",
-        pjob->ji_qs.ji_un.ji_exect.ji_momaddr);
-      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,tmpLine);
-      
+      sprintf(tmpLine, "unable to run job, send to MOM '%lu' failed", job_momaddr);
+      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,job_id,tmpLine);
+      *pjob_ptr = pjob;
       pjob->ji_qs.ji_destin[0] = '\0';
       
       svr_setjobstate(pjob, old_state, old_subst, FALSE);
@@ -1123,7 +1167,7 @@ static int svr_strtjob2(
 
 void finish_sendmom(
 
-  job                  *pjob,
+  char                  *job_id,
   struct batch_request *preq,
   long                  start_time,
   char                 *node_name,
@@ -1134,12 +1178,19 @@ void finish_sendmom(
   pbs_net_t  addr;
   int        newstate;
   int        newsub;
-  char       log_buf[LOCAL_LOG_BUF_SIZE];
+  char       log_buf[LOCAL_LOG_BUF_SIZE+1];
   time_t     time_now = time(NULL);
+  job *pjob;
+
+  if ((pjob = find_job(job_id)) == NULL)
+    {
+    req_reject(PBSE_JOBNOTFOUND, 0, preq, node_name, log_buf);
+    return;
+    }
 
   if (LOGLEVEL >= 6)
     {
-    log_record(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,"entering post_sendmom");
+    log_record(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,job_id,"entering finish_sendmom");
     }
 
   if (LOGLEVEL >= 1)
@@ -1150,7 +1201,7 @@ void finish_sendmom(
       (node_name != NULL) ? node_name : "???",
       status);
 
-    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_JOB,job_id,log_buf);
     }
 
   switch (status)
@@ -1188,7 +1239,7 @@ void finish_sendmom(
         depend_on_exec(pjob);
 
       /* set up the poll task */
-      set_task(WORK_Timed, time_now + JobStatRate, poll_job_task, strdup(pjob->ji_qs.ji_jobid), FALSE);
+      set_task(WORK_Timed, time_now + JobStatRate, poll_job_task, strdup(job_id), FALSE);
 
       break;
 
@@ -1201,7 +1252,7 @@ void finish_sendmom(
       /* send failed, requeue the job */
       log_event(PBSEVENT_JOB,
         PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
+        job_id,
         "unable to run job, MOM rejected/timeout");
 
       free_nodes(pjob);
@@ -1242,7 +1293,7 @@ void finish_sendmom(
       /* send failed, requeue the job */
       sprintf(log_buf, "unable to run job, MOM rejected/rc=%d", status);
       
-      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,job_id,log_buf);
       
       free_nodes(pjob);
       
@@ -1279,7 +1330,9 @@ void finish_sendmom(
           pjob->ji_momstat = 0;
           
           /* update mom-based job status */
-          stat_mom_job(pjob);
+          pthread_mutex_unlock(pjob->ji_mutex);
+          stat_mom_job(job_id);
+          pjob = find_job(job_id);
           }
         else
           {
@@ -1298,6 +1351,7 @@ void finish_sendmom(
       break;
       }
     }  /* END switch (status) */
+  pthread_mutex_unlock(pjob->ji_mutex);
 
   } /* END finish_sendmom() */
 
@@ -1397,7 +1451,10 @@ static job *chk_job_torun(
     unlock_queue(pque, __func__, NULL, LOGLEVEL);
     }
   else if (pjob == NULL)
+    {
+    req_reject(PBSE_JOBNOTFOUND, 0, preq, NULL, "job vanished while trying to lock queue.");
     return(NULL);
+    }
 
   /* where to execute the job */
 #ifdef ENABLE_BLCR
@@ -1599,6 +1656,7 @@ char *get_correct_spec_string(
   job  *pjob)
 
   {
+  static char   id[] = "get_correct_spec_string";
   char     mode[20];
   char    *mode_string;
   char    *request;
@@ -1625,7 +1683,13 @@ char *get_correct_spec_string(
     
     if (pres != NULL)
       {
-      /* assign what was in "neednodes" */
+      /* determine # of gpu requests in spec, we found 1 in given up above */
+      num_gpu_reqs = 1;
+      gpu_req = mode_string;
+      while ((gpu_req = strstr(gpu_req + 1, ":gpus=")) != NULL)
+        num_gpu_reqs++;
+
+      /* assign gpu mode that was in "neednodes" */
       request = pres->rs_value.at_val.at_str;
       
       if ((request != NULL) && 
@@ -1639,6 +1703,20 @@ char *get_correct_spec_string(
 
         if (*mode_string == ':')
           {
+          if (LOGLEVEL >= 7)
+            {
+            sprintf(log_buffer, "%s: job has %d gpu requests in node spec '%s'",
+                    id,
+                    num_gpu_reqs,
+                    given);
+
+            log_event(
+              PBSEVENT_JOB,
+              PBS_EVENTCLASS_JOB,
+              pjob->ji_qs.ji_jobid,
+              log_buffer);
+            }
+
           if ((outer_plus = strchr(mode_string, '+')) != NULL)
             *outer_plus = '\0';
 
@@ -1646,12 +1724,7 @@ char *get_correct_spec_string(
 
           if (outer_plus != NULL)
             *outer_plus = '+';
-          
-          num_gpu_reqs = 1;
-          
-          while ((gpu_req = strstr(gpu_req + 1, ":gpus=")) != NULL)
-            num_gpu_reqs++;
-          
+
           /* 20 is a little more than the max length of gpu modes */
           len = strlen(given) + 1 + (num_gpu_reqs * 20);
           if ((correct_spec = calloc(1, len)) != NULL)
@@ -1671,13 +1744,25 @@ char *get_correct_spec_string(
               
               if (plus != NULL)
                 {
-                *plus = '+';
+                strcat(correct_spec, "+");
                 one_req = plus + 1;
                 }
               else
                 one_req = NULL;
               }
             }
+          if ((LOGLEVEL >= 7) && (correct_spec != NULL) && (correct_spec[0] != '\0'))
+            {
+            sprintf(log_buffer, "%s: job gets adjusted gpu node spec of '%s'",
+                    id,
+                    correct_spec);
+
+            log_event(
+              PBSEVENT_JOB,
+              PBS_EVENTCLASS_JOB,
+              pjob->ji_qs.ji_jobid,
+              log_buffer);
+            }
           }
         else
           correct_spec = strdup(given);
