@@ -83,9 +83,9 @@
 
 #include <pbs_config.h>   /* the master config generated by configure */
 
-#define STAT_CNTL 1
 
 #include <stdlib.h>
+#include <stdio.h>
 #include "libpbs.h"
 #include <string.h>
 #include <pthread.h>
@@ -106,6 +106,7 @@
 #include "queue_func.h" /* find_queuebyname */
 #include "reply_send.h" /* reply_send_svr */
 #include "svr_func.h" /* get_svr_attr_* */
+#include "req_stat.h" /* stat_mom_job */
 
 
 /* Private Data */
@@ -287,19 +288,17 @@ static attribute_def state_sel =
  * @see req_statjob() - peer - process qstat request
  */
 
-void *req_selectjobs(
-
-  void *vp)
-
+int req_selectjobs(
+    struct batch_request *preq)
   {
-  struct batch_request *preq = (struct batch_request *)vp;
   int                   bad = 0;
 
   struct stat_cntl     *cntl;
   svrattrl             *plist;
   pbs_queue            *pque = NULL;
-  int                   rc;
+  int                   rc = PBSE_NONE;
   long                  poll_jobs = 0;
+  char log_buf[LOCAL_LOG_BUF_SIZE+1];
 
   struct select_list   *selistp;
 
@@ -312,21 +311,18 @@ void *req_selectjobs(
     /* FAILURE */
 
     reply_badattr(rc, bad, plist, preq);
-
     free_sellist(selistp);
-
-    return(NULL);
+    return rc;
     }
 
   if ((cntl = (struct stat_cntl *)calloc(1, sizeof(struct stat_cntl))) == NULL)
     {
-    /* FAILURE */
-
+    rc = PBSE_MEM_MALLOC;
+    snprintf(log_buf, LOCAL_LOG_BUF_SIZE,
+        "Error allocating memory for stat_cntl");
     free_sellist(selistp);
-
-    req_reject(PBSE_SYSTEM, 0, preq, NULL, NULL);
-
-    return(NULL);
+    req_reject(rc, 0, preq, NULL, log_buf);
+    return rc;
     }
 
   if (pque != NULL)
@@ -364,7 +360,7 @@ void *req_selectjobs(
   if (pque != NULL)
     unlock_queue(pque, "req_selectjobs", NULL, LOGLEVEL);
 
-  return(NULL);
+  return PBSE_NONE;
   }  /* END req_selectjobs() */
 
 
@@ -380,7 +376,7 @@ static void sel_step2(
   struct stat_cntl *cntl)
 
   {
-  job          *pjob;
+  job          *pjob = NULL;
   int           rc;
   int           exec_only = 0;
   int           summarize_arrays = 0;
@@ -388,15 +384,13 @@ static void sel_step2(
   int           iter;
   time_t        time_now = time(NULL);
   long          query_others = 0;
+  char job_id[PBS_MAXSVRJOBID+1];
+  int job_substate = -1;
+  time_t job_momstattime = -1;
 
   /* do first pass of finding jobs that match the selection criteria */
   get_svr_attr_l(SRV_ATR_query_others, &query_others);
 
-  if (cntl->sc_jobid[0] == '\0')
-    pjob = NULL;
-  else
-    pjob = find_job(cntl->sc_jobid);
-
   if (cntl->sc_origrq->rq_extend != NULL)
     {
     if (!strncmp(cntl->sc_origrq->rq_extend, EXECQUEONLY, strlen(EXECQUEONLY)))
@@ -417,47 +411,17 @@ static void sel_step2(
 
     if (summarize_arrays)
       {
-      if (pjob == NULL)
-        {
-        if (cntl->sc_pque)
-          pjob = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
-        else
-          next_job(&array_summary,&iter);
-        }
-      else if (cntl->sc_pque)
-        {
-        pthread_mutex_unlock(pjob->ji_mutex);
-
+      if (cntl->sc_pque)
         pjob = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
-        }
       else
-        {
-        pthread_mutex_unlock(pjob->ji_mutex);
-
         pjob = next_job(&array_summary,&iter);;
-        }
       }
     else
       {
-      if (pjob == NULL)
-        {
-        if (cntl->sc_pque)
-          pjob = next_job(cntl->sc_pque->qu_jobs,&iter);
-        else
-          pjob = next_job(&alljobs,&iter);
-        }
-      else if (cntl->sc_pque)
-        {
-        pthread_mutex_unlock(pjob->ji_mutex);
-
+      if (cntl->sc_pque)
         pjob = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
-        }
       else
-        {
-        pthread_mutex_unlock(pjob->ji_mutex);
-
         pjob = next_job(&alljobs,&iter);
-        }
       }
 
     if (pjob == NULL)
@@ -492,33 +456,36 @@ static void sel_step2(
 
       if (select_job(pjob, cntl->sc_select))
         {
-        if ((pjob->ji_qs.ji_substate == JOB_SUBSTATE_RUNNING) &&
-            ((time_now - pjob->ji_momstat) > JobStatRate))
+        strcpy(cntl->sc_jobid, pjob->ji_qs.ji_jobid);
+        strcpy(job_id, pjob->ji_qs.ji_jobid);
+        job_substate = pjob->ji_qs.ji_substate;
+        job_momstattime = pjob->ji_momstat;
+        pthread_mutex_unlock(pjob->ji_mutex);
+
+        if ((job_substate == JOB_SUBSTATE_RUNNING) &&
+            ((time_now - job_momstattime) > JobStatRate))
           {
-          strcpy(cntl->sc_jobid, pjob->ji_qs.ji_jobid);
 
-          if ((rc = stat_to_mom(pjob, cntl)) == PBSE_SYSTEM)
+          if ((rc = stat_to_mom(job_id, cntl)) == PBSE_MEM_MALLOC)
             {
-            pthread_mutex_unlock(pjob->ji_mutex);
-
             break;
             }
 
           if (rc == 0)
             {
-            pthread_mutex_unlock(pjob->ji_mutex);
-
             return;
             }
 
-          rc = 0;
-
+          rc = PBSE_NONE;
           /* ignore this job */
-
           continue;
           }
         }
+      else
+        pthread_mutex_unlock(pjob->ji_mutex);
       }
+    else
+      pthread_mutex_unlock(pjob->ji_mutex);
     }
 
   sel_step3(cntl);
