@@ -81,17 +81,19 @@
  */
 #include <pbs_config.h>   /* the master config generated by configure */
 
-#include <sys/types.h>
 #include "libpbs.h"
 #include <errno.h>
-#include <fcntl.h>
 #include <signal.h>
 #include <string.h>
 #include <stdio.h>
+#include <pthread.h>
+#include <unistd.h>
 #include "server_limits.h"
 #include "list_link.h"
 #include "work_task.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "attribute.h"
 #include "server.h"
 #include "credential.h"
@@ -101,10 +103,13 @@
 #include "pbs_error.h"
 #include "svrfunc.h"
 #include "csv.h"
+#include "threadpool.h"
+#include "svr_func.h" /* get_svr_attr_* */
+
 
 /* Private Fuctions Local to this File */
 
-static int shutdown_checkpoint(job *);
+static int shutdown_checkpoint(job **);
 static void post_checkpoint(struct work_task *);
 static void rerun_or_kill(job *, char *text);
 
@@ -114,7 +119,6 @@ static struct batch_request *pshutdown_request = 0;
 
 /* Global Data Items: */
 
-extern tlist_head svr_alljobs;
 extern char *msg_abort_on_shutdown;
 extern char *msg_daemonname;
 extern char *msg_init_queued;
@@ -124,12 +128,12 @@ extern char *msg_leftrunning;
 extern char *msg_stillrunning;
 extern char *msg_on_shutdown;
 extern char *msg_job_abort;
-
-extern tlist_head task_list_event;
+extern int   lockfds;
 
 extern struct server server;
 extern attribute_def svr_attr_def[];
 extern int    LOGLEVEL;
+extern struct all_jobs alljobs;
 
 /*
  * svr_shutdown() - Perform (or start of) the shutdown of the server
@@ -140,32 +144,34 @@ void svr_shutdown(
   int type) /* I */
 
   {
-  attribute *pattr;
-  job     *pjob;
-  job     *pnxt;
-  long     *state;
+  attribute    *pattr;
+  job          *pjob;
+  long          state = SV_STATE_DOWN;
+  int           iter;
+  char          log_buf[LOCAL_LOG_BUF_SIZE];
+
+  close(lockfds);
 
   /* Lets start by logging shutdown and saving everything */
+  get_svr_attr_l(SRV_ATR_State, &state);
 
-  state = &server.sv_attr[SRV_ATR_State].at_val.at_long;
+  strcpy(log_buf, msg_shutdown_start);
 
-  strcpy(log_buffer, msg_shutdown_start);
-
-  if (*state == SV_STATE_SHUTIMM)
+  if (state == SV_STATE_SHUTIMM)
     {
     /* if already shuting down, another Immed/sig will force it */
-
     if ((type == SHUT_IMMEDIATE) || (type == SHUT_SIG))
       {
-      *state = SV_STATE_DOWN;
+      state = SV_STATE_DOWN;
+      set_svr_attr(SRV_ATR_State, &state);
 
-      strcat(log_buffer, "Forced");
+      strcat(log_buf, "Forced");
 
       log_event(
         PBSEVENT_SYSTEM | PBSEVENT_ADMIN | PBSEVENT_DEBUG,
         PBS_EVENTCLASS_SERVER,
         msg_daemonname,
-        log_buffer);
+        log_buf);
 
       return;
       }
@@ -173,35 +179,38 @@ void svr_shutdown(
 
   if (type == SHUT_IMMEDIATE)
     {
-    *state = SV_STATE_SHUTIMM;
+    state = SV_STATE_SHUTIMM;
+    set_svr_attr(SRV_ATR_State, &state);
 
-    strcat(log_buffer, "Immediate");
+    strcat(log_buf, "Immediate");
     }
   else if (type == SHUT_DELAY)
     {
-    *state = SV_STATE_SHUTDEL;
+    state = SV_STATE_SHUTDEL;
+    set_svr_attr(SRV_ATR_State, &state);
 
-    strcat(log_buffer, "Delayed");
+    strcat(log_buf, "Delayed");
     }
   else if (type == SHUT_QUICK)
     {
-    *state = SV_STATE_DOWN; /* set to down to brk pbsd_main loop */
+    state = SV_STATE_DOWN; /* set to down to brk pbsd_main loop */
+    set_svr_attr(SRV_ATR_State, &state);
 
-    strcat(log_buffer, "Quick");
+    strcat(log_buf, "Quick");
     }
   else
     {
-    *state = SV_STATE_SHUTIMM;
+    state = SV_STATE_SHUTIMM;
+    set_svr_attr(SRV_ATR_State, &state);
 
-    strcat(log_buffer, "By Signal");
+    strcat(log_buf, "By Signal");
     }
 
   log_event(
-
     PBSEVENT_SYSTEM | PBSEVENT_ADMIN | PBSEVENT_DEBUG,
     PBS_EVENTCLASS_SERVER,
     msg_daemonname,
-    log_buffer);
+    log_buf);
 
   if ((type == SHUT_QUICK) || (type == SHUT_SIG)) /* quick, leave jobs as are */
     {
@@ -210,12 +219,10 @@ void svr_shutdown(
 
   svr_save(&server, SVR_SAVE_QUICK);
 
-  pnxt = (job *)GET_NEXT(svr_alljobs);
+  iter = -1;
 
-  while ((pjob = pnxt) != NULL)
+  while ((pjob = next_job(&alljobs,&iter)) != NULL)
     {
-    pnxt = (job *)GET_NEXT(pjob->ji_alljobs);
-
     if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
       {
       pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HOTSTART | JOB_SVFLG_HASRUN;
@@ -229,8 +236,13 @@ void svr_shutdown(
         {
         /* do checkpoint of job */
 
-        if (shutdown_checkpoint(pjob) == 0)
+        if (shutdown_checkpoint(&pjob) == 0)
+          {
+          if (pjob != NULL)
+            pthread_mutex_unlock(pjob->ji_mutex);
+
           continue;
+          }
         }
 
       /* if no checkpoint (not supported, not allowed, or fails */
@@ -238,6 +250,8 @@ void svr_shutdown(
 
       rerun_or_kill(pjob, msg_on_shutdown);
       }
+
+    pthread_mutex_unlock(pjob->ji_mutex);
     }
 
   return;
@@ -254,8 +268,7 @@ void svr_shutdown(
  * before going away.
  */
 
-void
-shutdown_ack(void)
+void shutdown_ack(void)
 
   {
   if (pshutdown_request)
@@ -282,6 +295,8 @@ void req_shutdown(
   struct batch_request *preq)
 
   {
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
+
   if ((preq->rq_perm &
        (ATR_DFLAG_MGWR | ATR_DFLAG_MGRD | ATR_DFLAG_OPRD | ATR_DFLAG_OPWR)) == 0)
     {
@@ -290,16 +305,13 @@ void req_shutdown(
     return;
     }
 
-  sprintf(log_buffer, msg_shutdown_op,
-
-          preq->rq_user,
-          preq->rq_host);
+  sprintf(log_buf, msg_shutdown_op, preq->rq_user, preq->rq_host);
 
   log_event(
     PBSEVENT_SYSTEM | PBSEVENT_ADMIN | PBSEVENT_DEBUG,
     PBS_EVENTCLASS_SERVER,
     msg_daemonname,
-    log_buffer);
+    log_buf);
 
   pshutdown_request = preq;    /* save for reply from main() when done */
 
@@ -318,12 +330,12 @@ void req_shutdown(
 
 static int shutdown_checkpoint(
 
-  job *pjob)
+  job **pjob_ptr)
 
   {
-
+  job                  *pjob = *pjob_ptr;
   struct batch_request *phold;
-  attribute        temp;
+  attribute             temp;
 
   phold = alloc_br(PBS_BATCH_HoldJob);
 
@@ -348,34 +360,40 @@ static int shutdown_checkpoint(
         &phold->rq_ind.rq_hold.rq_orig.rq_attr,
         job_attr_def[JOB_ATR_hold].at_name,
         NULL,
-        ATR_ENCODE_CLIENT) < 0)
+        ATR_ENCODE_CLIENT,
+        0) < 0)
     {
+    free_br(phold);
     return(PBSE_SYSTEM);
     }
 
-  if (relay_to_mom(pjob, phold, post_checkpoint) != 0)
+  /* The phold is freed in relay_to_mom (failure)
+   * or in issue_Drequest (success) */
+  if (relay_to_mom(&pjob, phold, post_checkpoint) != 0)
     {
     /* FAILURE */
 
     return(-1);
     }
 
-  pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN;
-
-  pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_FILE;
-
-  if (LOGLEVEL >= 1)
+  if (pjob != NULL)
     {
-    log_event(
-      PBSEVENT_SYSTEM | PBSEVENT_JOB | PBSEVENT_DEBUG,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      "shutting down with active checkpointable job");
+    pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN;
+    pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_FILE;
+    
+    if (LOGLEVEL >= 1)
+      {
+      log_event(
+        PBSEVENT_SYSTEM | PBSEVENT_JOB | PBSEVENT_DEBUG,
+        PBS_EVENTCLASS_JOB,
+        pjob->ji_qs.ji_jobid,
+        "shutting down with active checkpointable job");
+      }
+  
+    job_save(pjob, SAVEJOB_QUICK, 0);
     }
 
-  job_save(pjob, SAVEJOB_QUICK, 0);
-
-  return(0);
+  return(PBSE_NONE);
   }  /* END shutdown_checkpoint() */
 
 
@@ -427,6 +445,9 @@ static void post_checkpoint(
     }
 
   release_req(ptask);
+
+  if (pjob != NULL)
+    pthread_mutex_unlock(pjob->ji_mutex);
   }  /* END post_checkpoint() */
 
 
@@ -441,26 +462,32 @@ static void rerun_or_kill(
   char *text)  /* I */
 
   {
-  long server_state = server.sv_attr[SRV_ATR_State].at_val.at_long;
+  long       server_state = SV_STATE_DOWN;
+  char       log_buf[LOCAL_LOG_BUF_SIZE];
+  pbs_queue *pque;
 
+  get_svr_attr_l(SRV_ATR_State, &server_state);
   if (pjob->ji_wattr[JOB_ATR_rerunable].at_val.at_long)
     {
     /* job is rerunable, mark it to be requeued */
 
-    issue_signal(pjob, "SIGKILL", release_req, 0);
+    issue_signal(&pjob, "SIGKILL", release_req, 0);
 
-    pjob->ji_qs.ji_substate  = JOB_SUBSTATE_RERUN;
+    if (pjob != NULL)
+      {
+      pjob->ji_qs.ji_substate  = JOB_SUBSTATE_RERUN;
+      if ((pque = get_jobs_queue(&pjob)) != NULL)
+        {
+        snprintf(log_buf, sizeof(log_buf), "%s%s%s", msg_init_queued, pque->qu_qs.qu_name, text);
 
-    strcpy(log_buffer, msg_init_queued);
-    strcat(log_buffer, pjob->ji_qhdr->qu_qs.qu_name);
-    strcat(log_buffer, text);
+        unlock_queue(pque, __func__, NULL, LOGLEVEL);
+        }
+      }
     }
   else if (server_state != SV_STATE_SHUTDEL)
     {
     /* job not rerunable, immediate shutdown - kill it off */
-
-    strcpy(log_buffer, msg_job_abort);
-    strcat(log_buffer, text);
+    snprintf(log_buf, sizeof(log_buf), "%s%s", msg_job_abort, text);
 
     /* need to record log message before purging job */
 
@@ -468,24 +495,27 @@ static void rerun_or_kill(
       PBSEVENT_SYSTEM | PBSEVENT_JOB | PBSEVENT_DEBUG,
       PBS_EVENTCLASS_JOB,
       pjob->ji_qs.ji_jobid,
-      log_buffer);
+      log_buf);
 
-    job_abt(&pjob, log_buffer);
+    job_abt(&pjob, log_buf);
 
     return;
     }
   else
     {
     /* delayed shutdown, leave job running */
-
-    strcpy(log_buffer, msg_leftrunning);
-    strcat(log_buffer, text);
+    snprintf(log_buf, sizeof(log_buf), "%s%s", msg_leftrunning, text);
     }
 
-  log_event(PBSEVENT_SYSTEM | PBSEVENT_JOB | PBSEVENT_DEBUG,
-
-            PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid,
-            log_buffer);
+  if (pjob != NULL)
+    {
+    log_event(
+      PBSEVENT_SYSTEM | PBSEVENT_JOB | PBSEVENT_DEBUG,
+      PBS_EVENTCLASS_JOB,
+      pjob->ji_qs.ji_jobid,
+      log_buf);
+    }
 
   return;
   }  /* END rerun_or_kill() */
+
