@@ -83,9 +83,9 @@
  *  Status Server Batch Requests.
  */
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "req_stat.h"
 #include <pthread.h>
 
-#define STAT_CNTL 1
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -114,10 +114,11 @@
 #include "node_func.h" /* find_nodebyname */
 #include "issue_request.h" /* issue_Drequest */
 #include "../lib/Libutils/u_lock_ctl.h" /* lock_node, unlock_node */
-#include "svr_connect.h" /* svr_connect */
+#include "svr_connect.h" /* svr_connect, svr_disconnect_sock */
 #include "queue_func.h" /* find_queuebyname */
 #include "reply_send.h" /* reply_send_svr */
 #include "svr_func.h" /* get_svr_attr_* */
+#include "node_manager.h" /* tfind_addr */
 
 
 /* Global Data Items: */
@@ -134,8 +135,6 @@ extern attribute_def   node_attr_def[];   /* node attributes defs */
 extern int        pbs_mom_port;
 extern char       *msg_init_norerun;
 
-extern struct pbsnode *tfind_addr(const u_long, uint16_t, job *);
-
 extern int             LOGLEVEL;
 
 extern pthread_mutex_t *netrates_mutex;
@@ -144,7 +143,6 @@ extern pthread_mutex_t *netrates_mutex;
 
 int status_job(job *, struct batch_request *, svrattrl *, tlist_head *, int *);
 int status_attrib(svrattrl *, attribute_def *, attribute *, int, int, tlist_head *, int *, int);
-extern int  svr_connect(pbs_net_t, unsigned int, int *, struct pbsnode *, void *(*)(void *), enum conn_type);
 extern int  status_nodeattrib(svrattrl *, attribute_def *, struct pbsnode *, int, int, tlist_head *, int*);
 extern int  hasprop(struct pbsnode *, struct prop *);
 extern void rel_resc(job*);
@@ -155,7 +153,6 @@ static void update_state_ct(attribute *, int *, char *);
 static int  status_que(pbs_queue *, struct batch_request *, tlist_head *);
 static int  status_node(struct pbsnode *, struct batch_request *, int *, tlist_head *);
 static void req_stat_job_step2(struct stat_cntl *);
-static void stat_update(struct work_task *);
 
 #ifndef TMAX_JOB
 #define TMAX_JOB 999999999
@@ -192,20 +189,18 @@ enum TJobStatTypeEnum
   tjstLAST
   };
 
-void *req_stat_job(
+int req_stat_job(
 
-  void *vp)  /* ptr to the decoded request */
+  struct batch_request *preq)  /* ptr to the decoded request */
 
   {
   struct stat_cntl     *cntl; /* see svrfunc.h  */
   char                 *name;
   job                  *pjob = NULL;
   pbs_queue            *pque = NULL;
-  int                   rc = 0;
+  int                   rc = PBSE_NONE;
   long                  poll_jobs = 0;
 
-  struct batch_request *preq = (struct batch_request *)vp;
-
   enum TJobStatTypeEnum type = tjstNONE;
 
   /*
@@ -292,8 +287,10 @@ void *req_stat_job(
     /* is invalid - an error */
     if (pque != NULL) 
       unlock_queue(pque, "req_stat_job", "invalid", LOGLEVEL);
+
     req_reject(rc, 0, preq, NULL, NULL);
-    return(NULL);
+
+    return(rc);
     }
 
   preq->rq_reply.brp_choice = BATCH_REPLY_CHOICE_Status;
@@ -308,7 +305,7 @@ void *req_stat_job(
       unlock_queue(pque, "req_stat_job", "no memory cntl", LOGLEVEL);
     req_reject(PBSE_SYSTEM, 0, preq, NULL, NULL);
 
-    return(NULL);
+    return(PBSE_SYSTEM);
     }
 
   if ((type == tjstTruncatedQueue) ||
@@ -338,7 +335,7 @@ void *req_stat_job(
     unlock_queue(pque, "req_stat_job", "success", LOGLEVEL);
 
   free(cntl);
-  return(NULL);
+  return(PBSE_NONE);
   }  /* END req_stat_job() */
 
 
@@ -368,7 +365,6 @@ static void req_stat_job_step2(
   struct stat_cntl *cntl)  /* I/O (free'd on return) */
 
   {
-  char                  id[] = "req_stat_job_step2";
   svrattrl              *pal;
   job                   *pjob = NULL;
 
@@ -378,17 +374,18 @@ static void req_stat_job_step2(
   enum TJobStatTypeEnum  type;
   pbs_queue             *pque = NULL;
   int                    exec_only = 0;
-
   int                    bad = 0;
   long                   DTime;  /* delta time - only report full attribute list if J->MTime > DTime */
   static svrattrl       *dpal = NULL;
   int                    job_array_index = 0;
   job_array             *pa = NULL;
-  char                   log_buf[LOCAL_LOG_BUF_SIZE];
+  char                   log_buf[LOCAL_LOG_BUF_SIZE+1];
   int                    iter;
   time_t                 time_now = time(NULL);
   long                   poll_jobs = 0;
-  
+  char job_id[PBS_MAXSVRJOBID+1];
+  int job_substate = -1;
+  time_t job_momstattime = -1;
 
   preq   = cntl->sc_origrq;
   type   = (enum TJobStatTypeEnum)cntl->sc_type;
@@ -478,6 +475,7 @@ static void req_stat_job_step2(
               {
               if ((pjob = find_job(pa->job_ids[job_array_index])) != NULL)
                 {
+                pthread_mutex_unlock(pjob->ji_mutex);
                 break;
                 }
               }
@@ -493,7 +491,7 @@ static void req_stat_job_step2(
         }    /* END if (pjob == NULL) */
       else
         {
-        /* get next job */
+        strcpy(job_id, pjob->ji_qs.ji_jobid);
         pthread_mutex_unlock(pjob->ji_mutex);
 
         if (type == tjstJob)
@@ -511,6 +509,7 @@ static void req_stat_job_step2(
               {
               if ((pjob = find_job(pa->job_ids[job_array_index])) != NULL)
                 {
+                pthread_mutex_unlock(pjob->ji_mutex);
                 break;
                 }
               }
@@ -518,42 +517,41 @@ static void req_stat_job_step2(
           }
         else
           pjob = next_job(&alljobs,&iter);
-          
         }
 
       if (pjob == NULL)
         break;
 
-      /* PBS_RESTAT_JOB defaults to 30 seconds */
+      strcpy(job_id, pjob->ji_qs.ji_jobid);
+      job_substate = pjob->ji_qs.ji_substate;
+      job_momstattime = pjob->ji_momstat;
+      strcpy(cntl->sc_jobid, job_id);
+      pthread_mutex_unlock(pjob->ji_mutex);
+      pjob = NULL;
 
-      if ((pjob->ji_qs.ji_substate == JOB_SUBSTATE_RUNNING) &&
-          ((time_now - pjob->ji_momstat) > JobStatRate))
+      /* PBS_RESTAT_JOB defaults to 30 seconds */
+      if ((job_substate == JOB_SUBSTATE_RUNNING) &&
+          ((time_now - job_momstattime) > JobStatRate))
         {
         /* go to MOM for status */
 
-        strcpy(cntl->sc_jobid, pjob->ji_qs.ji_jobid);
 
-        if ((rc = stat_to_mom(pjob, cntl)) == PBSE_SYSTEM)
-          {
-          pthread_mutex_unlock(pjob->ji_mutex);
-          
+        if ((rc = stat_to_mom(job_id, cntl)) == PBSE_MEM_MALLOC)
           break;
-          }
 
         if (rc != 0)
           {
+          pjob = find_job(job_id);
           rc = 0;
-
           continue;
           }
         
         if (LOGLEVEL >= 7)
           {
-          sprintf(log_buf, "%s: unlocked ai_mutex", id);
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+          sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
           }
 
-        pthread_mutex_unlock(pjob->ji_mutex);
         if (pa != NULL)
           pthread_mutex_unlock(pa->ai_mutex);
 
@@ -561,17 +559,14 @@ static void req_stat_job_step2(
         }
       }    /* END while(1) */
 
-    if (cntl->sc_conn >= 0)
-      svr_disconnect(cntl->sc_conn);  /* close connection to MOM */
-
     if (rc != 0)
       {
       if (pa != NULL)
         pthread_mutex_unlock(pa->ai_mutex);
         if(LOGLEVEL >= 7)
           {
-          sprintf(log_buf, "%s: unlocked ai_mutex", id);
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+          sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
           }
 
       reply_free(preply);
@@ -722,7 +717,7 @@ static void req_stat_job_step2(
             pthread_mutex_unlock(pa->ai_mutex);
             if(LOGLEVEL >= 7)
               {
-              sprintf(log_buf, "%s: unlocked ai_mutex", id);
+              sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
               log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
               }
             }
@@ -753,7 +748,7 @@ static void req_stat_job_step2(
       
     if (LOGLEVEL >= 7)
       {
-      sprintf(log_buf, "%s: unlocked ai_mutex", id);
+      sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
       log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pa->ai_qs.parent_id, log_buf);
       }
 
@@ -811,7 +806,7 @@ static void req_stat_job_step2(
         pthread_mutex_unlock(pa->ai_mutex);
         if (LOGLEVEL >= 7)
           {
-          sprintf(log_buf, "%s: unlocked ai_mutex", id);
+          sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
           log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
           }
         }
@@ -862,7 +857,7 @@ nextjob:
     {
     if (LOGLEVEL >= 7)
       {
-      sprintf(log_buf, "%s: unlocked ai_mutex", id);
+      sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
       log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pa->ai_qs.parent_id, log_buf);
       }
     pthread_mutex_unlock(pa->ai_mutex);
@@ -898,85 +893,93 @@ nextjob:
 
 int stat_to_mom(
 
-  job              *pjob,  /* I */
+  char             *job_id,
   struct stat_cntl *cntl)  /* M */
 
   {
   struct batch_request *newrq;
-  int                   rc = 0;
+  int                   rc = PBSE_NONE;
   unsigned long         addr;
-  char                  log_buf[LOCAL_LOG_BUF_SIZE];
-
-  struct work_task     *pwt = 0;
-
+  char                  log_buf[LOCAL_LOG_BUF_SIZE+1];
   struct pbsnode       *node;
+  int handle = -1;
+  unsigned long job_momaddr = -1;
+  unsigned short job_momport = -1;
+  char *job_momname = NULL;
+  job *pjob = NULL;
+
+  if ((pjob = find_job(job_id)) == NULL)
+    return PBSE_JOBNOTFOUND;
+
+  job_momaddr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
+  job_momport = pjob->ji_qs.ji_un.ji_exect.ji_momport;
+  job_momname = strdup(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
+  pthread_mutex_unlock(pjob->ji_mutex);
+
+  if (job_momname == NULL)
+    return PBSE_MEM_MALLOC;
 
   if ((newrq = alloc_br(PBS_BATCH_StatusJob)) == NULL)
     {
-    return(PBSE_SYSTEM);
+    free(job_momname);
+    return PBSE_MEM_MALLOC;
     }
 
-  /* set up status request, save address of cntl in request for later */
-  newrq->rq_extra = (void *)cntl;
-
   if (cntl->sc_type == 1)
-    strcpy(newrq->rq_ind.rq_status.rq_id, pjob->ji_qs.ji_jobid);
+    strcpy(newrq->rq_ind.rq_status.rq_id, job_id);
   else
     newrq->rq_ind.rq_status.rq_id[0] = '\0';  /* get stat of all */
 
   CLEAR_HEAD(newrq->rq_ind.rq_status.rq_attr);
 
   /* if MOM is down just return stale information */
-  addr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
+  addr = job_momaddr;
 
-  node = tfind_addr(addr,pjob->ji_qs.ji_un.ji_exect.ji_momport,pjob);
+  node = tfind_addr(addr,job_momport,job_momname);
+  free(job_momname);
 
-  if ((node  != NULL) &&
-      (node->nd_state & INUSE_DOWN))
+  if (node == NULL)
+    return PBSE_UNKNODE;
+  if (node->nd_state & INUSE_DOWN)
     {
     if (LOGLEVEL >= 6)
       {
-      sprintf(log_buf,"node '%s' is allocated to job but in state 'down'",
-        node->nd_name);
+      snprintf(log_buf, LOCAL_LOG_BUF_SIZE,
+          "node '%s' is allocated to job but in state 'down'",
+          node->nd_name);
 
-      log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+      log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_JOB,job_id,log_buf);
       }
 
     unlock_node(node, __func__, "no rely mom", LOGLEVEL);
     free_br(newrq);
 
-    return(PBSE_NORELYMOM);
+    return PBSE_NORELYMOM;
     }
 
   /* get connection to MOM */
-  unlock_node(node, __func__, "after svr_connect", LOGLEVEL);
-  cntl->sc_conn = svr_connect(
-                    pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
-                    pjob->ji_qs.ji_un.ji_exect.ji_momport,
-                    &rc,
-                    node,
-                    process_Dreply,
-                    ToServerDIS);
+  unlock_node(node, __func__, "before svr_connect", LOGLEVEL);
+  handle = svr_connect(job_momaddr, job_momport, &rc, NULL, NULL, ToServerDIS);
 
-
-  if ((rc = cntl->sc_conn) >= 0)
-    rc = issue_Drequest(cntl->sc_conn, newrq, stat_update, &pwt);
-
-  if (rc != 0)
+  /* Unlock job here */
+  if (handle >= 0)
     {
-    /* request failed */
-    if (pwt)
-      delete_task(pwt);
+    if ((rc = issue_Drequest(handle, newrq, NULL, NULL)) != PBSE_NONE)
+      {
+      }
+    else
+      {
+      stat_update(newrq, cntl);
+      }
+    }
+  else
+    rc = PBSE_CONNECT;
+  if (rc == PBSE_SYSTEM)
+    rc = PBSE_MEM_MALLOC;
 
-    /* There are cases where this is free'd inside issue_Drequest in
-     * failure cases */
-/*    free_br(newrq); */
+  free_br(newrq);
 
-    if (cntl->sc_conn >= 0)
-      svr_disconnect(cntl->sc_conn);
-    }  /* END if (rc != NULL) */
-
-  return(rc);
+  return rc;
   }  /* END stat_to_mom() */
 
 
@@ -987,27 +990,19 @@ int stat_to_mom(
  * stat_update - take reply to status request from MOM and update job status
  */
 
-static void stat_update(
-
-  struct work_task *pwt)
-
+void stat_update(
+    struct batch_request *preq,
+    struct stat_cntl *cntl)
   {
-  struct stat_cntl     *cntl;
   job                  *pjob;
-
-  struct batch_request *preq;
-
   struct batch_reply   *preply;
-
   struct brp_status    *pstatus;
   svrattrl             *sattrl;
   int                   oldsid;
   int                   bad = 0;
   time_t                time_now = time(NULL);
 
-  preq = pwt->wt_parm1;
   preply = &preq->rq_reply;
-  cntl = preq->rq_extra;
 
   if (preply->brp_choice == BATCH_REPLY_CHOICE_Status)
     {
@@ -1015,7 +1010,7 @@ static void stat_update(
 
     while (pstatus != NULL)
       {
-      if ((pjob = find_job(pstatus->brp_objname)))
+      if ((pjob = find_job(pstatus->brp_objname)) != NULL)
         {
         sattrl = (svrattrl *)GET_NEXT(pstatus->brp_attr);
 
@@ -1067,7 +1062,7 @@ static void stat_update(
       {
       /* we sent a stat request, but mom says it doesn't know anything about
          the job */
-      if ((pjob = find_job(preq->rq_ind.rq_status.rq_id)))
+      if ((pjob = find_job(preq->rq_ind.rq_status.rq_id)) != NULL)
         {
         /* job really isn't running any more - mom doesn't know anything about it
            this can happen if a diskless node reboots and the mom_priv/jobs
@@ -1082,11 +1077,9 @@ static void stat_update(
         }
       }
     }
-
-  release_req(pwt);
-
   cntl->sc_conn = -1;
 
+  /* MUTSU - Unlock job here? */
   if (cntl->sc_post)
     cntl->sc_post(cntl); /* continue where we left off */
 
@@ -1109,11 +1102,8 @@ static void stat_update(
  */
 
 void stat_mom_job(
-
-  job *pjob)
-
+    char *job_id)
   {
-
   struct stat_cntl *cntl;
 
   cntl = (struct stat_cntl *)calloc(1, sizeof(struct stat_cntl));
@@ -1131,7 +1121,7 @@ void stat_mom_job(
   cntl->sc_post   = 0;  /* tell stat_update() to free cntl */
   cntl->sc_jobid[0] = '\0'; /* cause "start from beginning" */
 
-  if (stat_to_mom(pjob, cntl) != 0)
+  if (stat_to_mom(job_id, cntl) != 0)
     {
     free(cntl);
     }
@@ -1155,31 +1145,31 @@ void poll_job_task(
   struct work_task *ptask)
 
   {
-  char      *jobid = (char *)ptask->wt_parm1;
+  char      *job_id = (char *)ptask->wt_parm1;
   job       *pjob;
   time_t     time_now = time(NULL);
   long       poll_jobs = 0;
+  int job_state = -1;
 
-  if (jobid != NULL)
+  if (job_id != NULL)
     {
-    pjob  = find_job(jobid);
+    pjob  = find_job(job_id);
     
     if (pjob != NULL)
       {
+      job_state = pjob->ji_qs.ji_state;
+      pthread_mutex_unlock(pjob->ji_mutex);
       get_svr_attr_l(SRV_ATR_PollJobs, &poll_jobs);
-      if ((poll_jobs) &&
-          (pjob->ji_qs.ji_state == JOB_STATE_RUNNING))
+      if ((poll_jobs) && (job_state == JOB_STATE_RUNNING))
         {
-        stat_mom_job(pjob);
+        stat_mom_job(job_id);
         
         /* add another task */
-        set_task(WORK_Timed, time_now + JobStatRate, poll_job_task, strdup(jobid), FALSE);
+        set_task(WORK_Timed, time_now + JobStatRate, poll_job_task, strdup(job_id), FALSE);
         }
-      
-      pthread_mutex_unlock(pjob->ji_mutex);
       }
       
-    free(jobid);
+    free(job_id);
     }
 
   free(ptask->wt_mutex);
@@ -1196,18 +1186,16 @@ void poll_job_task(
  * the set of queues at a destinaion.
  */
 
-void *req_stat_que(
-
-  void *vp) /* ptr to the decoded request   */
-
+int req_stat_que(
+    struct batch_request *preq)
   {
   char                 *name;
   pbs_queue            *pque = NULL;
 
   struct batch_reply   *preply;
-  struct batch_request *preq = (struct batch_request *)vp;
   int                   rc   = 0;
   int                   type = 0;
+  char log_buf[LOCAL_LOG_BUF_SIZE+1];
 
   /*
    * first, validate the name of the requested object, either
@@ -1226,9 +1214,10 @@ void *req_stat_que(
 
     if (pque == NULL)
       {
-      req_reject(PBSE_UNKQUE, 0, preq, NULL, "cannot locate queue");
-
-      return NULL;
+      rc = PBSE_UNKQUE;
+      snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "can not locate queue %s", name);
+      req_reject(rc, 0, preq, NULL, log_buf);
+      return rc;
       }
     }
 
@@ -1281,7 +1270,7 @@ void *req_stat_que(
     reply_send_svr(preq);
     }
 
-  return NULL;
+  return rc;
   }  /* END req_stat_que() */
 
 
@@ -1421,21 +1410,15 @@ int get_numa_statuses(
  * set of nodes at a destination.
  */
 
-void *req_stat_node(
-
-  void *vp) /* ptr to the decoded request   */
-
+int req_stat_node(
+    struct batch_request *preq)
   {
-  static char          *id = "req_stat_node";
+  int                   rc = PBSE_NONE;
   char                 *name;
-
-  int                   rc   = 0;
   int                   type = 0;
   int                   bad  = 0;
-
   struct pbsnode       *pnode = NULL;
   struct batch_reply   *preply;
-  struct batch_request *preq = (struct batch_request *)vp;
   struct prop props;
   svrattrl             *pal;
 
@@ -1448,14 +1431,15 @@ void *req_stat_node(
 
   if (LOGLEVEL >= 6)
     {
-    log_record( PBSEVENT_SCHED, PBS_EVENTCLASS_REQUEST, id, "entered");
+    log_record( PBSEVENT_SCHED, PBS_EVENTCLASS_REQUEST, __func__, "entered");
     }
 
   if (svr_totnodes <= 0)
     {
-    req_reject(PBSE_NONODES, 0, preq, NULL, "node list is empty - check 'server_priv/nodes' file");
+    rc = PBSE_NONODES;
+    req_reject(rc, 0, preq, NULL, "node list is empty - check 'server_priv/nodes' file");
 
-    return(NULL);
+    return rc;
     }
 
   name = preq->rq_ind.rq_status.rq_id;
@@ -1491,8 +1475,9 @@ void *req_stat_node(
     pnode = find_nodebyname(name);
     if (pnode == NULL)
       {
-      req_reject(PBSE_UNKNODE, 0, preq, NULL, "cannot locate specified node");
-      return NULL;
+      rc = PBSE_UNKNODE;
+      req_reject(rc, 0, preq, NULL, "cannot locate specified node");
+      return rc;
       }
 
     /* get the status on all of the numa nodes */
@@ -1545,7 +1530,7 @@ void *req_stat_node(
       }
     }
 
-  return(NULL);
+  return rc;
   }  /* END req_stat_node() */
 
 
@@ -1628,15 +1613,14 @@ static int status_node(
  * This request processes the request for status of the Server
  */
 
-void *req_stat_svr(
+int req_stat_svr(
 
-  void *vp) /* ptr to the decoded request */
+  struct batch_request *preq) /* ptr to the decoded request */
 
   {
   svrattrl             *pal;
 
   struct batch_reply   *preply;
-  struct batch_request *preq = (struct batch_request *)vp;
 
   struct brp_status    *pstat;
   int                  *nc;
@@ -1663,12 +1647,15 @@ void *req_stat_svr(
   pthread_mutex_unlock(server.sv_jobstates_mutex);
 
   nc = netcounter_get(); /* locks netrates mutex */
-  sprintf(nc_buf, "%d %d %d", *nc, *(nc + 1), *(nc + 2));
+  snprintf(nc_buf, 127, "%d %d %d", *nc, *(nc + 1), *(nc + 2));
   
   pthread_mutex_unlock(netrates_mutex);
 
-  server.sv_attr[SRV_ATR_NetCounter].at_val.at_str = nc_buf;
-  server.sv_attr[SRV_ATR_NetCounter].at_flags |= ATR_VFLAG_SET;
+  if (server.sv_attr[SRV_ATR_NetCounter].at_val.at_str != NULL)
+    free(server.sv_attr[SRV_ATR_NetCounter].at_val.at_str);
+  server.sv_attr[SRV_ATR_NetCounter].at_val.at_str = strdup(nc_buf);
+  if (server.sv_attr[SRV_ATR_NetCounter].at_val.at_str != NULL)
+    server.sv_attr[SRV_ATR_NetCounter].at_flags |= ATR_VFLAG_SET;
   pthread_mutex_unlock(server.sv_attr_mutex);
 
   /* allocate a reply structure and a status sub-structure */
@@ -1687,7 +1674,7 @@ void *req_stat_svr(
     req_reject(PBSE_SYSTEM, 0, preq, NULL, NULL);
     pthread_mutex_unlock(server.sv_attr_mutex);
 
-    return(NULL);
+    return(PBSE_SYSTEM);
     }
 
   CLEAR_LINK(pstat->brp_stlink);
@@ -1722,7 +1709,7 @@ void *req_stat_svr(
     }
     
 
-  return(NULL);
+  return(PBSE_NONE);
   }  /* END req_stat_svr() */
 
 /* DIAGTODO: write req_stat_diag() */
