@@ -94,6 +94,7 @@
 #include "pbs_error.h"
 #include "../lib/Libdis/lib_dis.h" /* DIS_tcp_setup */
 #include "pbsd_main.h" /* process_pbs_server_port */
+#include "process_request.h" /*process_request */
 
 /* Global Data */
 
@@ -195,13 +196,13 @@ static int contact_sched(
     pbs_scheduler_addr,
     pbs_scheduler_port,
     PBS_SOCK_INET,
-    process_pbs_server_port);
+    process_pbs_server_port_scheduler);
 
   pthread_mutex_lock(svr_conn[sock].cn_mutex);
-
   svr_conn[sock].cn_authen = PBS_NET_CONN_FROM_PRIVIL;
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
-  net_add_close_func(sock, scheduler_close, TRUE);
+  net_add_close_func(sock, scheduler_close);
 
   /* send command to Scheduler */
 
@@ -213,17 +214,11 @@ static int contact_sched(
 
     log_ext(errno,id,tmpLine,LOG_ALERT);
 
-    close_conn(sock, TRUE);
-
-    pthread_mutex_unlock(svr_conn[sock].cn_mutex);
+    close_conn(sock, FALSE);
 
     return(-1);
     }
     
-  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
-
-  DIS_tcp_setup(sock);
-
   sprintf(log_buf, msg_sched_called, (cmd != SCH_ERROR) ? PSchedCmdType[cmd] : "ERROR");
 
   log_event(PBSEVENT_SCHED,PBS_EVENTCLASS_SERVER,server_name,log_buf);
@@ -250,6 +245,7 @@ int schedule_jobs(void)
   int cmd;
 
   static int first_time = 1;
+  int tmp_sched_sock = -1;
 
   pthread_mutex_lock(svr_do_schedule_mutex);
 
@@ -264,32 +260,42 @@ int schedule_jobs(void)
   pthread_mutex_unlock(svr_do_schedule_mutex);
 
   pthread_mutex_lock(scheduler_sock_jobct_mutex);
-
   if (scheduler_sock == -1)
-    {
     scheduler_jobct = 0;
+  else
+    tmp_sched_sock = scheduler_sock;
+  pthread_mutex_unlock(scheduler_sock_jobct_mutex);
 
-    if ((scheduler_sock = contact_sched(cmd)) < 0)
+  if (tmp_sched_sock != -1)
+    {
+    if ((tmp_sched_sock = contact_sched(cmd)) < 0)
       {
-      pthread_mutex_unlock(scheduler_sock_jobct_mutex);
       return(-1);
       }
-      
+    pthread_mutex_lock(scheduler_sock_jobct_mutex);
+    scheduler_sock = tmp_sched_sock;
     pthread_mutex_unlock(scheduler_sock_jobct_mutex);
 
     first_time = 0;
 
     return(0);
     }
-  else
-    {
-    pthread_mutex_unlock(scheduler_sock_jobct_mutex);
-    }
-
   return(1);
   }  /* END schedule_jobs() */
 
 
+
+void *start_process_request(void *vp)
+  {
+  int sock = *(int *)vp;
+  struct tcp_chan *chan = NULL;
+  if ((chan = DIS_tcp_setup(sock)) == NULL)
+    return NULL;
+  process_request(chan);
+  DIS_tcp_cleanup(chan);
+  return(NULL);
+  }
+
 /*
  * contact_listener - open connection to the listener and send it a command
  */
@@ -347,14 +353,13 @@ static int contact_listener(
     listener_conns[l_idx].address,
     listener_conns[l_idx].port,
     PBS_SOCK_INET,
-    process_request);
+    start_process_request);
 
   pthread_mutex_lock(svr_conn[sock].cn_mutex);
-
   svr_conn[sock].cn_authen = PBS_NET_CONN_FROM_PRIVIL;
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
-
-  net_add_close_func(sock, listener_close, TRUE);
+  net_add_close_func(sock, listener_close);
 
   /* send command to Listener */
   pthread_mutex_lock(listener_command_mutex);
@@ -370,17 +375,14 @@ static int contact_listener(
 
     log_err(errno, id, tmpLine);
 
-    close_conn(sock, TRUE);
+    close_conn(sock, FALSE);
   
-    pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
     return(-1);
     }
 
   pthread_mutex_unlock(listener_command_mutex);
 
-  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
-
   sprintf(log_buf, msg_listnr_called, l_idx + 1,
     (listener_command != SCH_ERROR) ? PSchedCmdType[listener_command] : "ERROR");
 
