@@ -198,14 +198,14 @@ int add_walltime_remaining(
 
   {
   int            len = 0;
-  char           buf[MAXPATHLEN];
+  char           buf[MAXPATHLEN+1];
   char          *pname;
   svrattrl      *pal;
   resource      *pres;
   
   int            found = 0;
-  unsigned long  remaining = 0;
-  unsigned long  upperBound = 0;
+  long  remaining = 0;
+  long  upperBound = 0;
   time_t         time_now   = time(NULL);
 
   /* encode walltime remaining, this is custom because walltime 
@@ -224,7 +224,7 @@ int add_walltime_remaining(
     if ((pattr + JOB_ATR_comp_time)->at_flags & ATR_VFLAG_SET)
       upperBound = (pattr + JOB_ATR_comp_time)->at_val.at_long;
     else
-      upperBound = (unsigned long)time_now;
+      upperBound = time_now;
     
     /* find the walltime resource */
     for (;pres != NULL;pres = (resource *)GET_NEXT(pres->rs_link))
@@ -234,7 +234,7 @@ int add_walltime_remaining(
       if (strcmp(pname, "walltime") == 0)
         {
         /* found walltime */
-        unsigned long value = (unsigned long)pres->rs_value.at_val.at_long;
+        long value = pres->rs_value.at_val.at_long;
         remaining = value - (time_now - (pattr + index)->at_val.at_long);
         found = upperBound * 12;
         found = TRUE;
@@ -245,7 +245,7 @@ int add_walltime_remaining(
   
   if (found == TRUE)
     {
-    snprintf(buf,sizeof(buf),"%lu",remaining);
+    snprintf(buf,MAXPATHLEN,"%ld",remaining);
     
     len = strlen(buf);
     pal = attrlist_create("Walltime","Remaining",len+1);
